/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.mega.moods.generator.LanguageTranslator.Language;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class CustomTypesGenerator {

    private static Vector classLanguages = new Vector();
    private static Vector classFiles = new Vector();
    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    public static boolean isMulticastCommunicationModelMethodsToAdd = false;
    public static boolean isUnicastCommunicationModelMethodsToAdd = false;
    private static final String PACKAGE_UGASP_CUSTOMTYPES = "org.ugasp.games.demonstrator";
    
    
    // t: strings array to write
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: constructor
    // t[5]: encodeData
    // t[6]: decodeData
    // t[7]: encodingMethods
    // t[8]: decodingMethods
    // t[9]: string encoding utils methods
    // t[10]: communication methods (multicast)
    // t[11]: communication methods (unicast)
    // t[12]: end of class
  
    public static void init(){
    	classFiles = new Vector();
    	classLanguages = new Vector();
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    	
    }
    
    
    //TODO generer le serveur pour les hastables au lieu des objets
    public static void startGenerateCustomTypes(Language language, boolean generateUGaspLogic, int encoding, String _package) throws IOException{
    	System.out.println("CustomTypesGenerator: start to generate custom types for language: "+language.toString());
    	String[] strings = new String[]{"","","","","","","","","","","","",""};
    	String path = Generator.getPath(language,false);
    	// Affect the name to the generated file
    	String className = getCurrentClassName(language);
    	String linkClass = path+className;
    	ClassMaker.createClassEntry(language,linkClass,className,strings,false);

	    classNames.add(linkClass);
	    classLanguages.add(language);
	    
    	// strings to write table for that class

    	// init strings table
    	strings[0] = "/** \n"
			+" * MooDS Encoder/Decoder.\n"
			+" * Encoding version: "+Generator.ENCODING_VERSION+"\n"
	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	 		+" */\n\n";
    	
    	// package declaration
    	if(language.equals(Language.JAVA) ){
    		strings[0] += writeSuitablePackageForJavaCustomTypes(language, _package);
    	}
    	// required imports
        if (language.equals(Language.FLASH)){
        	strings[0] += LanguageTranslator.writePackage(language,_package+".client.flash")+"\n";
        	strings[1] += LanguageTranslator.writeImport(language, "Hashtable.as")+"\n\n";	
        }
        if (language.equals(Language.JAVA) ){
        	if(!Generator.isGenerationForUGaspLogic()){
        		// If generation is made fot uGASP, we don't need to import other classes
        		// as every needed classes would be in the same package.
        		strings[1] += LanguageTranslator.writeImport(language,  _package+".*")+"\n";
        	}
        	
        	strings[1] += LanguageTranslator.writeImport(language,"org.ugasp.protocol.moods.CustomTypes")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(language,"java.util.Hashtable")+"\n"; 
	    	if (encoding == Generator.BINARY){
	    		strings[1] += LanguageTranslator.writeImport(language, "java.io.DataOutputStream")+"\n";
	    		strings[1] += LanguageTranslator.writeImport(language, "java.io.DataInputStream")+"\n";	
        	}
	    	strings[1] += LanguageTranslator.writeImport(language,"java.io.IOException")+"\n"; 
        }
        if(language.equals(Language.CPP) ){
        	strings[1] += LanguageTranslator.writeMacro(className)+"\n";
        	strings[1] += LanguageTranslator.writeImport(language, "map")+"\n";
        	strings[1] += LanguageTranslator.writeImport(language, "Object.h")+"\n";
        	strings[1] += LanguageTranslator.writeImport(language, "SocketStream.h")+"\n";
        	
        }
        if(language.equals(Language.CSHARP) ){
	    	strings[1] += LanguageTranslator.writeImport(language,"System")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(language,"System.IO")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(language,"System.Net")+"\n"; 	
	    	strings[1] += LanguageTranslator.writeImport(language,"System.Collections")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(language,"MiscUtil.IO")+"\n"; 
        }
    	// class declaration
        strings[2] += LanguageTranslator.writeComments(language, "This class has been generated by MooDS Generator to handle "+Generator.ioApproach+"\n// for binarisation input/output.")+"\n";
        
        if (language.equals(Language.CSHARP) )
        	strings[2] +=  LanguageTranslator.writePackage(language,_package)+"\n";
        
    	if (Generator.isClient){
    		if (encoding == Generator.BINARY) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypesImpl",true,false,"org.ugasp.protocol.moods.CustomTypes")+"\n";
	    	if (encoding == Generator.STRING) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypes4StringEncodingImpl",true,false,"org.ugasp.protocol.moods.string.CustomTypes")+"\n"; 
    	}
		else{ //server
			if(generateUGaspLogic){
				if (encoding == Generator.BINARY) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypesImpl",true,false,"org.ugasp.protocol.moods.CustomTypes")+"\n";
		    	if (encoding == Generator.STRING) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypes4StringEncodingImpl",true,false,"org.ugasp.protocol.moods.string.CustomTypes")+"\n"; 
			}else{
				if (encoding == Generator.BINARY) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypesImpl",true,false,"org.ugasp.protocol.moods.CustomTypes")+"\n";
		    	if (encoding == Generator.STRING) strings[2] += LanguageTranslator.writeClassDeclaration(language,"CustomTypes4StringEncoding",true,false,"org.ugasp.protocol.moods.string.CustomTypes")+"\n"; 
			}
		    	 	
		}
     	
    	// constructor just for uGasp
    	/*if (generateUGaspLogic && !Generator.isClient ){*/
    		if(encoding == Generator.BINARY) strings[4] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,"CustomTypesImpl",null);
    		if(encoding == Generator.STRING) strings[4] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,"CustomTypes4StringEncodingImpl",null);
    	/*}*/
    	
    	// start to generate encodeData root method
     	strings[5] += startToWriteEncodeDataMethod(language, encoding);
    	
     	// start to generate decodeData root method
     	strings[6] += startToWriteDecodeDataMethod(language, encoding);
    	
    	
    	if (encoding == Generator.STRING) strings[9] += addStringEncodingSpecificMethods(language);
    	strings[12] += LanguageTranslator.closeBlock(language);
    	if (language.equals(Language.CSHARP) )
    		strings[12] += LanguageTranslator.closeBlock(language);
    	
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("CustomTypesGenerator: start to generate class named "+ path+className);
    }
    
    /*
     * Get the appropriate name for the generated CustomTypes file :
     * - "CustomTypes" for clients in every language
     * - "CustomTypes" or "CustomTypes4StringEncoding" for Gasp server, depending on encoding
     * - "CustomTypesImpl" or "CustomTypes4StringEncodingImpl" for uGasp server, depending on encoding
     */
    private static String getCurrentClassName(Language language){
    	String className = "";
    	if(Generator.isClient){
    		className = "CustomTypesImpl";
    	}else{
    		if(Generator.encoding == Generator.BINARY){
    			className = (Generator.isGenerationForUGaspLogic())?"CustomTypesImpl":"CustomTypesImpl";
    		}else if(Generator.encoding == Generator.STRING){
      			className = (Generator.isGenerationForUGaspLogic())?"CustomTypes4StringEncodingImpl":"CustomTypes4StringEncodingImpl";
    		}
    	}
    	return className;
    }
    
    private static String getCurrentClassLink(Language language){
    	String className = getCurrentClassName(language);
    	String currentClass = Generator.getPath(language,false)+className;
    	return currentClass;
    }
    
    public static String startToWriteEncodeDataMethod(Language language, int encoding) {	
    	String encodeData = "";
    	if (encoding == Generator.STRING){
       		encodeData += "\t"+LanguageTranslator.addGlobalVar(language,true,false,Generator.isClient,true,"","String","encoding")+"\n\n"; 
       		//encodeData += "\t private static var encoding:String = \"\";\n\n";
       		
       		String[][] params = {{"Hashtable","h"}};
	       	if (Generator.isClient) encodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,false,false,false,"String","encodeData",params,false,null)+"\n";
	       	else encodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,false,false,false,"String","encodeData",params,false,null)+"\n";
	       	//encodeData += "\t public static function encodeData(h:Hashtable):String {\n";
        
	       	// Add a version number to be compliant with B4J binary encoding/decoding
       		String[] vars = {"\"\"+"+Generator.ENCODING_VERSION};
       		String value = LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",vars);
       		encodeData += "\t\t"+LanguageTranslator.affectValueToVar(language,"encoding",value) +"\n";
       		//encodeData += "\t\t encoding = addEncodingSpacer(\""+ENCODING_VERSION+"\");\n"; 
       		
       	  	String[] vars1 = {"\"\"+"+LanguageTranslator.callMethod(language,true,"h","size",null)};
	       	String[] itemsToAdd1 = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",vars1)};
	       	encodeData += "\t\t"+LanguageTranslator.add(language,true,"String","encoding",itemsToAdd1)+"\n";  
	       	//encodeData += "\t\t encoding += addEncodingSpacer(h.getSize());\n";
       	}
       	else if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CSHARP) )){  		
       		// start to build root method of the Encoder : encodeData
       		String[][] params = null;
       		if (language.equals(Language.JAVA)  || language.equals(Language.FLASH) )
       			params = new String[][]{{"Hashtable","h"},{"DataOutputStream","dos"}};
       		if (language.equals(Language.CSHARP) )
       			params = new String[][]{{"Hashtable","h"},{"EndianBinaryWriter","dos"}};
       		
       		String[] exceptions = {"IOException"};
       		if (Generator.isClient) encodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,"","encodeData",params,true,exceptions)+"\n";
	       	else encodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,"","encodeData",params,true,exceptions)+"\n";
	      
       		//encodeData += "\t public void encodeData(Hashtable h, DataOutputStream dos) throws Exception {\n";

            // Add a version number to be compliant with B4J binary encoding/decoding
            encodeData += "\t\t"+LanguageTranslator.writeBinaryType(language,"short",""+Generator.ENCODING_VERSION,"dos")+"\n";
       		//encodeData += "\t\t dos.writeShort("+ENCODING_VERSION+");\n";
           
            if (language.equals(Language.JAVA) )
            	encodeData += "\t\t"+LanguageTranslator.writeBinaryType(
            							language,
            							"short",
            							LanguageTranslator.callMethod(language,true,"h","size",null),
            							"dos")+"\n";	
            if (language.equals(Language.CSHARP) )
            	encodeData += "\t\t"+LanguageTranslator.writeBinaryType(
            							language,
            							"short",
            							"(short) h.Count",
            							"dos")+"\n";	
            //encodeData += "\t\t dos.writeShort(h.size());\n";
       	} else if (encoding == Generator.BINARY && language.equals(Language.CPP) ){  
       		String[][] params = {{"map<int,Object*>&","h"},{"SocketStream&","ss"}};
       		encodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,"","encodeData",params,false,null)+"\n";

            // Add a version number to be compliant with B4J binary encoding/decoding
            encodeData += "\t\t"+LanguageTranslator.writeBinaryType(language,"short",""+Generator.ENCODING_VERSION,"ss")+"\n";

            encodeData += "\t\t"+LanguageTranslator.writeBinaryType(
            							language,
            							"short",
            							LanguageTranslator.callMethod(language,true,"h","size",null),
            							"ss")+"\n";				
       	}
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CSHARP) ){
    		
    		if (language.equals(Language.JAVA) )
	    		encodeData += "\t\t"+LanguageTranslator.addLocalVar(
	    				language,
	    				true,
	    				LanguageTranslator.callMethod(language,true,"h","size",null)
	    				,"int"
	    				,"nbSend"
	    		)+"\n";
    		if (language.equals(Language.CSHARP) )
	    		encodeData += "\t\t"+LanguageTranslator.addLocalVar(
	    				language,
	    				true,
	    				"h.Count"
	    				,"int"
	    				,"nbSend"
	    		)+"\n";
    		
    		//encodeData += "\t\t var nbSend = h.getSize();\n";

    		encodeData += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0","nbSend")+"\n";
    		//encodeData += "\t\t for(var i=0; i<nbSend; i++){\n";

    		if (Generator.objectsApproachMode){
    			String[] vars2 = {"\"\"+i"};
    			if (language.equals(Language.JAVA) )
    				encodeData += "\t\t\t"+LanguageTranslator.addLocalVar(
    					language,
    					true,
    					//TODO Avant language, FLASH, pourquoi??
    					LanguageTranslator.callMethod(language,true,"h","get",vars2)
    					,"Object"
    					,"o"
    			)+"\n";
    			//encodeData += "\t\t\t var o  = h.get(\"\"+i);\n";
    			
    			if (language.equals(Language.CSHARP) )
    				encodeData += "\t\t\t"+LanguageTranslator.addLocalVar(
    					language,
    					true,
    					//TODO Avant language, FLASH, pourquoi??
    					"h["+vars2[0]+"]"//LanguageTranslator.callMethod(language,true,"h","get",vars2)
    					,"Object"
    					,"o"
    			)+"\n";
    			//encodeData += "\t\t\t var o  = h.get(\"\"+i);\n";
    		}
    		if (Generator.hashtablesApproachMode){
    			String[] vars2 = {"\"\"+i"};
    			if (language.equals(Language.JAVA) )
    			encodeData += "\t\t\t"+LanguageTranslator.addLocalVar(
    					language,
    					true,
    					LanguageTranslator.cast(language,"Hashtable",LanguageTranslator.callMethod(language,true,"h","get",vars2))
    					,"Hashtable"
    					,"hs"
    			)+"\n";
    			
    			if (language.equals(Language.CSHARP) )
        			encodeData += "\t\t\t"+LanguageTranslator.addLocalVar(
        					language,
        					true,
        					LanguageTranslator.cast(language,"Hashtable","h["+vars2[0]+"]")
        					,"Hashtable"
        					,"hs"
        			)+"\n";
    			
    			//encodeData += "\t\t\t var hs:Hashtable  = h.get(\"\"+i);\n";
    		}

    		if (Generator.objectsApproachMode && Generator.isCompactMode){
    			String init1 = LanguageTranslator.callMethod(language,true,LanguageTranslator.cast(language,Generator.destinationClassName,"o"),"getType",null);
    			encodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language, true, init1,"byte","type")+"\n";
    		}

    	}    
    	// this method is the c++ way to get an object from a map within it's type, and use the generic encode() method on it
    	if(language.equals(Language.CPP) ){
    		encodeData +=  "\t\t"+"for(int i=0; i <h.size(); i++){\n"
    					+"\t\t\t"+"// get a pointer to the object at the key i in the map\n"
    					+"\t\t\t"+"Object* o = ((Object*)(h.find(i)->second));\n"
    					+"\t\t\t"+"o->encodeType(ss);\n"
    					+"\t\t\t"+"o->encode(ss);\n";
    	}
    	return encodeData;
    }

    
    public static void continueToWriteEncodeDataMethod(Language language, int encoding, String typeNameFinal, String typeName, String typeClassName) {	
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
    		 
        	if (Generator.objectsApproachMode){
             	if (Generator.isCompactMode){        
                 	String[] itemsToEquals = {"type", typeNameFinal};
                 	strings[5] += "\t\t\t"+LanguageTranslator.ifDeclaration(language, LanguageTranslator.testEqualType(language,true,"int",itemsToEquals))+"\n";
                     //encodeData += "\t\t\t if ((("+destClassName+") o).getType() == type){\n";   
                 }
                 else{
                	 strings[5] += "\t\t\t"+LanguageTranslator.ifDeclaration(language,
    	                						LanguageTranslator.testInstanceOfType(language,"o", typeClassName)
    	                						)+"\n";
    	                //encodeData += "\t\t\t if (o instanceof "+typeClassName+"){\n";
                 }
             }
        	
             if (Generator.hashtablesApproachMode){
            	 System.out.println("HASHTABLE: encode data test");
                 String[] params3 = {"\"type\""};
                 String[] items21 = null;
                 if (language.equals(Language.JAVA) )
                	 	items21 = new String[]{
                		 			LanguageTranslator.callMethod(language,true,"hs","get",params3),
                 					LanguageTranslator.newType(language, "Byte", new String[]{typeNameFinal})
                 					};
                 
                 if (language.equals(Language.CSHARP) )
             	 	items21 = new String[]{
              					"h["+params3[0]+"]",
              					LanguageTranslator.newType(language, "Byte", new String[]{typeNameFinal})
              					};
                 
                 String cond1 = LanguageTranslator.testEqualType(language, true, typeClassName,items21);
                 System.out.println("cond1:"+cond1+" equals:"+LanguageTranslator.newType(language, "Byte", new String[]{typeNameFinal}));
                 
                 strings[5] += "\t\t\t"+LanguageTranslator.ifDeclaration(language,cond1)+"\n";
                 //encodeData += "\t\t\t if (hs.get(\"type\") === \""+typeClassName+"\"){\n";
             }
                 	
             if (encoding == Generator.STRING){
    	            String[] vars11 = {"\"\"+"+ typeNameFinal};//(isCompactMode? typeNameFinal:(""+indexType))};//"\"\"+"+LanguageTranslator.callMethod(language,true,"h","size",null)};
    	     	    String[] itemsToAdd11 = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",vars11)};
    	     	    strings[5] += "\t\t\t\t"+LanguageTranslator.add(language,true,"String","encoding",itemsToAdd11)+"\n";
    	            //encodeData += "\t\t\t\t encoding += addEncodingSpacer(\""+indexType+"\");\n"; 
             } else if (encoding == Generator.BINARY){
            	  	strings[5] += "\t\t\t\t"+LanguageTranslator.writeBinaryType(language,"byte",typeNameFinal,"dos")+"\n";
    	            //encodeData += "\t\t\t\t dos.writeByte("+indexType+");\n";
             }
                 	
      	    if (Generator.objectsApproachMode){
      	    	String type1 = typeClassName;
    	        	if (Generator.isCompactMode) type1 = Generator.destinationClassName;
      	        
    	        	if (encoding == Generator.STRING){
      	        	String[] params21 = {LanguageTranslator.cast(language,type1,"o")};
      	        	strings[5] += "\t\t\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+typeName,params21)+LanguageTranslator.endOfLineMarker(language)+"\n";
      	        	//encodeData += "\t\t\t\t encode"+typeName+"("+typeClassName+"(o));\n";
      	     	}
      	        else if (encoding == Generator.BINARY){
      	        	String[] params21 = {"dos",LanguageTranslator.cast(language,""+type1,"o")};
      	        	strings[5] += "\t\t\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+typeName,params21)+LanguageTranslator.endOfLineMarker(language)+"\n";
    	     	      	//encodeData += "\t\t\t\t encode"+typeName+"(dos,("+typeClassName+") o);\n";
      	       	}
      	   	}
    	    if (Generator.hashtablesApproachMode){
    	           	if (encoding == Generator.STRING){
    		         	String[] params21 = {"hs"};
    		         	strings[5] += "\t\t\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+typeName,params21)+LanguageTranslator.endOfLineMarker(language)+"\n";
    		         	//encodeData += "\t\t\t\t encode"+typeName+"(hs);\n";
    	          	}
    	           	else if (encoding == Generator.BINARY){
    	             	String[] params21 = {"dos","hs"};
    	             	strings[5] += "\t\t\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+typeName,params21)+LanguageTranslator.endOfLineMarker(language)+"\n";
    		           	//encodeData += "\t\t\t\t encodeData += "\t\t\t\t encode"+typeName+"(dos, hs);\n";
    	       		}
    	        }
    	                
    	    strings[5] += "\t\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	    //encodeData += "\t\t\t }\n";

            //strings[5] = encodeData;
            //stringsToWrite.put(currentCustomTypes,strings);

    	}
    	if(language.equals(Language.CPP) ){
    		//To write the include of the proper classes
        	strings[1] += LanguageTranslator.writeImport(language,(typeClassName+".h"))+"\n";     
    	}  
    }
    
    
    public static String startToWriteDecodeDataMethod(Language language, int encoding) {	
    	String decodeData = "";
    	String init = "";
       	String[] vars3 = {"encodedString"};
       	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CSHARP) ){
           	if (encoding == Generator.STRING){
    	       	// start to build root method of the Decoder : decodeData
    	       	decodeData += "\t"+LanguageTranslator.addGlobalVar(language,
    	            			false,false,Generator.isClient,true,"0","int","indexTreatment")+"\n";
    	       	//decodeData += "\t private static var indexTreatment = 0;\n\n";
    	            
    	       	String[][] params2 = {{"String","encodedString"}};
    	        if (Generator.isClient)decodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,
    	            						true,false,false,false,"Hashtable","decodeData",params2,false,null)+"\n";  
    	        else decodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,
               						true,false,false,false,"Hashtable","decodeData",params2,false,null)+"\n"; 
    	       	//decodeData += "\t public static function decodeData(encodedString:String):Hashtable {\n";
    	            
    	       	// read the version number to be compliant with B4J binary encoding/decoding
    	       	decodeData += "\t\t"+LanguageTranslator.affectValueToVar(language,"indexTreatment","0")+"\n";
    	       	//decodeData += "\t\t indexTreatment = 0;\n";
    	       	
    	       	decodeData += "\t\t"+LanguageTranslator.callMethod(language,false,"","getNextParam",vars3)+LanguageTranslator.endOfLineMarker(Language.FLASH)+"\n";  
    	       	//decodeData += "\t\t var encoding_version = getNextParam(encodedString); // ENCODING_VERSION\n";  
    	        decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,true,"","Hashtable","h")+"\n";
    	    	//decodeData += "\t\t var h:Hashtable = new Hashtable();\n";
           	
    	    	init = LanguageTranslator.callMethod(language,false,"","getNextParam",vars3);
    	       	decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,
    	            				true,
    	            				LanguageTranslator.parseStringType(language,"short",init)
    	            				,"short"
    	            				,"nb")+"\n"; 
    	       	//decodeData += "\t\t var nb = parseInt(getNextParam(encodedString));\n";          
           	} else if (encoding == Generator.BINARY){
           		//start to build root method of the Decoder : decodeData
    	       	String[][] params2 = null;
    	       	
    	       	if (language.equals(Language.JAVA) ) params2 = new String[][] {{"DataInputStream","dis"}};
    	       	if (language.equals(Language.CSHARP) ) params2 = new String[][] {{"EndianBinaryReader","dis"}};
    	       	
    	       	String[] exceptions = {"IOException"};
    	        if (Generator.isClient)decodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,
    	            						true,false,false,false,"Hashtable","decodeData",params2,true,exceptions)+"\n"; 
    	        else decodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,
               						true,false,false,false,"Hashtable","decodeData",params2,true,exceptions)+"\n";  
           		//decodeData += "\t public Hashtable decodeData(DataInputStream dis) throws Exception {\n";
                
                // read the version number to be compliant with B4J binary encoding/decoding
                decodeData += "\t\t"+LanguageTranslator.readBinaryType(language,"short","dis")+LanguageTranslator.endOfLineMarker(language)+"\n";
    	        //decodeData += "\t\t dis.readShort(); // ENCODING_VERSION\n";  
                decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,true,"","Hashtable","h")+"\n";
    	    	//decodeData += "\t\t var h:Hashtable = new Hashtable();\n";
    	       	decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,
    	            				true,
    	            				LanguageTranslator.readBinaryType(language,"short","dis")
    	            				,"short"
    	            				,"nb")+"\n"; 
                //decodeData += "\t\t short nb = dis.readShort();\n";          
           	}
    	   
           
            decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,true,"nb","int","nbObj")+"\n";
           	//decodeData += "\t\t var nbObj = nb;\n";
           	String[] items1 = {"nb","0"};
           	decodeData += "\t\t"+LanguageTranslator.whileLoopDeclaration(language,
                					LanguageTranslator.testEqualType(language,false,"int",items1)
                					) +"\n";
           	//decodeData += "\t\t while(nb!=0){\n";
           	String[] items2 = {"nbObj","nb"};
           	decodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language,
                				true,
                				"\"\"+"+LanguageTranslator.addValueToValue(language,false,items2),
                				"string",
                				"key")+"\n";
           	//decodeData += "\t\t\t var key:String = \"\"+(nbObj - nb);\n"; 
           	
           	if (encoding == Generator.STRING){
    	       	decodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language,
    	            				true,
    	            				LanguageTranslator.parseStringType(language,"byte",init)
    	            				,"byte"
    	            				,"type")+"\n"; 
    	       	//decodeData += "\t\t\t var type = parseInt(getNextParam(encodedString));\n";  
           	}else if (encoding == Generator.BINARY){
           		decodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language,
        				true,
        				LanguageTranslator.readBinaryType(language,"byte","dis")
        				,"byte"
        				,"type")+"\n";  
           		//decodeData += "\t\t\t byte type = dis.readByte();\n";
           	}
       	}
       	if(language.equals(Language.CPP) ){
       		String[][] params2 = {{"map<int,Object*>&", "h"},{"SocketStream&","ss"}};
       		decodeData += "\t"+LanguageTranslator.writeFunctionDeclaration(language,
					true,true,false,true,"","decodeData",params2,false,null)+"\n"; 
       		// read the version number to be compliant with B4J binary encoding/decoding
            decodeData += "\t\t"+LanguageTranslator.readBinaryType(language,"short","ss")+LanguageTranslator.endOfLineMarker(language)+"\n";
            //decodeData += "\t\t short nb = dis.readShort();\n";  
            decodeData += "\t\t"+LanguageTranslator.addLocalVar(language,
    				true,
    				LanguageTranslator.readBinaryType(language,"short","ss")
    				,"short"
    				,"nb")+"\n"; 
            // decodeData += "\t\t int nbObject = nb;
            decodeData += "\t\t"+LanguageTranslator.addLocalVar(Language.CPP, true, "nb", "int", "nbObject")+"\n";
            // decodeData += "\t\t Object* = o;
            decodeData += "\t\t"+LanguageTranslator.addLocalVar(Language.CPP, false, "", "Object*", "o")+"\n";
            //decodeData += "\t\t while(nb!=0){\n";
        	String[] items1 = {"nb","0"};
            decodeData += "\t\t"+LanguageTranslator.whileLoopDeclaration(language,
					LanguageTranslator.testEqualType(language,false,"int",items1)
					) +"\n";
            //decodeData += "\t\t\t int key = (nbObj - nb);\n";  
        	String[] items2 = {"nbObject","nb"};
            decodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language,
    				true,
    				LanguageTranslator.addValueToValue(language,false,items2),
    				"int",
    				"key")+"\n";
       		//decodeData += "\t\t\t unsigned char type = dis.readByte();\n";
            decodeData += "\t\t\t"+LanguageTranslator.addLocalVar(language,
    				true,
    				LanguageTranslator.readBinaryType(language,"byte","ss")
    				,"char"
    				,"type")+"\n";  
          //decodeData += "\t\t\t switch(type){
            decodeData += "\t\t\t"+LanguageTranslator.switchDeclaration(Language.CPP, "type")+"\n";
       	}
       	return decodeData;
    }
    
    
    public static void continueToWriteDecodeDataMethod(Language language, int encoding, int indexType, boolean isFirstIf, String typeNameFinal, String typeName, String typeClassName) {	
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)   || language.equals(Language.CSHARP) ){
        	//String decodeData = strings[6];
        	
        	String[] itemsToEquals = {"type",(Generator.isCompactMode? typeNameFinal:(""+indexType))};
        	String cond2 = LanguageTranslator.testEqualType(language,true,"int",itemsToEquals);
            if (isFirstIf) {isFirstIf = false; strings[6] += "\t\t\t"+LanguageTranslator.ifDeclaration(language,cond2)+"\n";}
            else strings[6] += "\t\t\t"+ LanguageTranslator.elseIfDeclaration(language,cond2)+"\n";
            //if (isFirstIf) {isFirstIf = false; decodeData+="\t\t\t ";}
            //else decodeData+="\t\t\t else ";
            //decodeData += "if (type ===" +" "+indexType+"){\n";
                   
            if (Generator.objectsApproachMode){	
            	String type1 = typeClassName;
    	        	if (Generator.isCompactMode) type1 = Generator.destinationClassName;
    	        	
            	if (encoding == Generator.STRING){
                 	String[] vars31 = {"encodedString"};
                 	strings[6] += "\t\t\t\t"+LanguageTranslator.addLocalVar(language,
    	     	            				true,
    	     	            				LanguageTranslator.callMethod(language,false,"","decode"+typeName,vars31)
    	     	            				,""+type1
    	     	            				,"customType")+"\n"; 
    	         	//decodeData += "\t\t\t\t var customType:"+typeClassName+" = decode"+typeName+"(encodedString);\n";
    	                
             	} else if (encoding == Generator.BINARY){
    	          	String[] vars31 = {"dis"};
    	          	strings[6] += "\t\t\t\t"+LanguageTranslator.addLocalVar(language,
     	            				true,
     	            				LanguageTranslator.callMethod(language,false,"","decode"+typeName,vars31)
     	            				,""+type1
     	            				,"customType")+"\n"; 
    	                	//decodeData += "\t\t\t\t "+typeClassName+" customType = decode"+typeName+"(dis);\n";
    	        }
            	
            	String[] params22 = {"key","customType"};
            	if (language.equals(Language.JAVA) )
            		strings[6] += "\t\t\t\t"+LanguageTranslator.callMethod(language,true,"h","put",params22)+LanguageTranslator.endOfLineMarker(language)+"\n";
         	  	if (language.equals(Language.CSHARP) )
         	  		strings[6] += "\t\t\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",params22)+LanguageTranslator.endOfLineMarker(language)+"\n";
            	//decodeData += "\t\t\t\t h.put(key,customType);\n";
            }
        	if (Generator.hashtablesApproachMode){
                  	if (encoding == Generator.STRING){
                    	String[] vars31 = {"encodedString"};
                    	strings[6] += "\t\t\t\t"+LanguageTranslator.addLocalVar(language,
    	     	            				true,
    	     	            				LanguageTranslator.callMethod(language,false,"","decode"+typeName,vars31)
    	     	            				,"Hashtable"
    	     	            				,"hs")+"\n"; 
    	             	//decodeData += "\t\t\t\t var hs:Hashtable = decode"+typeName+"(encodedString);\n";  
                   	} else if (encoding == Generator.BINARY){
                    	String[] vars31 = {"dis"};
                    	strings[6] += "\t\t\t\t"+LanguageTranslator.addLocalVar(language,
         	            				true,
         	            				LanguageTranslator.callMethod(language,false,"","decode"+typeName,vars31)
         	            				,"Hashtable"
         	            				,"hs")+"\n"; 
                    	//decodeData += "\t\t\t\t Hashtable hs = decode"+typeName+"(dis);\n";
                   	}
                    	
                 	String[] params23 = {"key","hs"};
                 	if (language.equals(Language.JAVA) )
                 		strings[6] += "\t\t\t\t"+LanguageTranslator.callMethod(language,true,"h","put",params23)+LanguageTranslator.endOfLineMarker(language)+"\n";
                 	if (language.equals(Language.CSHARP) )
                 		strings[6] += "\t\t\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",params23)+LanguageTranslator.endOfLineMarker(language)+"\n";
                 	
                 	//decodeData += "\t\t\t\t h.put(key,hs);\n";
        	}
                    
        	strings[6] += "\t\t\t"+LanguageTranslator.closeBlock(language)+"\n";
        	//decodeData += "\t\t\t }\n";
            
        	//strings[6] = decodeData;
            //stringsToWrite.put(currentCustomTypes,strings);
        	
    	}
    	if(language.equals(Language.CPP) ){
    		//strings[6] += case typeClassName::type : 
    		strings[6] += "\t\t\t\t"+LanguageTranslator.caseDeclaration(Language.CPP, typeClassName+"::TYPE")+"\n";
    		//strings[6] += o = typeClassName::decode(this) :
    		strings[6] += "\t\t\t\t\t"+LanguageTranslator.affectValueToVar(Language.CPP, "o", LanguageTranslator.newType(Language.CPP, typeClassName, ""))+"\n";
    		//strings[6] += "\t\t\t\t\t"+LanguageTranslator.affectValueToVar(CPP, "o", typeClassName+"::decode(ss)")+"\n";
    		strings[6] += "\t\t\t\t\t"+"break;"+"\n\n";
    	}
    	
    }
    
    
    /*****************
    appel: 	obj> unicastRecipientEncoding(o.recipient);
    		hasht> unicastRecipientEncoding(h.get("recipient"));
    */
    	
    /*****************
    appel: 	obj> short recipient = unicastRecipientDecoding(dis);
    			 o = new Type(recipient)
    		hasht> h.put("recipient",unicastRecipientDecoding(dis));
    */
    	
    /*****************
    appel: 	obj> multicastRecipientsEncoding(dos,o.recipients);
    		hasht> h.put("recipients",multicastRecipientsDecoding(dos));
    */
    	
    /*****************
     appel: obj> short[] recipients = multicastRecipientsDecoding(dis);
     			 Type o = new Type(recipients);
    		hasht> Hashtable h = new Hashtable();
    			   h.put("recipients",multicastRecipientsDecoding(dis));
     */
    
    public static void addRecipients4MulticastComModelEncoding(Language language,int encoding){
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if (encoding == Generator.BINARY){
	    	if (Generator.objectsApproachMode && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",new String[]{"dos",LanguageTranslator.callMethod(language,true,"customType","getRecipients",null)})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode){
	    		if (language.equals(Language.JAVA) )
	    			strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",
	    					new String[]{"dos",LanguageTranslator.cast(language, "short[]",
	    							LanguageTranslator.callMethod(language,true,"h","get",new String[]{"\"recipients\""}))})
	    							+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		if (language.equals(Language.CSHARP) )
	    			strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",
	    					new String[]{"dos",LanguageTranslator.cast(language, "short[]",
	    							"h[\"recipients\"]")})
	    							+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	}
    	}
    	else if (encoding == Generator.STRING){
    		if (Generator.objectsApproachMode) strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",new String[]{LanguageTranslator.callMethod(language,true,"customType","getRecipients",null)})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode) {
	    		if (language.equals(Language.JAVA) )
	    		strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",
	    				new String[]{  LanguageTranslator.cast(language, "short[]",
	    						LanguageTranslator.callMethod(language,true,"h","get",new String[]{"\"recipients\""}))})
	    						+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		
	    		if (language.equals(Language.CSHARP) )
		    		strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","multicastRecipientsEncoding",
		    				new String[]{  LanguageTranslator.cast(language, "short[]",
		    						"h[\"recipients\"]")})
		    						+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	}
    	}
    }
    
    
    public static void addRecipients4MulticastComModelDecoding(Language language,int encoding){
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if (encoding == Generator.BINARY){
    		// TODO en C++
	    	if (Generator.objectsApproachMode&& (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) strings[8] += "\t\t"+LanguageTranslator.addLocalVar(language,true,LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"dis"}),LanguageTranslator.arrayDeclaration(language,"short",1),"recipients")+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode){
	    		if (language.equals(Language.JAVA) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",new String[]{"\"recipients\"", LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"dis"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		if (language.equals(Language.CSHARP) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",new String[]{"\"recipients\"", LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"dis"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		
	    	}
    	}
    	else if (encoding == Generator.STRING){
    		if (Generator.objectsApproachMode) strings[8] += "\t\t"+LanguageTranslator.addLocalVar(language,true,LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"encodedString"}),LanguageTranslator.arrayDeclaration(language,"short",1),"recipients")+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode){
	    		if (language.equals(Language.JAVA) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",new String[]{"\"recipients\"",LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"encodedString"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		if (language.equals(Language.JAVA) )
		    		strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",new String[]{"\"recipients\"",LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"encodedString"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
		
	    	}
    	}
    	
    	if(!language.equals(Language.CPP)) strings[10] = writeMulticastCommunicationModelMethods(language,encoding);
    }
    
    
    
    public static void addRecipient4UnicastComModelEncoding(Language language,int encoding){
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);

    	if (encoding == Generator.BINARY){
		    if (Generator.objectsApproachMode&& (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",new String[]{"dos",LanguageTranslator.callMethod(language,true,"customType","getRecipient",null)})+LanguageTranslator.endOfLineMarker(language)+"\n";
		    if (Generator.hashtablesApproachMode) 
		    		
		    	if (language.equals(Language.JAVA) )
		    	strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",
		    				new String[]{"dos",LanguageTranslator.getObjectTypeValue(
		    							language,
		    							"short", LanguageTranslator.cast(language, "short", LanguageTranslator.callMethod(language,true,"h","get",new String[]{"\"recipient\""})))})+LanguageTranslator.endOfLineMarker(language)+"\n";
		    	if (language.equals(Language.CSHARP) )
		    	strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",
		    				new String[]{"dos",LanguageTranslator.getObjectTypeValue(
		    							language,
		    							"short", LanguageTranslator.cast(language, "short", 
		    									"h[\"recipient\"]"))})
		    									+LanguageTranslator.endOfLineMarker(language)+"\n";
	
    		}
    	else if (encoding == Generator.STRING){
    		if (Generator.objectsApproachMode) strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",new String[]{LanguageTranslator.callMethod(language,true,"customType","getRecipient",null)})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode) {
	    		if (language.equals(Language.JAVA) )
	    		strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",
	    				new String[]{LanguageTranslator.cast(language, "short",
	    						LanguageTranslator.callMethod(language,true,"h","get",new String[]{"\"recipient\""}))})
	    						+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		
	    		if (language.equals(Language.CSHARP) )
		    		strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","unicastRecipientEncoding",
		    				new String[]{LanguageTranslator.cast(language, "short",
		    						"h[\"recipient\"]" )})
		    						+LanguageTranslator.endOfLineMarker(language)+"\n";
	        	
	    	}
	    }
    	
    	strings[11] = writeUnicastCommunicationModelMethods(language,encoding);
    }
    
    
    public static void addRecipient4UnicastComModelDecoding(Language language,int encoding){
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if (encoding == Generator.BINARY){
	    	if (Generator.objectsApproachMode&& (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) strings[8] += "\t\t"+LanguageTranslator.addLocalVar(language,true,LanguageTranslator.callMethod(language,false,"","unicastRecipientDecoding",new String[]{"dis"}),"short","recipient")+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode){
	    		if (language.equals(Language.JAVA) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",new String[]{"\"recipient\"",LanguageTranslator.callMethod(language,false,"","unicastRecipientDecoding",new String[]{"dis"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		if (language.equals(Language.CSHARP) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",new String[]{"\"recipients\"", LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"dis"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		
	    	}
    	}
    	else if (encoding == Generator.STRING){
    		if (Generator.objectsApproachMode) strings[8] += "\t\t"+LanguageTranslator.addLocalVar(language,true,LanguageTranslator.callMethod(language,false,"","unicastRecipientDecoding",new String[]{"encodedString"}),"short","recipient")+LanguageTranslator.endOfLineMarker(language)+"\n";
	    	if (Generator.hashtablesApproachMode) strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",new String[]{"\"recipient\"",LanguageTranslator.callMethod(language,false,"","unicastRecipientDecoding",new String[]{"encodedString"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
    	}
    }
   
    
    public static void startToWriteEncodingMethods(Language language, int encoding, String typeClassName, String typeName, String destinationClassName,int currentTypeCommunicationModel ) {	
    	if(language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
    		System.out.println("CustomTypesGenerator: add method encode"+typeName);
        	String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
        	//String encodeMethods = strings[7];
        	
        	if (Generator.objectsApproachMode){ 
        		String type1 = typeClassName;
         		if (Generator.isCompactMode) type1 = destinationClassName;
    		        	
         		if (encoding == Generator.STRING){
                	String[][] attrs = {{type1,"customType"}};
                	strings[7]  += "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","encode"+typeName, attrs, false, null)+"\n";
                	//encodeMethods+="\t public static function encode"+typeName+"(customType:"+typeClassName+"){ \n";
            	} else if (encoding == Generator.BINARY){
            		// TODO 19/O6/O9
            		String[] exceptions = {"IOException"};
            		String[][] attrs = null; // {{"DataOutputStream","dos"},{type1,"customType"}};
            		if (language.equals(Language.JAVA) ) attrs = new String[][]{{"DataOutputStream","dos"},{type1,"customType"}};
                	if (language.equals(Language.CSHARP) ) attrs = new String[][] {{"EndianBinaryWriter","dos"},{type1,"customType"}};
            		strings[7]  += "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","encode"+typeName, attrs,true,exceptions)+"\n";
                	// encodeMethods+="\t public void encode"+typeName+"(DataOutputStream dos,"+typeClassName+" customType) throws Exception { \n";
            	}
         	}
        	if (Generator.hashtablesApproachMode){
            	if (encoding == Generator.STRING){
                	String[][] attrs = {{"Hashtable","h"}};
                	strings[7]  += "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","encode"+typeName, attrs,false,null)+"\n";
                	//encodeMethods+="\t public static function encode"+typeName+"(h:Hashtable) { \n";
            	}else if (encoding == Generator.BINARY){
            		String[] exceptions = {"IOException"};
            		String[][] attrs = null; //{{"DataOutputStream","dos"},{"Hashtable","h"}};
            		if (language.equals(Language.JAVA) ) attrs = new String[][]{{"DataOutputStream","dos"},{"Hashtable","h"}};
                	if (language.equals(Language.CSHARP) ) attrs = new String[][] {{"EndianBinaryWriter","dos"},{"Hashtable","h"}};
            		strings[7]  += "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","encode"+typeName, attrs,true,exceptions)+"\n";
                	//encodeMethods+="\t public void encode"+typeName+"(DataOutputStream dos, Hashtable h) throws Exception { \n";
            	}
        	}
    	}
    	    	    
    	switch (currentTypeCommunicationModel){
	        case Generator.MULTICAST: 
	        	CustomTypesGenerator.addRecipients4MulticastComModelEncoding(language,encoding);
	        	break;
	        case Generator.UNICAST: 
	        	CustomTypesGenerator.addRecipient4UnicastComModelEncoding(language,encoding);
	        	break;

	        default: break;
    	}
        //strings[7] = encodeMethods;
        //stringsToWrite.put(currentCustomTypes,strings);
    }
    
    
    // TODO: rajouter la gestion des autres param
	public static String startToWriteEncodingMethodsForCpp(int encoding) {	
		String res = "";
		if (encoding == Generator.BINARY){
    		// TODO 19/O6/O9
    		String[][] attrs = {{"SocketStream&","ss"}};
    		res  += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.CPP,false,false,false,true,"","encode", attrs,false,null)+"\n";
        	// encodeMethods+="\t public void encode"+typeName+"(DataOutputStream dos,"+typeClassName+" customType) throws Exception { \n";
    	}
		return res;
	}
    
    
    public static void continueToWriteEncodingMethods(Language language, int encoding, String elementType, String elementName, boolean isASequence) {	
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	//String encodeMethods = strings[7];
    	
    	//FIXME
    	
        if (Generator.objectsApproachMode){
        	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
        		// for complex types
        		if(encoding == Generator.BINARY && LanguageTranslator.getPrimitiveJavaType(elementType).equals("null")){
               		String[] params25 = {"dos",LanguageTranslator.callMethod(language,true,"customType","get"+Generator.upCaseFirstLetter(elementName),null)};
               		strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+elementType,params25)+LanguageTranslator.endOfLineMarker(language)+"\n";
                    //encodeMethods+="\t\t encode"+elementType+"(customType.get"+elementName+"());\n";
        		}
        	}
        }
        if (Generator.hashtablesApproachMode){
        	String[] params25 = null;
            if (isASequence) {
            	if (language.equals(Language.JAVA) )
            	params25 = new String[]{"dos",LanguageTranslator.cast(
            										language, 
            										"Hashtable", 
            										LanguageTranslator.callMethod(
            												language, 
            												true, 
            												"h", 
            												"get",   												
            												new String[]{"\""+elementName+"\""}))};
            	if (language.equals(Language.CSHARP) )
                	params25 = new String[]{"dos",LanguageTranslator.cast(
                										language, 
                										"Hashtable", 
                										"h[\""+elementName+"\"]" 												
                												)};
            	
            }
            
            
            else params25 = new String[]{"dos","hs"};
            strings[7] += "\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+elementType,params25)+LanguageTranslator.endOfLineMarker(language)+"\n";
            //encodeMethods+="\t\t encode"+elementType+"(dos,hs);\n";
        }
        
        //strings[7] = encodeMethods;
        //stringsToWrite.put(currentCustomTypes,strings);
    }
    /*
    public static String continueToWriteEncodingMethodsForCpp(String elementType, String elementName) {	
    	String currentCustomTypes = getCurrentClassLink(LanguageTranslator.CPP);
    	System.out.println("Current CUSTOM TYPES = "+currentCustomTypes);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	for(int i = 0; i < strings.length; i++){
    		System.out.println("STRINGS "+i+" = "+strings[i]);
    	}
    	//String encodeMethods = strings[7];
    	
        // (Generator.objectsApproachMode)
        String[] params25 = {"dos",LanguageTranslator.callMethod(LanguageTranslator.CPP,true,"",Generator.upCaseFirstLetter(elementName),null)};
    	for(int i = 0; i < params25.length; i++){
    		System.out.println("PARAM 25 - "+i+" = "+params25[i]);
    	}
        strings[7] += "\t\t"+LanguageTranslator.callMethod(LanguageTranslator.CPP,false,"","encode"+elementType,params25)+LanguageTranslator.endOfLineMarker(LanguageTranslator.CPP)+"\n";
       return strings[7];
        
    }*/
        
    
    
    public static void generateEncodingType(Language language, int encoding, String elementName, String type) {
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	//String encodeMethods = strings[7];
        
    	if (Generator.objectsApproachMode) {
        	if (encoding == Generator.STRING){
        		String toEncode = LanguageTranslator.callMethod(language,true,"customType","get"+elementName,null);
        		
        		//specific treatments to handle booleans on String
        		if (LanguageTranslator.getType(language,type).toLowerCase().equals("boolean")){
        			toEncode = "encodeBoolean("+toEncode+")";
        			Generator.isThereABooleanAttribute = true;
        		} else toEncode = "\"\"+"+toEncode;

	        	String[] params = {toEncode};
	        	String[] itemsToAdd = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",params)};
	        	strings[7]  += "\t\t"+LanguageTranslator.add(language,true,"String","encoding",itemsToAdd)+"\n";  
	        	//toEncoder="\t\t encoding += addEncodingSpacer("+getFlashTypeEncoder(ClassGenerator.getFlashType(type),"customType.get"+elementName+"()")+");\n";
        	} else if (encoding == Generator.BINARY){
        		if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
        			strings[7]  += "\t\t"+LanguageTranslator.writeBinaryType(
							language,
							type,
							LanguageTranslator.callMethod(language,true,"customType","get"+elementName,null),
							"dos")+"\n";
        			// toEncoder="\t\t dos.write"+eType+"(customType.get"+elementName+"());\n";

        		}/*
        		if(language.equals(Language.CPP) ){
        			strings[7]  += "\t\t"+LanguageTranslator.writeBinaryType(
							language,
							type,
							LanguageTranslator.callMethod(language,true,"customType","get"+elementName,null),
							"ss")+"\n";

        		}*/
        	}
        }
        if (Generator.hashtablesApproachMode) {
        	if (encoding == Generator.STRING){
        		String[] params2 = {"\""+elementName+"\""};
        		String toEncode = null;
        		
        		if (language.equals(Language.JAVA) ) 
        			toEncode = LanguageTranslator.callMethod(language,true,"h","get",params2);
        		if (language.equals(Language.CSHARP) )
        			toEncode = "h["+params2[0]+"]";
        		
        		
        		//specific treatments to handle booleans on String
        		if (LanguageTranslator.getType(language,type).toLowerCase().equals("boolean")){
        			toEncode = "encodeBoolean("+LanguageTranslator.getObjectTypeValue(
        										language, 
        										type,
        										LanguageTranslator.cast(language,type,toEncode))+")";
        			Generator.isThereABooleanAttribute = true;
        		} else toEncode = "\"\"+"+toEncode;

	        	String[] params = {toEncode};
	        	String[] itemsToAdd = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",params)};
	        	strings[7]  += "\t\t"+LanguageTranslator.add(language,true,"String","encoding",itemsToAdd)+"\n";  
	        	//toEncoder="\t\t encoding += addEncodingSpacer("+getFlashTypeEncoder(ClassGenerator.getFlashType(type),"h.get(\""+elementName+"\")")+");\n";
        	} else if (encoding == Generator.BINARY){
	        	String[] params = {"\""+elementName+"\""};
        		
	        	String toWrite = null;
	        	
	        	if (language.equals(Language.JAVA) ) 
	        		toWrite = 
	        			LanguageTranslator.getObjectTypeValue(
							language,
							type,
							LanguageTranslator.cast(
								language, 
								type,
								LanguageTranslator.callMethod(language,true,"h","get",params))
							);
        		
        		if (language.equals(Language.CSHARP) )
        			toWrite = 
	        			LanguageTranslator.getObjectTypeValue(
							language,
							type,
							LanguageTranslator.cast(
								language, 
								type,
								"h["+params[0]+"]"
							)
						);
        		
        		strings[7]  += "\t\t"+LanguageTranslator.writeBinaryType(
										language,
										type,
										toWrite,
										"dos")+"\n";
	        	//toEncoder="\t\t dos.write"+eType+"((("+getCastType(type)+") h.get(\""+elementName+"\"))."+type.toLowerCase()+"Value());\n";
        	}
        }
       
        //strings[7] = encodeMethods;
       // stringsToWrite.put(currentCustomTypes,strings);
	}
    
    public static void generateEncodingArrayType(Language language, int encoding, boolean isACustomTypeArray, int dimension, int n, String type, boolean isLastElement){
    	generateEncodingArrayType(language,encoding,isACustomTypeArray,dimension,n,type,type,isLastElement);
    }
    
    
    public static void generateEncodingArrayType(Language language, int encoding, boolean isACustomTypeArray, int dimension, int n, String name, String type, boolean isLastElement) {
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
    		String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);

    		String t = type.toLowerCase();
    		String toEncoder="";
    		String nTable= "table"+n;
    		String nTableL= "table"+n+"_l";

    		if (Generator.objectsApproachMode){ 
    			// ex: UpdatePlayer[] table1 = customType.getUpdatePlayer();
    			strings[7]  += "\t\t"+LanguageTranslator.addLocalVar(
    					language,
    					true,
    					LanguageTranslator.callMethod(language,true,"customType","get"+upCaseFirstLetter(name),null),
    					LanguageTranslator.arrayDeclaration(language,(Generator.isCompactMode? Generator.destinationClassName:type),dimension), 
    					""+nTable
    			)+"\n";

    			//toEncoder+="\t\t var "+nTable+":Array = customType.get"+upCaseFirstLetter(type)+"();\n";
    		}
    		else if (Generator.hashtablesApproachMode){
    			String[] params = {"\""+name+"\""};
    			String arrayDeclaration = LanguageTranslator.arrayDeclaration(language,isACustomTypeArray?"Hashtable":type,dimension);    
    			
    			if (language.equals(Language.JAVA) )
	    			strings[7]  += "\t\t"+LanguageTranslator.addLocalVar(language,
	    					true,
	    					LanguageTranslator.cast(language,arrayDeclaration,LanguageTranslator.callMethod(language,true,"h","get",params)),
	    					arrayDeclaration, 
	    					""+nTable
	    			)+"\n";
    			if (language.equals(Language.CSHARP) )
	    			strings[7]  += "\t\t"+LanguageTranslator.addLocalVar(language,
	    					true,
	    					LanguageTranslator.cast(language,arrayDeclaration,"h["+params[0]+"]"),
	    					arrayDeclaration, 
	    					""+nTable
	    			)+"\n";
    			//toEncoder+="\t\t var "+nTable+":Array = h.get(\""+type+"\");\n";
    		}

    		int d = 0;
    		String ind;
    		String[] vars = new String[dimension];

    		while(d<dimension){
    			ind = indent(d);

    			//String[] params = {"\""+type+"\""};
    			// int table1_l0 = table1.length;
    			strings[7]  += "\t\t"+ind+LanguageTranslator.addLocalVar(language,
    					true,
    					LanguageTranslator.arrayLength(language,nTable+tableLength(language,d)),
    					"int", 
    					nTableL+d
    			)+"\n";
    			//toEncoder+="\t\t"+ind+" var "+nTableL+d+" = "+nTable+""+tableLength(d)+".length;\n";

    			if (encoding == Generator.STRING){
    				String[] params1 = {"\"\"+"+nTableL+d};
    				String[] itemsToAdd = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer", params1)};
    				strings[7]  += "\t\t"+ind+LanguageTranslator.add(language,
    						true,
    						"String",
    						"encoding",
    						itemsToAdd)+"\n";  
    				//toEncoder+="\t\t"+ind+" encoding += addEncodingSpacer("+nTableL+d+");\n";
    			}
    			else if (encoding == Generator.BINARY){
    				//dos.writeInt((int) table1_l0);
    				strings[7] +="\t\t"+LanguageTranslator.writeBinaryType(language,"int",LanguageTranslator.castPrimitive(language,"int",""+nTableL+d),"dos")+"\n";
    			}

    			//if (table1_l0>0) 
    			vars[d] = LanguageTranslator.testSup(language, true, new String[]{""+nTableL+d, ""+0});
    			d++;
    		}

    		if (dimension > 1)
    			strings[7]  += "\t\t"+LanguageTranslator.ifDeclaration(language,LanguageTranslator.testAnd(language, true, vars))+"\n";
    		else 
    			strings[7]  += "\t\t"+LanguageTranslator.ifDeclaration(language, vars[0])+"\n";

    		d = 0;

    		while(d<dimension){
    			ind = indent(d);

    			strings[7]  += "\t\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+nTableL+d)+"\n";
    			//toEncoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";

    			if (isACustomTypeArray){
    				//		        	 if (Generator.objectsApproachMode){
    				if (encoding ==  Generator.STRING){
    					String[] params = {nTable+accessTable(language,dimension)};
    					strings[7]  += "\t\t\t\t"+ind+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
    					//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"("+nTable+accessTable(dimension)+");\n";
    				}
    				else if (encoding == Generator.BINARY){
       					String[] params = {"dos",nTable+accessTable(language,dimension)};
    					strings[7]  += "\t\t\t\t"+ind+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
    					//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"(dos,"+nTable+accessTable(dimension)+");\n";
    				}
    				//		        	 }
    				//		        	 else if (Generator.hashtablesApproachMode){
    				//		        		if (encoding ==  Generator.STRING){
    				//		 	        		String[] params = {nTable+accessTable(language,dimension)};
    				//		 	        		strings[7]  += "\t\t"+indent(dimension)+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
    				//		 	            	//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"("+nTable+accessTable(dimension)+");\n";
    				//		 	        	}
    				//		 	        	else if (encoding == Generator.BINARY){
    				//		 	        		//String[] params = {"dos",nTable+accessTable(language,dimension)};
    				//		 	        		//strings[7]  += "\t\t"+indent(dimension)+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
    				//		 	        		//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"(dos,"+nTable+accessTable(dimension)+");\n";
    				//		 	        		String[] params = {"dos",nTable+accessTable(language,dimension)};
    				//		 	        		strings[7]  += "\t\t"+indent(dimension)+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
    				//		 	        		
    				//		 	        	}
    				//		        		//"encode"+type+"(dos,(Hashtable) hs.get("+name+"));";
    				//		        	 }
    			}
    			else{
    				if (encoding == Generator.STRING){	
    					strings[7]  +="\t\t\t\t"+ind+" " +
    					"encoding += addEncodingSpacer("+nTable+accessTable(language,dimension)+");\n"; 
    				} else if (encoding == Generator.BINARY){
    					//if((language.equals(Language.JAVA) )||(language.equals(Language.FLASH) )){
    					strings[7]  += "\t\t\t\t"+ind+LanguageTranslator.writeBinaryType(language,type,""+nTable+accessTable(language,dimension),"dos")+"\n";
    					/*}
			        	if(language.equals(Language.CPP) ){
			        		strings[7]  += "\t\t\t\t"+ind+LanguageTranslator.writeBinaryType(language,type,""+nTable+accessTable(language,dimension),"ss")+"\n";
			        	}*/
    				}
    			}

    			strings[7] += "\t"+closeQuote(language, dimension);

    			d++;
    		}

    		strings[7] += closeQuote(language, dimension);
    		if (isLastElement) strings[7]  += "\n";

    	}
	}
    
    
    public static void finishAnEncodingMethods(Language language) {
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
        	String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    		strings[7] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n\n";	
    	}
		//stringsToWrite.put(currentCustomTypes,strings);
	}
    
    
    public static void finishEncodeData(Language language, int encoding) {
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
		strings[5] += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
        if (encoding == Generator.STRING) strings[5] += "\t\t"+LanguageTranslator.returnValue(language,"encoding")+"\n"; 
        strings[5] += "\t"+LanguageTranslator.closeBlock(language)+"\n";
    	//stringsToWrite.put(currentCustomTypes,strings);
	}
    
    
    public static void startToWriteDecodingMethods(Language language, int encoding, String typeClassName, String typeNameFinal, String typeName, String destinationClassName, int currentTypeCommunicationModel, boolean isAMessage) {	
    	if(language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
    		System.out.println("CustomTypesGenerator: add method decode"+typeName);
        	String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
        	//String decodeMethods = strings[8];
        	
        	 
            if (Generator.objectsApproachMode){
            	String type1 = typeClassName;
            	if (Generator.isCompactMode) type1 = destinationClassName;
    		        	
            	if (encoding == Generator.STRING){
    	           	String[][] attrs = {{"String","encodedString"}};
    	           	strings[8] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,Generator.isClient,false,false,type1,"decode"+typeName, attrs, false, null)+"\n";  
    	          	//decodeMethods+="\t public static function decode"+typeName+"(encodedString:String):"+typeClassName+" {\n";
              	}else if (encoding == Generator.BINARY){
                	String[] exceptions = {"IOException"};
                	String[][] attrs = null;
                	
                	if (language.equals(Language.JAVA) ) attrs = new String[][]{{"DataInputStream","dis"}};
                	if (language.equals(Language.CSHARP) ) attrs = new String[][] {{"EndianBinaryReader","dis"}};
                	
                	strings[8] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,Generator.isClient,false,false,type1,"decode"+typeName, attrs, true, exceptions)+"\n";  
    	        	// decodeMethods+="\t public "+typeClassName+" decode"+typeName+"(DataInputStream dis) throws Exception { \n";
              	}
            	
            	boolean isACommunicationModelSpecified = true;
            	String attribute = "";
            	switch (currentTypeCommunicationModel){
            		case Generator.MULTICAST: 
    		        	CustomTypesGenerator.addRecipients4MulticastComModelDecoding(language,encoding);
    		        	attribute = "recipients";
    		        	break;
            		case Generator.UNICAST: 
    		        	CustomTypesGenerator.addRecipient4UnicastComModelDecoding(language,encoding);
    		        	attribute = "recipient";
    		        	break;
    		        	
            		default: 
            			isACommunicationModelSpecified = false; 
            			break;
            	}
            	
            	strings[8] += "\t\t"+LanguageTranslator.addLocalVar(
    							language,
    								true,
    								isACommunicationModelSpecified? LanguageTranslator.newType(language,type1,attribute):""
    								,type1
    								,"customType")+"\n"+(Generator.isCompactMode? "":"\n");
              	//decodeMethods+="\t\t var customType:"+typeClassName+" = new "+typeClassName+"();\n";
            	
            	if (Generator.isCompactMode){
            		String[] items = {typeNameFinal};
            		strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"customType","setType",items)+LanguageTranslator.endOfLineMarker(language)+"\n\n";   
            	}
            }
                	
          	if (Generator.hashtablesApproachMode){
             	if (encoding == Generator.STRING){
                	String[][] attrs = {{"String","encodedString"}};
                	strings[8] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,Generator.isClient,false,false,"Hashtable","decode"+typeName, attrs,false,null)+"\n";
                	//decodeMethods+="\t public static function decode"+typeName+"(encodedString:String):Hashtable {\n";
            	}else if (encoding == Generator.BINARY){
                	String[] exceptions = {"IOException"};	
                	String[][] attrs = null;
                	if (language.equals(Language.JAVA) ) attrs = new String[][]{{"DataInputStream","dis"}};
                	if (language.equals(Language.CSHARP) ) attrs = new String[][] {{"EndianBinaryReader","dis"}};
                	strings[8] += "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,Generator.isClient,false,false,"Hashtable","decode"+typeName, attrs,true,exceptions)+"\n";
                	//decodeMethods+="\t public Hashtable decode"+typeName+"(DataInputStream dis) throws Exception { \n";
            	}
             	
             	strings[8] += "\t\t"+LanguageTranslator.addLocalVar(
    					language,
    					true,
    					"",
    					"Hashtable",
    					"h")+"\n";
             			//decodeMethods+="\t\t var h:Hashtable = new Hashtable();\n";
             	
             	boolean isACommunicationModelSpecified = true;
             	String attribute = "";
             	switch (currentTypeCommunicationModel){
    	    		case Generator.MULTICAST: 
    		        	CustomTypesGenerator.addRecipients4MulticastComModelDecoding(language,encoding);
    		        	attribute = "recipients";
    		        	break;
    	    		case Generator.UNICAST: 
    		        	CustomTypesGenerator.addRecipient4UnicastComModelDecoding(language,encoding);
    		        	attribute = "recipient";
    		        	break;
    	
    	    		default: 
    	    			isACommunicationModelSpecified = false; 
    	    			break;
             	}
             	
             	//isACommunicationModelSpecified? LanguageTranslator.newType(language,typeClassName,attribute):
    	            	
            	String[] params24 = {"\"type\"",isAMessage?LanguageTranslator.newType(language, "Byte", new String[]{typeNameFinal}):"\""+typeName.toLowerCase()+"\""};
            	
            	if (language.equals(Language.JAVA) ) 
            		strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",params24)+LanguageTranslator.endOfLineMarker(language)+"\n\n";
            	if (language.equals(Language.CSHARP) )
	    			strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add",new String[]{"\"recipients\"", LanguageTranslator.callMethod(language,false,"","multicastRecipientsDecoding",new String[]{"dis"})})+LanguageTranslator.endOfLineMarker(language)+"\n";
	    		
            	
            	//decodeMethods+="\t\t h.put(\"type\",\""+typeClassName+"\");\n";
          	}   	
            
            //strings[8] = decodeMethods;
            //stringsToWrite.put(currentCustomTypes,strings);
    	}
    	if(language.equals(Language.CPP) ){
    		boolean isACommunicationModelSpecified = true;
        	String attribute = "";
        	switch (currentTypeCommunicationModel){
        		case Generator.MULTICAST: 
		        	CustomTypesGenerator.addRecipients4MulticastComModelDecoding(language,encoding);
		        	attribute = "recipients";
		        	break;
        		case Generator.UNICAST: 
		        	CustomTypesGenerator.addRecipient4UnicastComModelDecoding(language,encoding);
		        	attribute = "recipient";
		        	break;
		        	
        		default: 
        			isACommunicationModelSpecified = false; 
        			break;
        	}
    	}
    	
    }
    
	public static String startToWriteDecodingMethodsForCpp(int encoding) {	
		String res = "";
		if (encoding == Generator.BINARY){
			String[][] attrs = {{"const SocketStream&","ss"}};
    		res  += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.CPP,false,false,false,true,"","decode", attrs,false,null)+"\n";
        	// encodeMethods+="\t public void encode"+typeName+"(DataOutputStream dos,"+typeClassName+" customType) throws Exception { \n";
    		//res  += "\t"+LanguageTranslator.writeFunctionDeclaration(LanguageTranslator.CPP,false,false,false,true,"","encode", attrs,false,null)+"\n";<>
		}
		return res;
	}
    
    public static void continueToWriteDecodingMethods(Language language, int encoding,  String elementType, String elementName, boolean isASequence) {	
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	//String decodeMethods = strings[8];
        if (Generator.objectsApproachMode){
        	if(encoding == Generator.BINARY && (language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
        		String[] params25 = {LanguageTranslator.callMethod(language,false,"","decode"+elementType,new String[]{"dis"})};
        		strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"customType","set"+Generator.upCaseFirstLetter(elementName),params25)+LanguageTranslator.endOfLineMarker(language)+"\n";
        		//customType.set"+elementName+"(decode"+elementType+"(dis));"
        	}
        }
        if (Generator.hashtablesApproachMode){
            String[] params25 = {"dis"};
            if (isASequence){
            	String[] params26 = {"\""+elementName+"\"",LanguageTranslator.callMethod(language,false,"","decode"+elementType,params25)};
            	if (language.equals(Language.JAVA) )
            		strings[8] += "\t\t"+LanguageTranslator.callMethod(language, true, "h", "put", params26)
            		+LanguageTranslator.endOfLineMarker(language)+"\n";
            	if (language.equals(Language.CSHARP) )
            		strings[8] += "\t\t"+LanguageTranslator.callMethod(language, true, "h", "Add", params26)
                	+LanguageTranslator.endOfLineMarker(language)+"\n";
            }
            else strings[8] += "\t\t"+LanguageTranslator.callMethod(language,false,"","encode"+elementType,params25)+"\n";
            
            	;
            //h.put("ProposalData",decodeProposalData(dos,hs));
            //encodeMethods+="\t\t encode"+elementType+"(dos,hs);\n";
        }
        
        //strings[8] = decodeMethods;
        //stringsToWrite.put(currentCustomTypes,strings);
    }
    
    
    public static void generateDecodingType(Language language, int encoding, String elementName, String type) {
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	//String decodeMethods = strings[8];
    	
		if (Generator.objectsApproachMode){
			if (encoding == Generator.STRING){
				String[] params1 = {"encodedString"};
    			LanguageTranslator.callMethod(language,false,"","getNextParam",params1);
        		String toDecode = LanguageTranslator.callMethod(language,false,"","getNextParam",params1);
    			 
        		//specific treatments to handle booleans on String
        		if (LanguageTranslator.getType(language,type).toLowerCase().equals("boolean")){
        			//toDecode = "decodeBoolean("+toDecode+")";
        			Generator.isThereABooleanAttribute = true;
        		}
    			
        		String[] params = {LanguageTranslator.parseStringType(language,type, toDecode)};
        		strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"customType","set"+elementName, params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
    			//toDecoder = "\t\t customType.set"+elementName+"("+getFlashTypeDecoder(ClassGenerator.getFlashType(type),"getNextParam(encodedString)")+");\n";
			} else if (encoding == Generator.BINARY){
				if(language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
					String[] params = {LanguageTranslator.readBinaryType(language,type,"dis")};
					strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"customType","set"+elementName, params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
					//toDecoder = "\t\t customType.set"+elementName+"(dis.read";
				}
			}
		}
    	if (Generator.hashtablesApproachMode){
    		if (encoding == Generator.STRING){
    			String[] params1 = {"encodedString"};
    			LanguageTranslator.callMethod(language,false,"","getNextParam",params1);
        		String toDecode = LanguageTranslator.callMethod(language,false,"","getNextParam",params1);
    			
        		//specific treatments to handle booleans on String
        		if (LanguageTranslator.getType(language,type).toLowerCase().equals("boolean")){
        			toDecode = "decodeBoolean("+toDecode+")";
        			Generator.isThereABooleanAttribute = true;
        		}

    			String[] params = {"\""+elementName+"\"",LanguageTranslator.newType(language, type,LanguageTranslator.parseStringType(language,type,toDecode))};
    			if (language.equals(Language.JAVA) )
    				strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put", params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
    			if (language.equals(Language.CSHARP) )
    				strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add", params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
    			
    			//toDecoder = "\t\t h.put(\""+elementName+"\","+getFlashTypeDecoder(ClassGenerator.getFlashType(type),"getNextParam(encodedString)")+");\n";
    		} else if (encoding == Generator.BINARY){
    			String[] params = {"\""+elementName+"\"",LanguageTranslator.newType(language, type, LanguageTranslator.readBinaryType(language,type,"dis"))};
    			if (language.equals(Language.JAVA) )
    				strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put", params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
    			if (language.equals(Language.CSHARP) )
    				strings[8] += "\t\t"+LanguageTranslator.callMethod(language,true,"h","Add", params)+LanguageTranslator.endOfLineMarker(language)+"\n"; 
    			
    			//toDecoder = "\t\t h.put(\""+elementName+"\", new "+getCastType(type)+"(dis.read";
    		}
    	}
    	
        //strings[8] = decodeMethods;
        //stringsToWrite.put(currentCustomTypes,strings);
	}
    
    
    public static void generateDecodingArrayType(Language language, int encoding, boolean isACustomTypeArray, int dimension, int n, String type, boolean isLastElement){
    	generateDecodingArrayType(language,encoding,isACustomTypeArray,dimension,n,type,type,isLastElement);
    }
    
    
    public static void generateDecodingArrayType(Language language, int encoding, boolean isACustomTypeArray, int dimension, int n, String name, String type, boolean isLastElement) {
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
    		String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
        	//String decodeMethods = strings[8];
        	
        	String t = type.toLowerCase();
            String toDecoder="";
            String nTable= "table"+n;
            String nTableL= "table"+n+"_l";

            boolean noLowerCase = false;
            if (type.equals("String")||isACustomTypeArray) noLowerCase = true;

            int d = 0;
            String[] vars = new String[dimension];
            String ind;
            
            while(d<dimension){
            	ind = indent(d);
            	if (encoding == Generator.STRING){
            		String[] params = {"encodedString"};
            		String parse = LanguageTranslator.callMethod(language,false,"","getNextParam",params);
            		strings[8]+= "\t\t"+ind+LanguageTranslator.addLocalVar(
    	            							language,
    	            							true,
    	            							LanguageTranslator.parseStringType(language,"int",parse),
    	            							"int",
    	            							nTableL+d
    	            			)+"\n";  
            		//toDecoder+="\t\t"+ind+" var "+nTableL+d+" = parseInt(getNextParam(encodedString));\n";
            	} else if (encoding == Generator.BINARY ){
            		strings[8]+= "\t\t"+ind+LanguageTranslator.addLocalVar(
    									language,
    									true,
    									LanguageTranslator.readBinaryType(language,"int","dis"),
    									"int",
    									nTableL+d
            		 				)+"\n";  
            	}
            
            	vars[d] = LanguageTranslator.testSup(language, true, new String[]{""+nTableL+d, ""+0});
            	d++;	
            }
            
            if (dimension > 1)
            	strings[8]  += "\t\t"+LanguageTranslator.ifDeclaration(language,LanguageTranslator.testAnd(language, true, vars))+"\n";
            else 
            	strings[8]  += "\t\t"+LanguageTranslator.ifDeclaration(language, vars[0])+"\n";
            
            d = 0;
            
            while(d<dimension){
            	ind = indent(d);
            	
            	  String init = "";
      	        if (Generator.objectsApproachMode){ 
      			    if (noLowerCase) init = LanguageTranslator.arrayInit(language,(Generator.isCompactMode? Generator.destinationClassName:type),dimension,nTableL+d);
      			    else init = LanguageTranslator.arrayInit(language,t,dimension,nTableL+d);
      			    String init1 = LanguageTranslator.getAttributesInitiator(language,init);
      			           
      			    strings[8]+= "\t\t\t"+ind+LanguageTranslator.addLocalVar(
      			            		language,
      								true,
      								init1,
      								LanguageTranslator.arrayDeclaration(language,(Generator.isCompactMode? Generator.destinationClassName:type),dimension)
      								,""+nTable)+"\n";
      			    //toDecoder+="\t\t"+ind+" var "+nTable+":Array";
      			    //if (noLowerCase)
      			    //	toDecoder+=initTable(nTableL,dimension,d,type);
      			    //else toDecoder+=initTable(nTableL,dimension,d,t);
      			    //toDecoder+=";\n";
      	 
      			    strings[8]+= "\t\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+(nTableL+d))+"\n";
      		        //toDecoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";
      	        }
      	        if (Generator.hashtablesApproachMode){
      	        	init = LanguageTranslator.arrayInit(language,isACustomTypeArray?"Hashtable":type,dimension,nTableL+d);
      			    String init1 = LanguageTranslator.getAttributesInitiator(language,init);
      			           
      			    strings[8]+= "\t\t\t"+ind+LanguageTranslator.addLocalVar(
      			            		language,
      								true,
      								init1,
      								LanguageTranslator.arrayDeclaration(language,isACustomTypeArray?"Hashtable":type,dimension)
      								,""+nTable)+"\n";
      			    //toDecoder+="\t\t"+ind+" var "+nTable+":Array";
      			    //if (noLowerCase)
      			    //	toDecoder+=initTable(nTableL,dimension,d,type);
      			    //else toDecoder+=initTable(nTableL,dimension,d,t);
      			    //toDecoder+=";\n";
      	 
      			    strings[8]+= "\t\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+(nTableL+d))+"\n";
      		        //toDecoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";
      	        }
      	        
      	    	
            	
    	        if (isACustomTypeArray){
    	        	if (encoding == Generator.STRING){
    		        	String[] params = {"encodedString"};
    		        	strings[8] += "\t\t\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
    		                		 			language,
    		                		 			nTable+accessTable(language,dimension),
    		                		 			LanguageTranslator.callMethod(language,false,"","decode"+upCaseFirstLetter(type),params)
    		                		 			)+"\n"; 
    		        	//toDecoder+="\t\t"+indent(dimension)+" "+nTable+accessTable(FLASH,dimension)+"= decode"+upCaseFirstLetter(type)+"(encodedString);\n";
    	        	}
    	        	else if (encoding == Generator.BINARY){
    	        		String[] params = {"dis"};
    	        		strings[8] += "\t\t\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
    		                		 			language,
    		                		 			nTable+accessTable(language,dimension),
    		                		 			LanguageTranslator.callMethod(language,false,"","decode"+upCaseFirstLetter(type),params)
    		                		 			)+"\n"; 
    		        		 //toDecoder+="\t\t"+indent(dimension)+" "+nTable+accessTable(FLASH,dimension)+"= decode"+upCaseFirstLetter(type)+"(encodedString);\n";
    	        	}
    	         }
    	         else{
    		        if (encoding == Generator.STRING){
    			       String[] params = {"encodedString"};
    			       String parse = LanguageTranslator.callMethod(language,false,"","getNextParam",params);
    			       strings[8] += "\t\t\t\t"+ind+LanguageTranslator.affectValueToVar(
    			                					language,
    			                		 			nTable+accessTable(language,dimension),
    			                		 			LanguageTranslator.parseStringType(language,type,parse)
    			                		 			)+"\n";  
    		        }
    		        else if (encoding == Generator.BINARY) {
    		        	strings[8] += "\t\t\t\t"+ind+LanguageTranslator.affectValueToVar(
    												        				language,
    												        		 		nTable+accessTable(language,dimension),
    												        		 		LanguageTranslator.readBinaryType(language,type,"dis")
    											        		 		)+"\n";
    		        	}
    	        }
    	             
    	        strings[8] += "\t"+closeQuote(language,dimension);
    	        d++;
            }
    	    
            if (Generator.objectsApproachMode) {
    			String[] params = { nTable };
    			strings[8] += "\t\t\t"+ LanguageTranslator.callMethod(language, true,"customType", "set" 
    										+ upCaseFirstLetter(name),params)+ LanguageTranslator.endOfLineMarker(language) + "\n";
    			// toDecoder+="\t\t
    			// customType.set"+upCaseFirstLetter(type)+"("+nTable+");\n";
    		}
    		if (Generator.hashtablesApproachMode) {
    			String[] params = { "\"" + name + "\"", nTable };
    			if (language.equals(Language.JAVA) )
	    			strings[8] += "\t\t\t"
	    					+ LanguageTranslator.callMethod(language, true, "h", "put",
	    							params)
	    					+ LanguageTranslator.endOfLineMarker(language) + "\n";
    			// toDecoder+="\t\t h.put(\""+type+"\","+nTable+");\n";
    			if (language.equals(Language.CSHARP) )
	    			strings[8] += "\t\t\t"
	    					+ LanguageTranslator.callMethod(language, true, "h", "Add",
	    							params)
	    					+ LanguageTranslator.endOfLineMarker(language) + "\n";
    			// toDecoder+="\t\t h.put(\""+type+"\","+nTable+");\n";
    		}
      
    		strings[8] += closeQuote(language,1);
            if (!isLastElement) strings[8] += "\n";
            
            //strings[8] = decodeMethods;
            //stringsToWrite.put(currentCustomTypes,strings);
		
    	}
    	
    }

    
    public static void finishADecodingMethods(Language language) {
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
        	String currentCustomTypes = getCurrentClassLink(language);
        	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
        	if (Generator.objectsApproachMode)strings[8] += "\t\t"+LanguageTranslator.returnValue(language,"customType")+"\n";
            if (Generator.hashtablesApproachMode) strings[8] += "\t\t"+LanguageTranslator.returnValue(language,"h")+"\n";
            strings[8] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n\n";
    	}
		//stringsToWrite.put(currentCustomTypes,strings);
	}

	
	public static void finishDecodeData(Language language, int encoding) {
		String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	if(language.equals(Language.CPP) ){
    		// default case
    		strings[6] += "\t\t\t\t"+"default:"+"\n"+
    						"\t\t\t\t\t"+"break;"+"\n";	
    		// close the switch block
    		strings[6] += "\t\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    		
    		strings[6] += "\t\t\t"+"o->decode(ss);"+"\n";
		
    		strings[6] += "\t\t\t"+LanguageTranslator.affectValueToVar(
    															Language.CPP,
    															LanguageTranslator.arrayAccess(Language.CPP, "h","key"),
    															"o")+"\n";
    	}
    	strings[6] += "\t\t\t"+LanguageTranslator.increment(language,false,"nb")+"\n";
    	strings[6] += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ) strings[6] += "\t\t"+LanguageTranslator.returnValue(language,"h")+"\n";
    	strings[6] += "\t"+LanguageTranslator.closeBlock(language)+"\n";
	}
    
	
    public static void addImport(Language language, String _import) {	
    	String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	strings[1] += LanguageTranslator.writeImport(language, _import)+"\n";
    	System.out.println("currentCustomTypes:"+currentCustomTypes);
		System.out.println("strings:"+strings);
    	stringsToWrite.put(currentCustomTypes,strings);
    }
    

    public static void finalize(int i) throws IOException{
    	String[] strings = (String[]) stringsToWrite.get(classNames.get(i));
    	Language language = (Language) classLanguages.get(i);
    	
    	if (!strings[0].equals("")) strings[0] += "\n";
    	if (!strings[1].equals("")) strings[1] += (language.equals(Language.CPP) )?"using namespace std;"+"\n\n":"\n";
    	if (!strings[2].equals("")) strings[2] += (language.equals(Language.CPP) )?"\n{\n":"\n";
    	if (!strings[3].equals("") && !(language.equals(Language.CPP) )) strings[3] = "\t// types\n"+strings[3]+"\n\n";
    	if (!strings[3].equals("") && (language.equals(Language.CPP) )) strings[3] = "";
    	if (!strings[4].equals("")) strings[4] = "\t// constructor\n"+strings[4]+"\n\t"+LanguageTranslator.closeBlock(language)+"\n\n";
 
    	if (!strings[5].equals(""))
    		strings[5] = ((language.equals(Language.CPP) )?"\tpublic:\n\n":"")+"\t// encodeData root method\n"+strings[5]+"\n\n";

    	if (!strings[6].equals(""))
    		strings[6] = "\t// decodeData root method\n"+strings[6]+"\n\n";

    	if (!strings[7].equals(""))
    		strings[7] = "\t// encoding method for each custom type\n"+strings[7]+"\n";
    	
    	if (!strings[8].equals(""))
    		strings[8] = "\t// decoding method for each custom type\n"+strings[8]+"\n";

    	if (!strings[9].equals(""))
    		strings[9] = "\t// string encoding required methods\n"+strings[9];

    	if (!strings[10].equals(""))
    		strings[10] = "\t// multicast communication methods\n"+strings[10];
    	
    	if (!strings[11].equals(""))
    		strings[11] = "\t// unicast communication methods\n"+strings[11];
    	
    	if (!strings[12].equals("")){
    		// New line after #endif to avoid warning or error with some compiler
    		strings[12] += ((language.equals(Language.CPP) )?";\n"+LanguageTranslator.writeEndOfFile():"\n");
    	}
    		
    }
    
    public static void finalizeCustomTypesClasses() throws IOException{
	   	System.out.println("CustomTypesGenerator: finalize CustomTypes files...");
	   	for (int i=0; i<classNames.size(); i++) 
	   		finalize(i);
		System.out.println("CustomTypesGenerator: CustomTypes files finalized.");
   }

	public static void addGlobalVar(Language language, boolean isStatic, String init, String type, String name) {
		String currentCustomTypes = getCurrentClassLink(language);
    	String[] strings = (String[]) stringsToWrite.get(currentCustomTypes);
    	strings[3] += "\t"+LanguageTranslator.addGlobalVar(language,true,true,isStatic/*Generator.isClient*/,true,init,type,name)+"\n";
    	stringsToWrite.put(currentCustomTypes,strings);
	}

    public static String writeUnicastCommunicationModelMethods(Language language, int encoding){
    	String code = "";
	    String init = "";
	    String[][] params2 = null;
	    
	    if (encoding == Generator.BINARY){
	    	if (language.equals(Language.JAVA) ) params2 = new String[][]{{"DataInputStream","dis"}};
        	if (language.equals(Language.CSHARP) ) params2 = new String[][] {{"EndianBinaryReader","dis"}};
	    }
	    else /*(encoding == Generator.STRING)*/
	    	params2 = new String[][]{{"String","encodedString"}};


	   
	    // Unused for CPP
	    if(!language.equals(Language.CPP)){
	    	 /************************************************************/
	    	/** unicastRecipientEncoding(dos,recipient)                **/
	    	/************************************************************/
	    	String[][] params3 = null;
	    	if (encoding == Generator.BINARY){
	    		//params3 = new String[][]{{"DataOutputStream","dos"},{"short","recipient"}};
	    		if (language.equals(Language.JAVA) ) params3 = new String[][]{{"DataOutputStream","dos"},{"short","recipient"}};
            	if (language.equals(Language.CSHARP) ) params3 = new String[][] {{"EndianBinaryWriter","dos"},{"short","recipient"}};
	    	}
	    	else params3 = new String[][]{{"short","recipient"}};

	    	if (encoding == Generator.BINARY) {
	    		code +=  "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","unicastRecipientEncoding",params3,true,new String[]{"IOException"})+"\n";
	    		code += "\t\t"+LanguageTranslator.writeBinaryType(language,"short","recipient","dos")+"\n";
	    	}
	    	else /*(encoding == Generator.STRING)*/{
	    		code +=  "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","unicastRecipientEncoding",params3,false,null)+"\n";
	    		code += "\t\t"+LanguageTranslator.add(language,
	    				true,"String","encoding",
	    				new String[]{LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",new String[]{LanguageTranslator.newType(language,"short","recipient")})}
	    		)+"\n";
	    	}

	    	code +=  "\t"+LanguageTranslator.closeBlock(language)+"\n\n";

	    	/************************************************************/
	    	/** unicastRecipientEncoding(dos,recipient)                **/
	    	/************************************************************/

	    	if (encoding == Generator.BINARY){ 
	    		code +=  "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"short","unicastRecipientDecoding",params2,true,new String[]{"IOException"})+"\n";
	    		code += "\t\t"+LanguageTranslator.returnValue(language,LanguageTranslator.readBinaryType(language,"short","dis"))+"\n";
	    	}
	    	else /*(encoding == Generator.STRING)*/{
	    		code +=  "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"short","unicastRecipientDecoding",params2,false,null)+"\n";
	    		code += "\t\t"+LanguageTranslator.returnValue(language,LanguageTranslator.parseStringType(language,"short",LanguageTranslator.callMethod(language,false,"","getNextParam", new String[]{"encodedString"})))+"\n";
	    	}


	    	code +=  "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
	    }
    	return code;
    }
    
    
    public static String writeMulticastCommunicationModelMethods(Language language, int encoding){
    	String code = "";
    	
    	/************************************************************/
    	/** multicastRecipientsDecoding(dos,short[])               **/
    	/************************************************************/
    	if (encoding == Generator.BINARY && language.equals(Language.CPP) ){
	    	String[][] params = {{"SocketStream&","ss"}};
		    code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,false,false,true,"","multicastRecipientsEncoding",params,false,null)+"\n";
		    code+= "\t\t"+LanguageTranslator.writeBinaryType(language,"byte",LanguageTranslator.castPrimitive(language,"byte",LanguageTranslator.arrayLength(language,"recipients")),"ss")+"\n";

	    }else if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
	    	String[][] params = null;// {{"DataOutputStream","dos"},{LanguageTranslator.arrayDeclaration(language,"short",1),"recipients"}};
		    
	    	if (language.equals(Language.JAVA) ) params = new String[][]{{"DataOutputStream","dos"},{LanguageTranslator.arrayDeclaration(language,"short",1),"recipients"}};
        	if (language.equals(Language.CSHARP) ) params = new String[][] {{"EndianBinaryWriter","dos"},{LanguageTranslator.arrayDeclaration(language,"short",1),"recipients"}};
	    	code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","multicastRecipientsEncoding",params,true,new String[]{"IOException"})+"\n";
		    code+= "\t\t"+LanguageTranslator.writeBinaryType(language,"byte",LanguageTranslator.castPrimitive(language,"byte",LanguageTranslator.arrayLength(language,"recipients")),"dos")+"\n";

	    }
    	else if (encoding == Generator.STRING){
	    	String[][] params = {{LanguageTranslator.arrayDeclaration(language,"short",1),"recipients"}};
		    code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,true,"","multicastRecipientsEncoding",params,false,null)+"\n";
		    String newType =LanguageTranslator.newType(language,"int",LanguageTranslator.arrayLength(language,"recipients"));
		    code+= "\t\t"+LanguageTranslator.add(language, true, "String","encoding",new String[]{ LanguageTranslator.callMethod(language,false,"","addEncodingSpacer",new String[]{newType})})+"\n";
	    }
    	
	    code += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0",LanguageTranslator.arrayLength(language,"recipients"))+"\n";
	    
	    if (encoding == Generator.BINARY && language.equals(Language.CPP) ) code += "\t\t\t"+LanguageTranslator.writeBinaryType(language,"short",LanguageTranslator.arrayAccess(language,"recipients","i"),"ss")+"\n";
	    if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) code += "\t\t\t"+LanguageTranslator.writeBinaryType(language,"short",LanguageTranslator.arrayAccess(language,"recipients","i"),"dos")+"\n";
	    else if  (encoding == Generator.STRING){
	    	String[] params = {LanguageTranslator.newType(language,"short",LanguageTranslator.arrayAccess(language,"recipients","i"))};

	    	code += "\t\t\t"+LanguageTranslator.add(language,
	    			 							  true,"String","encoding",
	    			 							  new String[]{LanguageTranslator.callMethod(language,
	    			 									                                     false,"","addEncodingSpacer",
	    			 									                                     params)}
	    										 )+"\n";
	    }
	    
	    code += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
	    code += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
	    
	    /************************************************************/
    	/** short[] multicastRecipientsDecoding(dis)               **/
    	/************************************************************/
	    
	     String init = "";
	    String[][] params2 = null;
	    String valueToInitArray = "";
	    if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
	    	if (language.equals(Language.JAVA) ) params2 = new String[][]{{"DataInputStream","dis"}};
        	if (language.equals(Language.CSHARP) ) params2 = new String[][] {{"EndianBinaryReader","dis"}};
	    	valueToInitArray = LanguageTranslator.readBinaryType(language,"byte","dis");
	    }
	    else if (encoding == Generator.BINARY && language.equals(Language.CPP) ){
	    	params2 = new String[][]{{"const SocketStream&","ss"},{"std::vector<short>& ","_recipients"}};
	    	valueToInitArray = LanguageTranslator.readBinaryType(language,"byte","ss");
	    }
	    else /*(encoding == Generator.STRING)*/{
	    	params2 = new String[][]{{"String","encodedString"}};
	    	valueToInitArray = LanguageTranslator.parseStringType(language,"short",LanguageTranslator.callMethod(language,false,"","getNextParam",new String[]{"encodedString"}));	
	    }
	    
	    if (encoding == Generator.BINARY && language.equals(Language.CPP) )
	    	code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,true,true,false,true,"",
				"multicastRecipientsDecoding",params2,true,new String[]{"IOException"})+"\n";
	    else if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
	    	code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,LanguageTranslator.arrayDeclaration(language,"short",1),
				"multicastRecipientsDecoding",params2,true,new String[]{"IOException"})+"\n";
	    }
	    else /*(encoding == Generator.STRING)*/
	    	code+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,LanguageTranslator.arrayDeclaration(language,"short",1),
					"multicastRecipientsDecoding",params2,false,null)+"\n";
	    init = LanguageTranslator.getAttributesInitiator(
        									language,					
        									LanguageTranslator.arrayInit(
        											language,
        											"short",
        											1,
        											valueToInitArray
        									)
        						);
	    if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
	    	code += "\t\t"+LanguageTranslator.addLocalVar(language,true,init,LanguageTranslator.arrayDeclaration(language,"short",1),"recipients")+"\n";
		    code += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0",LanguageTranslator.arrayLength(language,"recipients"))+"\n";
	    }
	    
	    if(language.equals(Language.CPP) ){
	    	String[] param = new String[]{LanguageTranslator.readBinaryType(language,"byte","ss")};
	    	code += "\t\t"+LanguageTranslator.callMethod(Language.CPP,true, "_recipients", "resize", param)+";\n";
		    code += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0",LanguageTranslator.arrayLength(language," _recipients"))+"\n";
	    }
	    
	    String valueToAffect = "";
	    if (encoding == Generator.BINARY && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
	    	valueToAffect = LanguageTranslator.readBinaryType(language,"short","dis");
	    }
	    if (encoding == Generator.BINARY && (language.equals(Language.CPP) )){
	    	valueToAffect = LanguageTranslator.readBinaryType(language,"short","ss");
	    }
	    
	    else /*(encoding == Generator.STRING)*/ valueToAffect = valueToInitArray;
	    
	    if((language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )){
	    	code += "\t\t\t"+LanguageTranslator.affectValueToVar(language, LanguageTranslator.arrayAccess(language,"recipients","i"),valueToAffect)+"\n";//(language,"short",LanguageTranslator.arrayAccess(language,"recipients","i"),"dos")+"\n";
		    code += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
		    code += "\t\t"+LanguageTranslator.returnValue(language,"recipients")+"\n";
		    code += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";	
	    }
	    if(language.equals(Language.CPP) ){
	    	code += "\t\t\t"+LanguageTranslator.affectValueToVar(language, LanguageTranslator.arrayAccess(language,"_recipients","i"),valueToAffect)+"\n";//(language,"short",LanguageTranslator.arrayAccess(language,"recipients","i"),"dos")+"\n";
		    code += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
		    code += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
	    }
	    
	    
    	return code;
    }
    


	 /**
     * Generate string : ("}")*
     * 
     * @param dimension
     */
    private static String closeQuote(Language language,int d) {
    	int i = 0;
        String ind = "";
        while (i<d){
            ind+=indent(d-i+1)+LanguageTranslator.closeBlock(language)+"\n";
            i++;
        }
        return ind;
    }


    /**
     * Generate string : ("[ii]")*
     * 
     * @param d dimension
     * @return
     */
    private static String accessTable(Language language, int d) {
        int i = 0;
        String form = "";
        while (i<d){
            form+=LanguageTranslator.arrayAccess(language,"i"+i);
            i++;
        }
        return form;
    }


    /**
     * Generate string : "table"("[]")*
     * 
     * @param d dimension
     * @return
     */
    private static String tableLength(Language language, int d) {
    	String form = "";
    	int i = 0;
        while (i<d){
        	form+=LanguageTranslator.arrayAccess(language,"i"+i);
	        i++;
	    }
        return form;
    }


    /**
     * Generate string : ("\t")*
     * 
     * @param d dimension
     * @return
     */
    private static String indent(int d) {
        int i = 0;
        String ind = "";
        while (i<d){
            ind+="\t";
            i++;
        }
        return ind;
    }


    /**
     * Generate table dimension code.
     * 
     * @param dimension
     * @return the string of the dimension code
     */
    public static String getDimension(int dimension) {
        String dim="";
        
        while(dimension>0){ 
            dim+="[]";
            dimension--;
        }
        return dim;
    }
    
    /**
     * @param s
     * @return
     */
    public static String upCaseFirstLetter(String s) {
        char[] cTab = s.toCharArray();
        cTab[0] = (cTab[0]+"").toUpperCase().charAt(0);
        return new String(cTab);
    }

	private static String addStringEncodingSpecificMethods(Language language){  /*TODO: binaire*/
		String methods = "";
		
    	/****************************************/
    	/** Method addEncodingSpacer(Object o) **/
    	/****************************************/
    	
    	String[][] params = {{"Object","o"}};
    	methods+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"String","addEncodingSpacer",params,false,null)+"\n";
    	//methods+= "\t private static function addEncodingSpacer(o:Object):String {\n";

    	methods+= "\t\t"+LanguageTranslator.returnValue(language,"o+\""+Generator.FLASH_ENCODING_SPACER+"\"")+"\n";
    	//methods+= "\t\t return o+\""+ FLASH_ENCODING_SPACER +"\";\n";
    	methods+= "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
    	//methods+= "\t }\n\n";
    	
    	/****************************************/
    	/** Method getNextParam(String code)   **/
    	/****************************************/
    	
    	String[][] params3 = {{"String","code"}};
    	methods+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"String","getNextParam",params3,false,null)+"\n";
    	//methods+= "\t private static function getNextParam(code:String):String{\n";
    		
    	methods+= "\t\t"+LanguageTranslator.addLocalVar(language,true,"","String","param")+"\n";
    	//methods+= "\t\t var param:String = \"\";\n";
    		
    	String[] params4 = {"indexTreatment"};
    	String init = LanguageTranslator.callMethod(language,true,"code","charAt",params4);
    	methods+= "\t\t"+LanguageTranslator.addLocalVar(language,true,init,"char","c")+"\n";
    	//methods+= "\t\t var c = code.charAt(indexTreatment);\n\n";
    		
    	String[] itemsToEquals2 = {"c","\'"+Generator.FLASH_ENCODING_SPACER+"\'"};
    	String[] itemsToTest2 = {"indexTreatment", LanguageTranslator.stringLength(language,"code")};
    	String[] itemsToTest = {LanguageTranslator.testSup(language,false,itemsToTest2),
    								LanguageTranslator.testEqualType(language,false,"char", itemsToEquals2)};
    	String cond1 = LanguageTranslator.testAnd(language,true,itemsToTest);
    	methods+= "\t\t"+LanguageTranslator.whileLoopDeclaration(language,cond1)+"\n";
    	//methods+= "\t\t while ((indexTreatment < length(code)) && c !== \""+ FLASH_ENCODING_SPACER +"\" ){\n";
    		
    	String[] paramsToAdd = {"c"};
    	methods+= "\t\t\t"+LanguageTranslator.add(language,true,"String","param", paramsToAdd)+"\n";
    	//methods+= "\t\t\t param += c;\n";
    		
    	methods+= "\t\t\t"+LanguageTranslator.increment(language,true,"indexTreatment")+"\n";
    	//methods+= "\t\t\t indexTreatment++;\n";
    		
    	methods+= "\t\t\t"+LanguageTranslator.ifDeclaration(language, LanguageTranslator.testSup(language,false,itemsToTest2))+"\n";
    	methods+= "\t\t\t\t"+LanguageTranslator.affectValueToVar(language,"c",LanguageTranslator.callMethod(language,true,"code","charAt",params4))+"\n";
    	methods+= "\t\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	//methods+= "\t\t\t if (indexTreatment < length(code)) c = code.charAt(indexTreatment);\n";
    		
    	methods+= "\t\t\t"+LanguageTranslator.elseDeclaration(language,"\t\t\t\t"+LanguageTranslator.loopInterrupt(language)) + "\n";
    	//methods+= "\t\t\t else break;\n";
    		
    	methods+= "\t\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	//methods+= "\t\t }\n";
    	
    	methods+= "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	//methods+= "\t\t }\n";
    	
    	methods+= "\t\t"+LanguageTranslator.ifDeclaration(language, LanguageTranslator.testSup(language,false,itemsToTest2))+"\n";
    	methods+= "\t\t\t"+LanguageTranslator.increment(language,true,"indexTreatment")+LanguageTranslator.endOfLineMarker(language)+"\n";
    	//methods+= "\t\t if (indexTreatment < (length(code))) indexTreatment++; // \"pointeur\" apres |\n\n";
    		
    	methods+=  "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	methods+=  "\t\t"+LanguageTranslator.returnValue(language,"param")+"\n";
    	//methods+= "\t\t return param;\n";

    	methods+=  "\t"+LanguageTranslator.closeBlock(language)+"\n";
    	//methods+= "\t }\n";
    	
    	if (Generator.isThereABooleanAttribute){
    		
	    	/****************************************/
	    	/** Method decodeBoolean(Object o)     **/
	    	/****************************************/
	    	
	    	String[][] params1 = {{"String","s"}};
	    	methods+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"boolean","decodeBoolean",params1,false,null)+"\n";
	    	//methods+= "\t private static function decodeBoolean(s:String):Boolean {\n";
	    		
	    	String[] itemsToEquals = {"s","\"0\""};
	    	String cond = LanguageTranslator.testEqualType(language,true,"String",itemsToEquals);
	    	methods+= "\t\t"+LanguageTranslator.ifDeclaration(language,cond)+"\n";
	    	methods+= "\t\t\t"+LanguageTranslator.returnValue(language,"false")+"\n";
	    	//methods+= "\t\t if (s === \"0\") return false;\n";
	    		
	    	methods+= "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
	    	methods+= "\t\t"+LanguageTranslator.elseDeclaration(language,"\t\t\t"+LanguageTranslator.returnValue(language,"true"))+"\n";
	    	methods+= "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
	    	//methods+= "\t\t else return true;\n";
	    		
	    	methods+= "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
	    	//methods+= "\t }\n\n";
	    		
	    	
	    	/****************************************/
	    	/** Method encodeBoolean(Boolean b)    **/
	    	/****************************************/
	    	
	    	String[][] params2 = {{"boolean","b"}};
	    	methods+= "\t"+LanguageTranslator.writeFunctionDeclaration(language,false,Generator.isClient,false,false,"String","encodeBoolean",params2,false,null)+"\n";
	    	//methods+= "\t private static function encodeBoolean(b:Boolean):String {\n";
	    		
	    	methods+= "\t\t"+LanguageTranslator.ifDeclaration(language,"b")+"\n";
	    	methods+= "\t\t\t"+LanguageTranslator.returnValue(language,"\"1\"")+"\n";
	    	methods+= "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
	    	//methods+= "\t\t if (b.valueOf()) return \"1\";\n";
	    		
	    	methods+= "\t\t"+LanguageTranslator.elseDeclaration(language, "\t\t\t"+LanguageTranslator.returnValue(language,"\"0\""))+"\n";
	    	methods+= "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
	    	//methods+= "\t\t else return \"0\";\n";
	    	methods+= "\t"+LanguageTranslator.closeBlock(language)+"\n\n";
	    	//strings[8]+= "\t }\n\n"; 	
    	}
    	
    	return methods;
	}
	
	/*
	 * Whether the generation is made for Gasp or uGasp, and for client or server, 
	 * the package will not be the same inside the CustomTypes generated file. 
	 * For uGasp, the package name allows a proper compilation of the future OSGI bundle, but doesn't fit
	 * to the folder where it's generated.
	 * 
	 * @return String 	the String corresponding to the suitable package declaration.
	 */
	private static String writeSuitablePackageForJavaCustomTypes(Language language, String _package){
		String packageForJavaCustomType = "";
		if(Generator.isClient){
			// Client
			packageForJavaCustomType = LanguageTranslator.writePackage(language,_package+".client")+"\n";
		} else{
			// Server
			if(Generator.isGenerationForUGaspLogic()){
				// uGasp
				packageForJavaCustomType = LanguageTranslator.writePackage(language,PACKAGE_UGASP_CUSTOMTYPES)+"\n";
			}else{
				// Gasp
				packageForJavaCustomType = LanguageTranslator.writePackage(language,_package+".server")+"\n";
			}	
		}    
		return packageForJavaCustomType;
	}
}
