/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.mega.moods.generator.TypesGenerator;
import org.mega.moods.generator.LanguageTranslator;
import org.mega.moods.generator.LanguageTranslator.Language;
import org.w3._2001.xmlschema.Attribute;
import org.w3._2001.xmlschema.ComplexType;
import org.w3._2001.xmlschema.Element;
import org.w3._2001.xmlschema.ElementRef;
import org.w3._2001.xmlschema.SimpleType;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN, Lim Chanty
 */
public class Generator {

	static final String MOODS_VERSION = "2.1.0";
	static final String MOODS_YEAR = "2006-2011";
	static final String MOODS_TEAM = "uGASP project";
	static final short ENCODING_VERSION = 1200;
	static final String FLASH_ENCODING_SPACER = "|";

	// FIXME Only BINARY ENCODING is used actually, cause ENCODING is not used a parameter of the main()
	// 		 When both encoding will really be available, changes will be required in generateFile()
	static final int BINARY = 0;
	static final int STRING = 1;
	
	static boolean objectsApproachMode = false;
	static boolean hashtablesApproachMode = false;
	
	static boolean isClient = false;
	static boolean isThereABooleanAttribute = false;
	static boolean isTypesClassesAlreadyGenerated = false;
	static boolean isCompactMode = false;
	
	static int encoding = -1;
	static String destinationClassName = "";
	static String ioApproach = "";
	
	// communication models
	final static int SERVER = 0;
	final static int UNICAST = 1;
	final static int MULTICAST = 2;
	final static int BROADCAST = 3;

	private static Vector customTypes;
	private static Vector customSimpleTypes;
	private static final String hashtables = "hashtables";
	private static final String objects = "objects";
	
	private static String servletPrefixName = "";
	private static int indexType;
	
	private static boolean generateUGaspLogic = false;
	private static boolean generateServerLogic = false;
	private static boolean generateSqlLogic = false;
	private static boolean generateServletLogic = false;

	private static String xmlFile;
	private static String _package = "";
	private static String cppDirectory = "client/c++";
	private static String flashDirectory = "client/flash";
	private static String csharpDirectory = "client/c#";
	private static String cppIncludeDirectory = "../../../../moods-base/src/main/java/org/mega/moods/generator/cpp";
	private static String csharpIncludeDirectory = "../../../../moods-base/src/main/java/org/mega/moods/generator/csharp";

	/**
	 * @param args  For direct access without MHI args must be = {	xsdFileName, 
	 * 																packageName,
	 * 																isCompactMode("true"/"false"),
	 * 																clientLanguage,
	 * 																clientCommunicationApproach,
	 * 															 	serverCommunicationApproach
	 * 															 }  
	 */
	public static void main(String[] args) {

		boolean isPackaged = false;

		try{
			File cTypesFileClient = null;
			File cTypesFileServer = null;

			// launch command line MHI
			if (args.length == 0)
				args = doCommandLineMHI(args);

			if (args.length>=2){
				_package= args[1];
				
				if (!validatePackageName(_package)){
					System.out.println("Invalid package name, generation aborted!");
					return;
				}
				
				isPackaged = true;
			}

			if (isPackaged) {
				File dir = new File(_package);
				File dirClient = new File(_package + "/client");
				File dirServer = new File(_package + "/server");
				File sqlServer = new File(_package + "/sql");
				File servletServer = new File(_package + "/servlet");
				dir.mkdir();
				dirClient.mkdir();
				dirServer.mkdir();
				sqlServer.mkdir();
				servletServer.mkdir();

			} else{
				System.out.println("The package argument must be provided.");
				System.exit(-1);
			}

			// Gasp or db mode
			if (args.length>=3){
				if ("g".equalsIgnoreCase(args[2]) || "gasp".equalsIgnoreCase(args[2])) {
					System.out.println("generation settings for gasp online: sql off / servlet off");
					generateSqlLogic = false;
					generateServletLogic = false;
				} else if ("u".equalsIgnoreCase(args[2]) || "ugasp".equalsIgnoreCase(args[2])) {
					System.out.println("generation settings for uGasp : sql off / servlet off");
					generateUGaspLogic = true;
					generateSqlLogic = false;
					generateServletLogic = false;
				} else if ("d".equalsIgnoreCase(args[2]) || "db".equalsIgnoreCase(args[2])) {
					System.out.println("generation settings for gasp online: sql on / servlet off");
					generateSqlLogic = true;
					generateServletLogic = false;
				} else if ("s".equalsIgnoreCase(args[2]) || "servlet".equalsIgnoreCase(args[2])) {
					System.out.println("generation settings for gasp online: sql off / servlet on");
					generateSqlLogic = false;
					generateServletLogic = true;
				} else throw new IllegalArgumentException("Unrecognized mode for generation. Please choose a mode between \"ugasp\", \"gasp\" and \"db\".");
			}

			// Client language, initialized for JAVA to avoid compilation error
			Language language4Client = Language.JAVA;			
			if (args.length>=5 && !generateSqlLogic && !generateServletLogic){
				if (args[4].equals("java")) language4Client = Language.JAVA;
				else if (args[4].equals("flash")) language4Client = Language.FLASH;
				else if (args[4].equals("cpp")) language4Client = Language.CPP;
				else if (args[4].equals("csharp")) language4Client = Language.CSHARP;
				else throw new IllegalArgumentException("Unrecognized language for generation. Please choose a mode between \"java\", \"cpp\", \"csharp\" and \"flash\".");
			}

			// Generate files related to the user choice
			if (generateSqlLogic) {
				// Generate SQLStorageManager if the user choose the db mode
				generateFile(false, Language.JAVA, args);
				SqlStorageManagerGenerator.finalizeSqlStorage();		
				CustomTypesGenerator.finalizeCustomTypesClasses();
				ClassMaker.flushClasses();
			} 
			if (generateServletLogic) {
				generateFile(false, Language.JAVA,args);
				ServletGenerator.finalizeServlet();
				CustomTypesGenerator.finalizeCustomTypesClasses();
				ClassMaker.flushClasses();
			} 
			
			// Generate client/server logic for GASP mode
			
			// FIXME For instance, only BINARY encoding is available for java generation
			//       --> no compatibility with Flash is assured ...
			// TODO	Romain, is it possible to use the last arg (boolean) of Generator without UI
			//		to set encoding ?
			if (language4Client.equals(Language.JAVA)){
				encoding = BINARY;
			} 
			else if (language4Client.equals(Language.FLASH)){
				preparesFlashGeneration(cTypesFileClient, cTypesFileServer);
			}
			else if (language4Client.equals(Language.CPP)){
		    	preparesCppGeneration(cTypesFileClient);
			}
		    else if (language4Client.equals(Language.CSHARP)){
		    	preparesCsharpGeneration(cTypesFileClient);
		    } 

			// client files generation
			clientFilesGeneration(language4Client, cTypesFileClient, args);
			
			// server files generation
			serverFilesGeneration(args);
	}
	catch (Exception e){
		e.printStackTrace();
	}
} 

/*
 * Generates client's CustomTypes class and flush ClassMaker's hashtables
 * that contains class to generate to finish process and enable next generation
 */
private static void clientFilesGeneration(Language language, File cTypesFileClient, String[] args) throws IOException{
	System.out.println("File generation starts for client !");
	generateFile(true, language, /*cTypesFileClient,*/ args);
	CustomTypesGenerator.finalizeCustomTypesClasses();
	ClassMaker.flushClasses();
}

/*
 * Generates server's CustomTypes class and flush ClassMaker's hashtables
 * that contains class to generate to finish process and enable next generation
 */
private static void serverFilesGeneration(String[] args) throws IOException{
	System.out.println("File generation starts for server !");
	generateFile(false, Language.JAVA, args); 
	CustomTypesGenerator.finalizeCustomTypesClasses();
	ClassMaker.flushClasses();
}
/*
 * Creates files that will contain CustomTypes class for client and server.
 * Set encoding to use between BINARY and STRING
 * --> only java generation is available for both encoding.
 * FIXME If Binary Encoding == String, why doesn't we give appropriate name to CustomTypes (CustomTypes4StringEncoding) here ?
 */
private static void preparesJavaGeneration(File cTypesFileClient, File cTypesFileServer, boolean isBinaryEncoding){
	//cTypesFileClient = new File("");
	//cTypesFileServer = new File("");
	encoding = isBinaryEncoding ? BINARY : STRING;
}

/*
 * Creates : 
 * - directory where Flash files will be located
 * - files that will contain CustomTypes class for client and server.
 * Also sets encoding to STRING, only encoding available for Flash.
 */
private static void preparesFlashGeneration(File cTypesFileClient, File cTypesFileServer){
	File dirClientFlash = new File(_package + "/" + flashDirectory);
	dirClientFlash.mkdir();
	cTypesFileClient = new File(_package + "/client/flash/CustomTypesImpl.as");
	cTypesFileServer = new File(_package + "/server/CustomTypes4StringEncoding.java");
	encoding = STRING;
}

/*
 * Creates : 
 * - directory where C++ files will be located
 * - files that will contain CustomTypes class for client.
 * Also sets encoding to BINARY, only encoding available for C++.
 */
private static void preparesCppGeneration(File cTypesFileClient){
	File dirClientCpp = new File(_package + "/" + cppDirectory);
	dirClientCpp.mkdir();
	// name is set by ClassMaker.createClassEntry
	cTypesFileClient = new File("");
	encoding = BINARY;
}

/*
 * Creates : 
 * - directory where C# files will be located
 * - files that will contain CustomTypes class for client.
 * Also sets encoding to BINARY, only encoding available for C#.
 */
private static void preparesCsharpGeneration(File cTypesFileClient){
	File dirClientFlash = new File(_package + "/" + csharpDirectory);
	dirClientFlash.mkdir();
	cTypesFileClient = new File(_package + "/" + csharpDirectory+"/CustomTypesImpl.cs");
	encoding = BINARY;
}


private static String[] doCommandLineMHI(String[] args) {
	try { 
		// text interface
		System.out.println("MooDS Generator shell interface... "); 

		System.out.print("1 - Enter your XML Schema filename :  "); 

		// read xsd filename
		java.io.BufferedReader lis = new java.io.BufferedReader(new java.io.InputStreamReader(System.in) );
		args = new String[7];
		args[0] = lis.readLine();

		System.out.print("2 - Enter your application application package name :  "); 

		// read package name
		args[1] = lis.readLine();  // lit une ligne

		String s = "";
		args[3] = "false";

		System.out.print("3 - Do you want MOODS to choose the uGASP, the GASP mode or the Database persistence Manager? (u:ugasp/ g:gasp / d:db) :  "); 

		// read gasp or db approach 
		args[2] = lis.readLine(); 
		if ("d".equalsIgnoreCase(args[2]) || "db".equalsIgnoreCase(args[2])){
			// The user choose to use the persistence mode
			generateSqlLogic = true;
			generateServletLogic = false;

			// Initialize some arguments
			args[4] = "";
			args[5] = "";
			args[6] = "";

			System.out.print("3.1 - Enter the client communication approach (o:objects / h:hastables) :  "); 

			// read client communication approach
			s = lis.readLine(); 
			if (s.equals("o") || s.toLowerCase().equals("object") || s.toLowerCase().equals("objects")){
				args[5] = objects;
				args[6] = objects;

				System.out.print("3.1.1 - Do you want to use the compact object mode? (y:yes / n:no) :  "); 

				s = lis.readLine(); 
				if (s.equals("y") || s.toLowerCase().equals("yes")){
					args[3] = "true";
				}
				else if (s.equals("n") || s.toLowerCase().equals("no")){
					//FIXME really args[5] ?? why isnt' 3
					args[5] = "false";
				}
				else{
					System.out.println("Invalide user response. Exit.");
					System.exit(-1);
				}


			}
			else if (s.equals("h") || s.toLowerCase().equals("hashtable") || s.toLowerCase().equals("hashtables")){
				args[5] = hashtables;
				args[6] = hashtables;
			} else if ("s".equalsIgnoreCase(args[2]) || "servlet".equalsIgnoreCase(args[2])){
				// The user choose to use the persistence mode
				generateSqlLogic = false;
				generateServletLogic = true;
				args[4] = "";
				args[5] = "false";
				args[6] = "false";
			}
			else{
				System.out.println("Incorrect client communication approach. Exit.");
				System.exit(-1);
			}

			return args;
		}
		else if (args[2].equals("g") || args[2].toLowerCase().equals("gasp")){
			// The user choose the GASP mode
			generateSqlLogic = false;
			generateServletLogic = false;
		}
		else if (args[2].equals("u") || args[2].toLowerCase().equals("ugasp")){
			// The user choose the GASP mode
			generateUGaspLogic = true;
			generateSqlLogic = false;
			generateServletLogic = false;
		}
		else{
			System.out.println("Invalide user response. Exit.");
			System.exit(-1);
		}

		System.out.print("4.1 - Enter the client language (j:java / f:flash / c:cpp/ ) :  "); 

		// read client communication approach
		s = lis.readLine(); 
		if (s.equals("j") || s.toLowerCase().equals("java") ){
			args[4] = "java";
		}
		else if (s.equals("f") || s.toLowerCase().equals("flash")){
			args[4] = "flash";
		}
		else if (s.equals("c") || s.toLowerCase().equals("cpp")){
			args[4] = "cpp";
		}
		else if (s.equals("cs") || s.toLowerCase().equals("csharp")){
			args[4] = "csharp";
		}
		else{
			System.out.println("Invalid language selection. Exit.");
			System.exit(-1);
		}

		System.out.print("4.2.1 - Enter the client communication approach (o:objects / h:hastables) :  "); 

		// read client communication approach
		s = lis.readLine(); 
		if (s.equals("o") || s.toLowerCase().equals("object") || s.toLowerCase().equals("objects")){
			args[5] = objects;

			System.out.print("4.2.2 - Do you want to use the compact object mode? (y:yes / n:no) :  "); 

			s = lis.readLine(); 
			if (s.equals("y") || s.toLowerCase().equals("yes")){
				args[3] = "true";
			}
			else if (s.equals("n") || s.toLowerCase().equals("no")){
				args[5] = "false";
			}
			else{
				System.out.println("Invalide user response. Exit.");
				System.exit(-1);
			}


		}
		else if (s.equals("h") || s.toLowerCase().equals("hashtable") || s.toLowerCase().equals("hashtables")){
			args[5] = hashtables;
		}
		else{
			System.out.println("Incorrect client communication approach. Exit.");
			System.exit(-1);
		}

		System.out.print("5.1 - Enter the server communication approach (o:objects / h:hastables) :  "); 

		// read server communication approach
		s = lis.readLine(); 
		if (s.equals("o") || s.toLowerCase().equals("object") || s.toLowerCase().equals("objects")){
			args[6] = objects;
		}
		else if (s.equals("h") || s.toLowerCase().equals("hashtable") || s.toLowerCase().equals("hashtables")){
			args[6] = hashtables;
		}
		else{
			System.out.println("Incorrect server communication approach. Exit.");
			System.exit(-1);
		}

		System.out.print("5.2 - Do you want MOODS to generate your server game logic squeleton? (y:yes / n:no) :  "); 

		// read server communication approach 
		s = lis.readLine(); 
		if (s.equals("y") || s.toLowerCase().equals("yes")){
			generateServerLogic = true;
		}
		else if (s.equals("n") || s.toLowerCase().equals("no")){
			generateServerLogic = false;
		}
		else{
			System.out.println("Invalide user response. Exit.");
			System.exit(-1);
		}
		System.out.println("");
		return args;
	} 
	catch (java.io.IOException e) {
		System.err.println("Error with shell interface handled user responses."); 
		System.exit(-1); 
	} 
	return null;
}


/*
 * Read args and set corresponding global values to MooDS
 * @param args  the string array :
 * 					- read from the command line,
 * 					- used by ant (in moods-base/build.xml)
 * 					- used by mvn (in moods-communication-test-{LANGUAGE}/pom.xml)
 */
/*private static void initializeMooDSModes(String[] args) throws IOException {
	TypesGenerator.init();
	SqlStorageManagerGenerator.init();

	objectsApproachMode = false;
	hashtablesApproachMode = false;

	xmlFile = args[0];
	_package= args[1];

	
	String encodeMethods ="";
	String decodeMethods = "";

	//For the UGASP, GASP or DB mode
	if (args == null || args.length < 3 || args[2] == null) {
		System.out.println("The ugasp, gasp or db mode argument must be provided.");
		return;
	}

	//for "compact" mode
	// Get classname attribute from the command line main argument
	if (args == null || args.length < 4 || args[3] == null) {
		System.out.println("The compact mode (true or false) argument must be provided.");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || args.length < 5 || args[4] == null)) {
		System.out.println("The client language mode must be provided (j:java / f:flash / c:cpp/ a:all).");
		return;
	}

	if (generateServletLogic && (args == null || args.length < 5 || args[4] == null)) {
		System.out.println("The servlet prefix name must be provided.");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || args.length < 6 || args[5] == null)) {
		System.out.println("The client i/o approach mode must be provided (objects or hashtables).");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || (!isClient && args.length < 7) || (!isClient && args[6] == null))) {
		System.out.println("The server i/o approach mode must be provided (objects or hashtables).");
		return;
	}

	// input output approach mode
	isCompactMode = (Boolean.valueOf(args[3])).booleanValue();
	ioApproach = "";

	if (isClient){
		ioApproach = args[5];
		System.out.println("Client approach mode selected: "+ ioApproach);
	}
	else if (!generateServletLogic && !generateSqlLogic) {
		ioApproach = args[6];
		System.out.println("Server approach mode selected: "+ ioApproach);

		if (args.length == 8) {
			generateServerLogic = Boolean.valueOf(args[7]).booleanValue();
		}
	}
	else if (generateSqlLogic) ioApproach = args[4];

	if (ioApproach.toLowerCase().equals("objects")){
		objectsApproachMode = true;
		hashtablesApproachMode = false;
		ioApproach = objects;
	}
	else if (ioApproach.toLowerCase().equals("hashtables")){
		objectsApproachMode = false;
		hashtablesApproachMode = true;
		ioApproach = hashtables;
	}
	//else if (ioApproach.toLowerCase().equals("values")) valuesApproachMode = true;
	//else if (ioApproach.toLowerCase().equals("xml") xmlApproachMode = true;
	else if (!generateServletLogic && !generateSqlLogic){
		System.out.println("Invalid approach mode specified, switch to default (objects).");
		objectsApproachMode = true;
		ioApproach = objects;
	}

	if (generateServletLogic) {
		servletPrefixName = args[4];
	}

	// Generator initialization
	if (objectsApproachMode) {
		TypesGenerator.init();
	} 

	if (generateSqlLogic) {
		SqlStorageManagerGenerator.init();
	}

	if (generateServletLogic) {
		ServletGenerator.init();
		ServletGenerator.startGenerateServlet(JAVA, _package, upCaseFirstLetter(servletPrefixName));
	}

	customTypes = new Vector();
	customSimpleTypes = new Vector();
}

private static void prepareFilesToInclude(boolean isClient, int language){
	Generator.isClient = isClient;
	
	// for hashtables approach in flash client, copying Hashtable.as to package
	if (isClient && (language.equals(Language.FLASH) )){
		File flashHashtableClass = new File("src/org/mega/moods/generator/flash/Hashtable.as");
		File flashHashtableClassToCopy = new File(_package+"/client/flash/Hashtable.as");
		copyFile(flashHashtableClass,flashHashtableClassToCopy);
	}    
	
	// for c++: includes SocketStream (.h, .cpp & exception) & Object.h
	if (isClient && (language.equals(Language.CPP) )){
		File fileToIncludeForCpp  = new File(cppIncludeDirectory+"/Object.h");
		File fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/Object.h");
		copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
		
		fileToIncludeForCpp  = new File(cppIncludeDirectory+"/SocketException.h");
		fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketException.h");
		copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
		
		fileToIncludeForCpp = new File(cppIncludeDirectory+"/SocketStream.h");
		fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketStream.h");
		copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
		
		fileToIncludeForCpp = new File(cppIncludeDirectory+"/SocketStream.cpp");
		fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketStream.cpp");
		copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
	}    
	
	// for c#: includes big endian encoding files
	if (isClient && (language.equals(Language.CSHARP) )){
		File fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/BigEndianBitConverter.cs");
		File fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/BigEndianBitConverter.cs");
		copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
		
		fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/DoubleConverter.cs");
		fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/DoubleConverter.cs");
		copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
		
		fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBinaryReader.cs");
		fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBinaryReader.cs");
		copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
		
		fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBinaryWriter.cs");
		fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBinaryWriter.cs");
		copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
		
		fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBitConverter.cs");
		fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBitConverter.cs");
		copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
	}    

}

private static void parseXSDDescriptionFile(){
	// get the types instance of the xml file
	TypesJAXBReader types = new TypesJAXBReader(xmlFile);

	// Get the classname from the XML schema grammar given by the game designer
	// and override the command line argument if exists
	destinationClassName = types.getOnElementStartFromName("Message");
	if (destinationClassName == null || "".equals(destinationClassName)) {
		destinationClassName = "";
	}

	// generate the server game logic squeleton
	if (generateServerLogic & !isClient) ServerGenerator.startGenerateServer(language, _package);

	// Get the message element under the root node from the given grammar file.
	// For the moment, the root node must be fixed as the messages under it.
	//List messages = types.getMessageElements();
	Vector messageTypes = new Vector();

	List messages = types.getMessageElements();
	for(Iterator iter0 = messages.iterator(); iter0.hasNext();){
		Element e = (Element) iter0.next();
		messageTypes.addElement(upCaseFirstLetter(e.getType().substring(4)));
		System.out.println("Message type:"+ upCaseFirstLetter(e.getType().substring(4)));
	}

	List complexTypes = types.getComplexTypes();
	complexTypes.remove(types.getComplexTypeFromName("Root"));
	complexTypes.remove(types.getComplexTypeFromName("Message"));

	for(Iterator iter = complexTypes.iterator(); iter.hasNext();){
		ComplexType type = (ComplexType) iter.next();
		customTypes.addElement(type.getName());
		System.out.println("Complex type:"+ upCaseFirstLetter(type.getName()));
	}

	List simpleTypes = types.getSimpleTypes();
	for(Iterator iter = simpleTypes.iterator(); iter.hasNext();){
		SimpleType type = (SimpleType) iter.next();
		customSimpleTypes.addElement(type.getName());
		System.out.println("Simple type:"+ upCaseFirstLetter(type.getName()));
	}
}
*/

private static void generateFile(boolean isClient, Language language, String[] args) throws IOException {
	/****************************************  READ ARGS       **********************************************************************/
	
	TypesGenerator.init();
	SqlStorageManagerGenerator.init();

	objectsApproachMode = false;
	hashtablesApproachMode = false;
	
	Generator.isClient  = isClient;

	xmlFile = args[0];
	_package= args[1];


	TypesJAXBReader types;
	String encodeMethods ="";
	String decodeMethods = "";

	//For the UGASP, GASP or DB mode
	if (args == null || args.length < 3 || args[2] == null) {
		System.out.println("The ugasp, gasp or db mode argument must be provided.");
		return;
	}
	
	ServerGenerator.init(
			generateUGaspLogic? ServerGenerator.SERVER_UGASP: ServerGenerator.SERVER_GASPONLINE
	);
		
	//for "compact" mode
	// Get classname attribute from the command line main argument
	if (args == null || args.length < 4 || args[3] == null) {
		System.out.println("The compact mode (true or false) argument must be provided.");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || args.length < 5 || args[4] == null)) {
		System.out.println("The client language mode must be provided (j:java / f:flash / c:cpp/ cs:csharp).");
		return;
	}

	if (generateServletLogic && (args == null || args.length < 5 || args[4] == null)) {
		System.out.println("The servlet prefix name must be provided.");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || args.length < 6 || args[5] == null)) {
		System.out.println("The client i/o approach mode must be provided (objects or hashtables).");
		return;
	}

	if (!generateServletLogic && !generateSqlLogic && (args == null || (!isClient && args.length < 7) || (!isClient && args[6] == null))) {
		System.out.println("The server i/o approach mode must be provided (objects or hashtables).");
		return;
	}

	// input output approach mode
	isCompactMode = (Boolean.valueOf(args[3])).booleanValue();
	ioApproach = "";

	if (isClient){
		ioApproach = args[5];
		System.out.println("Client approach mode selected: "+ ioApproach);
	}
	else if (!generateServletLogic && !generateSqlLogic) {
		ioApproach = args[6];
		System.out.println("Server approach mode selected: "+ ioApproach);

		if (args.length == 8) {
			generateServerLogic = Boolean.valueOf(args[7]).booleanValue();
		}
	}
	else if (generateSqlLogic) ioApproach = args[4];

	if (ioApproach.toLowerCase().equals("objects")){
		objectsApproachMode = true;
		hashtablesApproachMode = false;
		ioApproach = objects;
	}
	else if (ioApproach.toLowerCase().equals("hashtables")){
		objectsApproachMode = false;
		hashtablesApproachMode = true;
		ioApproach = hashtables;
	}
	//else if (ioApproach.toLowerCase().equals("values")) valuesApproachMode = true;
	//else if (ioApproach.toLowerCase().equals("xml") xmlApproachMode = true;
	else if (!generateServletLogic && !generateSqlLogic){
		System.out.println("Invalid approach mode specified, switch to default (objects).");
		objectsApproachMode = true;
		ioApproach = objects;
	}

	if (generateServletLogic) {
		servletPrefixName = args[4];
	}

	// Generator initialization
	if (objectsApproachMode) {
		TypesGenerator.init();
	} 

	if (generateSqlLogic) {
		SqlStorageManagerGenerator.init();
	}

	if (generateServletLogic) {
		ServletGenerator.init();
		ServletGenerator.startGenerateServlet(_package, upCaseFirstLetter(servletPrefixName));
	}

	customTypes = new Vector();
	customSimpleTypes = new Vector();
	
	
	
	/**********************************************  PREPARE FILES ********************************************************************************/

	try {
		// for hashtables approach in flash client, copying Hashtable.as to package
		if (isClient && (language.equals(Language.FLASH) )){
			File flashHashtableClass = new File("src/org/mega/moods/generator/flash/Hashtable.as");
			File flashHashtableClassToCopy = new File(_package+"/client/flash/Hashtable.as");
			copyFile(flashHashtableClass,flashHashtableClassToCopy);
		}    
		
		// for c++: includes SocketStream (.h, .cpp & exception) & Object.h
		if (isClient && (language.equals(Language.CPP) )){
			File fileToIncludeForCpp  = new File(cppIncludeDirectory+"/Object.h");
			File fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/Object.h");
			copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
			
			fileToIncludeForCpp  = new File(cppIncludeDirectory+"/SocketException.h");
			fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketException.h");
			copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
			
			fileToIncludeForCpp = new File(cppIncludeDirectory+"/SocketStream.h");
			fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketStream.h");
			copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
			
			fileToIncludeForCpp = new File(cppIncludeDirectory+"/SocketStream.cpp");
			fileToIncludeForCppToCopy = new File(_package+"/"+cppDirectory+"/SocketStream.cpp");
			copyFile(fileToIncludeForCpp,fileToIncludeForCppToCopy);
		}    
		
		// for c#: includes big endian encoding files
		if (isClient && (language.equals(Language.CSHARP) )){
			File fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/BigEndianBitConverter.cs");
			File fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/BigEndianBitConverter.cs");
			copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
			
			fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/DoubleConverter.cs");
			fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/DoubleConverter.cs");
			copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
			
			fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBinaryReader.cs");
			fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBinaryReader.cs");
			copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
			
			fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBinaryWriter.cs");
			fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBinaryWriter.cs");
			copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
			
			fileToIncludeForCsharp = new File(csharpIncludeDirectory+"/EndianBitConverter.cs");
			fileToIncludeForCsharpToCopy = new File(_package+"/"+csharpDirectory+"/EndianBitConverter.cs");
			copyFile(fileToIncludeForCsharp,fileToIncludeForCsharpToCopy);
		}    
		

		/********************************************  PARSE XML DESCRIPTION FILE**********************************************************************************/
		
		
		// get the types instance of the xml file
		types = new TypesJAXBReader(xmlFile);

		// Get the classname from the XML schema grammar given by the game designer
		// and override the command line argument if exists
		destinationClassName = types.getOnElementStartFromName("Message");
		if (destinationClassName == null || "".equals(destinationClassName)) {
			destinationClassName = "";
		}

		// generate the server game logic squeleton
		if (generateServerLogic & !isClient) ServerGenerator.startGenerateServer(_package);

		// Get the message element under the root node from the given grammar file.
		// For the moment, the root node must be fixed as the messages under it.
		//List messages = types.getMessageElements();
		Vector messageTypes = new Vector();

		List messages = types.getMessageElements();
		for(Iterator iter0 = messages.iterator(); iter0.hasNext();){
			Element e = (Element) iter0.next();
			messageTypes.addElement(upCaseFirstLetter(e.getType().substring(4)));
			System.out.println("Message type:"+ upCaseFirstLetter(e.getType().substring(4)));
		}

		List complexTypes = types.getComplexTypes();
		complexTypes.remove(types.getComplexTypeFromName("Root"));
		complexTypes.remove(types.getComplexTypeFromName("Message"));

		for(Iterator iter = complexTypes.iterator(); iter.hasNext();){
			ComplexType type = (ComplexType) iter.next();
			customTypes.addElement(type.getName());
			System.out.println("Complex type:"+ upCaseFirstLetter(type.getName()));
		}

		List simpleTypes = types.getSimpleTypes();
		for(Iterator iter = simpleTypes.iterator(); iter.hasNext();){
			SimpleType type = (SimpleType) iter.next();
			customSimpleTypes.addElement(type.getName());
			System.out.println("Simple type:"+ upCaseFirstLetter(type.getName()));
		}
		
		/***********************************************************************************************************************/

		// CustomTypes generation: start generate class, start to build encodeData and decodeData root methods
		if (!generateSqlLogic && !generateServletLogic) {
			System.out.println("CustomTypes generation turned on");
			CustomTypesGenerator.startGenerateCustomTypes(language,generateUGaspLogic,encoding,_package);
		}

		// SqlStorageManager generation: class declaration
		if (generateSqlLogic) {
			System.out.println("SQL generation turned on");
			SqlStorageManagerGenerator.startGenerateSqlStorageManager(_package, hashtablesApproachMode);
		}

		if (generateServletLogic) {
			System.out.println("Servlet generation turned on");
			ServletGenerator.generateDoPostMethod(_package, types);
		}

		/************************************************************/
		/**    start to treat custom types described in xsd file   **/
		/************************************************************/

		indexType = 0;
		boolean isFirstIf = true;
		if (!generateSqlLogic && !generateServletLogic && objectsApproachMode && isCompactMode){
			// prepare mono class for compact generation
			if (!isTypesClassesAlreadyGenerated){
				System.out.println("!!!!! in isTypesClassesAlreadyGenerated: language "+language);
				TypesGenerator.startGenerateClass(language,encoding,_package, destinationClassName, currentTypeCommunicationModel);
				TypesGenerator.addClassAttribute(language,false,0,true,destinationClassName,"type","byte");
			}
			if (generateServerLogic && !isClient){
				ServerGenerator.addImport(_package+"."+destinationClassName);
				/**possible merge error ex TODO:ServerGenerator.addCustomTypeTreatment(language,destinationClassName);*/
			}
		}

		// To not generate twice the same method for several same 
		// types presence in the choice and element.
		Hashtable singleElementHash = new Hashtable();

		for(Iterator iter = complexTypes.iterator(); iter.hasNext();){
			// Get the ComplexType
			//Element elem = (Element) iter.next();
			//ComplexType type = types.getComplexTypeFromName(elem.getType());
			ComplexType ctype = (ComplexType) iter.next();

			boolean isAMessage = messageTypes.contains(ctype.getName());

			// Get the ComplexType attribute
			List elements = ctype.getAttribute();
			//List tables = type.getTable();
			String realElementName = ctype.getName();
			String typeName = upCaseFirstLetter(ctype.getName());

			// Set the communication model for this type
			setCommunicationModelForElement(ctype);
			// Get type name
			//String typeName = type.getName();
			// Get the classname from the XML schema
			String typeClassName = typeName;  

			if (!singleElementHash.contains(typeName)) {
				singleElementHash.put(typeName, typeName);
			} else continue;

			if (typeName != null && !"".equals(typeName)) {
				String typeNameTmp = types.getOnElementStartFromName(typeName);
				if (typeNameTmp != null && !"".equals(typeNameTmp)) typeClassName = typeNameTmp; 
			}	                	

			// prepare the class file for the current custom type
			if (!generateSqlLogic && !generateServletLogic && objectsApproachMode && !isCompactMode && !isTypesClassesAlreadyGenerated){
				TypesGenerator.startGenerateClass(language,encoding,_package, typeClassName, currentTypeCommunicationModel);
				// add recipient or recipients attribute if this type is an unicast or a multicast
				if (currentTypeCommunicationModel == Generator.UNICAST){
					TypesGenerator.addClassAttribute(language,false,0,isCompactMode,typeClassName,"recipient","short");
				}
				else if (currentTypeCommunicationModel == Generator.MULTICAST){
					TypesGenerator.addClassAttribute(language,false,0,isCompactMode,typeClassName,"recipients",LanguageTranslator.arrayDeclaration(language,"short",1));
				}
			}
			if (generateServerLogic && !isClient){
				if (objectsApproachMode) ServerGenerator.addImport(_package+"."+typeClassName);
				ServerGenerator.addCustomTypeTreatment(language,encoding,typeClassName,currentTypeCommunicationModel);
			}

			String typeNameFinal = "TYPE_"+typeName.toUpperCase();

			if (!generateSqlLogic && !generateServletLogic) {

				if (isAMessage){
					// CustomTypes generation: add global var --> static for Gasp, not for uGasp
					if(generateUGaspLogic && !isClient){
						CustomTypesGenerator.addGlobalVar(language, false, ""+indexType, "byte", typeNameFinal); 
					}else{
						CustomTypesGenerator.addGlobalVar(language, true, ""+indexType, "byte", typeNameFinal);
					}
				}

				/************************************************************/
				/**    add treatments in root method encodeData for the    **/
				/**               current custom type					   **/
				/************************************************************/

				// CustomTypes generation: continue to build encodeData method
				if (isAMessage) CustomTypesGenerator.continueToWriteEncodeDataMethod(language,encoding,typeNameFinal,typeName,typeClassName);

				/************************************************************/
				/**    add treatments in root method decodeData for the    **/
				/**               current custom type					   **/
				/************************************************************/

				// CustomTypes generation: continue to build decodeData method
				if (isAMessage) CustomTypesGenerator.continueToWriteDecodeDataMethod(language,encoding,indexType,isFirstIf,typeNameFinal,typeName,typeClassName);


				/*************************************************************/
				/**  start to generate encoding method for the current type **/
				/*************************************************************/
				CustomTypesGenerator.startToWriteEncodingMethods(language,encoding,typeClassName,typeName,destinationClassName,currentTypeCommunicationModel);


				/*************************************************************/
				/**  start to generate decoding method for the current type **/
				/*************************************************************/
				CustomTypesGenerator.startToWriteDecodingMethods(language,encoding,typeClassName,typeNameFinal,typeName,destinationClassName,currentTypeCommunicationModel, isAMessage);
			}

			if (generateSqlLogic && SqlStorageManagerGenerator.canGenerate(ctype)) {
				SqlStorageManagerGenerator.generateRequestMethodStart(ctype, types);
				SqlStorageManagerGenerator.generateSqlBodyStart(ctype, typeClassName, 1, indexType, types);		        	
				SqlStorageManagerGenerator.generateElementHashBodyAttributeStart(types, ctype, typeClassName, "", 1, indexType, "", true);

			}      

			/****************************************************************/
			/**  continue to generate encoding and decoding methods for    **/
			/**             the current type by reading attributes         **/
			/****************************************************************/

		int tableNumber = 1;
		if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CPP)  || language.equals(Language.CSHARP) ){
			boolean hasSequence = (ctype.getSequence()!= null);

			// generate encoder/decoder methods for simple type elements
			for(Iterator iterElements = elements.iterator(); iterElements.hasNext();){
				Attribute element = (Attribute) iterElements.next();
				String elementName = element.getName();
				if (objectsApproachMode) elementName = upCaseFirstLetter(elementName);
				String elementType = element.getType();
				String nameOftype = "";

				if (!generateSqlLogic && !generateServletLogic) {
					if (customTypes.contains(elementType.substring(elementType.indexOf(":")+1))){ 
						CustomTypesGenerator.continueToWriteEncodingMethods(language,encoding,elementType,elementName,false);
						CustomTypesGenerator.continueToWriteDecodingMethods(language,encoding,elementType,elementName,false);
						if (objectsApproachMode && !isTypesClassesAlreadyGenerated)
							TypesGenerator.addClassAttribute(language,false,0,isCompactMode, (isCompactMode? destinationClassName:typeName), elementName, elementType);
					}
					else if (customSimpleTypes.contains(elementType.substring(elementType.indexOf(":")+1))){
						SimpleType simpleType = types.getSimpleTypeFromName(elementType);

						// list tag handling
						if (simpleType.getList()!=null){
							String type = simpleType.getList().getItemType();
							String tableType = type.substring(type.indexOf(":")+1, type.length());
							handleArray(customTypes.contains(tableType),tableType,typeName,lowCaseFirstLetter(elementName),tableNumber,hasSequence,language);
							tableNumber++;
						}

						//TODO: restriction tag handling
					}
					else {
						nameOftype = LanguageTranslator.getType(language,elementType);
						if (nameOftype.equals("boolean") || nameOftype.equals("Boolean")) Generator.isThereABooleanAttribute = true;

						if (objectsApproachMode && !isTypesClassesAlreadyGenerated){
							TypesGenerator.addClassAttribute(language,false,0,isCompactMode, (isCompactMode? destinationClassName:typeName), elementName, elementType);
						}
						// For C++, encoding and decoding methods are in their classes
						if(language.equals(Language.JAVA)  || language.equals(Language.CPP)   || language.equals(Language.CSHARP) ){
							CustomTypesGenerator.generateEncodingType(language,encoding,elementName,nameOftype);
							CustomTypesGenerator.generateDecodingType(language,encoding,elementName,nameOftype);  						
						}
					}
				}

				if (generateSqlLogic && SqlStorageManagerGenerator.canGenerate(ctype)) {
					SqlStorageManagerGenerator.generateElementHashBodyAttribute(types, ctype, realElementName, typeClassName, "", element, 1, indexType, "", true);
				}
			}

			if (generateSqlLogic && SqlStorageManagerGenerator.canGenerate(ctype)) {
				SqlStorageManagerGenerator.generateElementHashBodyAttributeEnd(types, ctype, "", elements, 1, indexType, "", true);
			}

		}

			encodeMethods+="";
			decodeMethods+="";     

			// generate encoder/decoder methods for sequence types
			if (ctype.getSequence() != null && !ctype.getSequence().getElement().isEmpty()) {
				Iterator sequenceElements = ctype.getSequence().getElement().iterator();
				ElementRef ref = null;
				while (sequenceElements.hasNext()) {
					ref = (ElementRef)sequenceElements.next();

					Element elem = types.getElementFromName(ref.getRef());
					String name = elem.getName();
					String type = elem.getType();
					String tableName = name.substring(name.indexOf(":")+1, name.length());
					String tableType = upCaseFirstLetter(type.substring(type.indexOf(":")+1, type.length()));

					if (ref.getMaxOccurs() == null){
						CustomTypesGenerator.continueToWriteEncodingMethods(language,encoding,tableType,tableName,true);
						CustomTypesGenerator.continueToWriteDecodingMethods(language,encoding,tableType,tableName,true);
						if (objectsApproachMode && !isTypesClassesAlreadyGenerated)
							TypesGenerator.addClassAttribute(language,false,0,isCompactMode, (isCompactMode? destinationClassName:typeName), tableName, tableType);
					}
					else{
						handleArray(customTypes.contains(tableType),tableType,typeName,tableName,tableNumber,!sequenceElements.hasNext(),language);	
						
						if (generateSqlLogic && SqlStorageManagerGenerator.canGenerate(ctype)) {
							SqlStorageManagerGenerator.generateElementHashBodySequenceBody(types, ctype, "", "", ref, 1, indexType, "", true);
						}

						//tableNumber++;
					}
				}
			} else {
				// if there is no sequence
				if (generateSqlLogic && SqlStorageManagerGenerator.canGenerate(ctype)) {
					SqlStorageManagerGenerator.generateElementHashBodySequenceBodyEnd(types, ctype, "", 1, indexType, "", true);
				}                   
			}

			if (!generateSqlLogic && !generateServletLogic && (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) )) {                
				CustomTypesGenerator.finishAnEncodingMethods(language);
				CustomTypesGenerator.finishADecodingMethods(language);
			}                              
			// increment indexType
			if (isAMessage) indexType++;
		}

		if (!generateSqlLogic && !generateServletLogic) {
			CustomTypesGenerator.finishEncodeData(language,encoding);
			CustomTypesGenerator.finishDecodeData(language,encoding);
		} else if (generateSqlLogic) {
			SqlStorageManagerGenerator.generateSqlEnd(types, 1, "");
		}               

	} 
	catch (FileNotFoundException e) {
		e.printStackTrace();
	}
	catch (IOException e) {
		e.printStackTrace();
	}
	catch (Exception e) {
		e.printStackTrace();
	}
	finally{
		if (!generateSqlLogic && !generateServletLogic && objectsApproachMode && ((language.equals(Language.FLASH) ) || (language.equals(Language.CPP) ) || (language.equals(Language.CSHARP) ) || (language.equals(Language.JAVA)  && !isTypesClassesAlreadyGenerated))){
			TypesGenerator.finalizeTypes(language);
			if (language.equals(Language.JAVA) ) isTypesClassesAlreadyGenerated = true;
		}
		
		if (generateServerLogic && !isClient){
			ServerGenerator.finalizeServer();
			//** MOD ServerGenerator.flushClasses(language);
		}
	}
}


private static void handleArray(boolean isACustomTypeArray,String tableType, String typeName, String tableName, int tableNumber, boolean b, Language language) throws IOException {
	int dimension = 1;   
	if (!isACustomTypeArray) tableType = LanguageTranslator.getType(language,tableType);

	if (!generateSqlLogic && !generateServletLogic && objectsApproachMode && !isTypesClassesAlreadyGenerated){ 
		TypesGenerator.addClassAttribute(language,
				true,
				dimension,
				isCompactMode,
				(isCompactMode? destinationClassName:typeName),
				tableName,
				(isCompactMode? destinationClassName:tableType)
		);
	}
	if (!generateSqlLogic && !generateServletLogic) {
		CustomTypesGenerator.generateEncodingArrayType(language,encoding,isACustomTypeArray,dimension,tableNumber,tableName,tableType,b);
		CustomTypesGenerator.generateDecodingArrayType(language,encoding,isACustomTypeArray,dimension,tableNumber,tableName,tableType,b);
	}

	tableNumber++;
}

/**
 * Generate the decoding code for a table.
 * 
 * @param n table number n
 * @param table
 * @param tableName
 * @param tableType
 */
private static String generateGenericTableEncoding(Language language,int n, /*String tableName,*/ String type, int dimension) {
	/** possible merge error**///String t = type.toLowerCase();
	String toEncoder="";
	String nTable= "table"+n;
	String nTableL= "table"+n+"_l";



	if (objectsApproachMode){ 
		toEncoder += "\t\t"+LanguageTranslator.addLocalVar(
				language,
				true,
				LanguageTranslator.callMethod(language,true,"customType","get"+upCaseFirstLetter(type),null),
				LanguageTranslator.arrayDeclaration(language,(isCompactMode? destinationClassName:type),dimension), 
				""+nTable
		)+"\n";
		//toEncoder+="\t\t var "+nTable+":Array = customType.get"+upCaseFirstLetter(type)+"();\n";
	}
	else if (hashtablesApproachMode){
		String[] params = {"\""+type+"\""};
		String arrayDeclaration = LanguageTranslator.arrayDeclaration(language,"Hashtable",dimension);    
		toEncoder += "\t\t"+LanguageTranslator.addLocalVar(language,
				true,
				LanguageTranslator.cast(language,arrayDeclaration,LanguageTranslator.callMethod(language,true,"h","get",params)),
				arrayDeclaration, 
				""+nTable
		)+"\n";
		//toEncoder+="\t\t var "+nTable+":Array = h.get(\""+type+"\");\n";
	}

	int d = 0;

	while(d<dimension){
		String ind = indent(d);

		//String[] params = {"\""+type+"\""};
		toEncoder += "\t\t"+ind+LanguageTranslator.addLocalVar(language,
				true,
				LanguageTranslator.arrayLength(language,nTable+tableLength(language,d)),
				"int", 
				nTableL+d
		)+"\n";
		//toEncoder+="\t\t"+ind+" var "+nTableL+d+" = "+nTable+""+tableLength(d)+".length;\n";

		if (encoding == STRING){
			String[] params1 = {"\"\"+"+nTableL+d};
			String[] itemsToAdd = {LanguageTranslator.callMethod(language,false,"","addEncodingSpacer", params1)};
			toEncoder += "\t\t"+ind+LanguageTranslator.add(language,
					true,
					"String",
					"encoding",
					itemsToAdd)+"\n";  
			//toEncoder+="\t\t"+ind+" encoding += addEncodingSpacer("+nTableL+d+");\n";
		}
		else if (encoding == BINARY){
			toEncoder+="\t\t"+LanguageTranslator.writeBinaryType(language,"short",LanguageTranslator.castPrimitive(language,"short",""+nTableL+d),"dos")+"\n";
			//toEncoder+="\t\t"+ind+" dos.writeShort((short) "+nTableL+d+");\n";
		}

		toEncoder += "\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+nTableL+d)+"\n";
		//toEncoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";
		d++;
	}
	if (customTypes.contains(upCaseFirstLetter(type))){
		if (encoding ==  STRING){
			String[] params = {nTable+accessTable(language,dimension)};
			toEncoder += "\t\t"+indent(dimension)+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
			//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"("+nTable+accessTable(dimension)+");\n";
		}
		else if (encoding == BINARY){
			String[] params = {"dos",nTable+accessTable(language,dimension)};
			toEncoder += "\t\t"+indent(dimension)+LanguageTranslator.callMethod(language,false,"","encode"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
			//toEncoder+="\t\t"+indent(dimension)+" encode"+upCaseFirstLetter(type)+"(dos,"+nTable+accessTable(dimension)+");\n";
		}

		/*TODO: */

	}
	else{
		if (encoding == STRING){	
			toEncoder+="\t\t"+indent(dimension)+" " +
			"encoding += addEncodingSpacer("+nTable+accessTable(language,dimension)+");\n"; 
		} else if (encoding == BINARY){
			toEncoder+= "\t\t"+LanguageTranslator.writeBinaryType(language,type,""+nTable+accessTable(language,dimension),"dos")+"\n";
			/*toEncoder+="\t\t"+indent(dimension)+" dos.write";
	 	        if (type.equals("String")) toEncoder+="UTF";
	 	        else toEncoder+=type;
	 	        toEncoder+="("+nTable+accessTable(dimension)+");\n";*/
		}
	}

	toEncoder+= closeQuote(language, dimension);

	return toEncoder;        
}


/**
 * Generate the decoding code for a table.
 * 
 * @param table
 * @param tableName
 * @param tableType
 */
private static String generateGenericTableDecoding(Language language,int n, String tableName, String type, int dimension) {
	String t = type.toLowerCase();
	String toDecoder="";
	String nTable= "table"+n;
	String nTableL= "table"+n+"_l";

	boolean noLowerCase = false;
	if (type.equals("String")||customTypes.contains(upCaseFirstLetter(type))) noLowerCase = true;

	int d = 0;

	while(d<dimension){
		String ind = indent(d);
		if (encoding == STRING){
			String[] params = {"encodedString"};
			String parse = LanguageTranslator.callMethod(language,false,"","getNextParam",params);
			toDecoder+= "\t\t"+ind+LanguageTranslator.addLocalVar(
					language,
					true,
					LanguageTranslator.parseStringType(language,"int",parse),
					"int",
					nTableL+d
			)+"\n";  
			//toDecoder+="\t\t"+ind+" var "+nTableL+d+" = parseInt(getNextParam(encodedString));\n";
		} else if (encoding == BINARY){
			toDecoder+= "\t\t"+ind+LanguageTranslator.addLocalVar(
					language,
					true,
					LanguageTranslator.readBinaryType(language,type,"dis"),
					"short",
					nTableL+d
			)+"\n";  
		}



		String init = "";
		if (objectsApproachMode){ 
			if (noLowerCase) init = LanguageTranslator.arrayInit(language,(isCompactMode? destinationClassName:type),dimension,nTableL+d);
			else init = LanguageTranslator.arrayInit(language,t,dimension,nTableL+d);
			String init1 = LanguageTranslator.getAttributesInitiator(language,init);

			toDecoder+= "\t\t"+ind+LanguageTranslator.addLocalVar(
					language,
					true,
					init1,
					LanguageTranslator.arrayDeclaration(language,(isCompactMode? destinationClassName:type),dimension)
					,""+nTable)+"\n";
			//toDecoder+="\t\t"+ind+" var "+nTable+":Array";
			//if (noLowerCase)
			//	toDecoder+=initTable(nTableL,dimension,d,type);
			//else toDecoder+=initTable(nTableL,dimension,d,t);
			//toDecoder+=";\n";

			toDecoder+= "\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+(nTableL+d))+"\n";
			//toDecoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";
		}
		if (hashtablesApproachMode){
			init = LanguageTranslator.arrayInit(language,"Hashtable",dimension,nTableL+d);
			String init1 = LanguageTranslator.getAttributesInitiator(language,init);

			toDecoder+= "\t\t"+ind+LanguageTranslator.addLocalVar(
					language,
					true,
					init1,
					LanguageTranslator.arrayDeclaration(language,"Hashtable",dimension)
					,""+nTable)+"\n";
			//toDecoder+="\t\t"+ind+" var "+nTable+":Array";
			//if (noLowerCase)
			//	toDecoder+=initTable(nTableL,dimension,d,type);
			//else toDecoder+=initTable(nTableL,dimension,d,t);
			//toDecoder+=";\n";

			toDecoder+= "\t\t"+ind+LanguageTranslator.forLoopDeclaration(language,"i"+d,"0",""+(nTableL+d))+"\n";
			//toDecoder+="\t\t"+ind+" for(var i"+d+"=0; i"+d+"<"+nTableL+d+"; i"+d+"++){\n";
		}
		d++;
	}

	if (customTypes.contains(upCaseFirstLetter(type))){
		if (encoding == STRING){
			String[] params = {"encodedString"};
			toDecoder += "\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
					language,
					nTable+accessTable(language,dimension),
					LanguageTranslator.callMethod(language,false,"","decode"+upCaseFirstLetter(type),params)
			)+"\n"; 
			//toDecoder+="\t\t"+indent(dimension)+" "+nTable+accessTable(FLASH,dimension)+"= decode"+upCaseFirstLetter(type)+"(encodedString);\n";
		}
		else if (encoding == BINARY){
			String[] params = {"dis"};
			toDecoder += "\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
					language,
					nTable+accessTable(language,dimension),
					LanguageTranslator.callMethod(language,false,"","decode"+upCaseFirstLetter(type),params)
			)+"\n"; 
			//toDecoder+="\t\t"+indent(dimension)+" "+nTable+accessTable(FLASH,dimension)+"= decode"+upCaseFirstLetter(type)+"(encodedString);\n";
		}
	}
	else{
		if (encoding == STRING){
			String[] params = {"encodedString"};
			String parse = LanguageTranslator.callMethod(language,false,"","getNextParam",params);
			toDecoder += "\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
					language,
					nTable+accessTable(language,dimension),
					LanguageTranslator.parseStringType(language,type,parse)
			)+"\n";  
			//toDecoder+="\t\t"+indent(dimension)+" "+nTable+accessTable(FLASH,dimension)+"= ";
			//toDecoder+= getFlashTypeDecoder(type,"getNextParam(encodedString)")+";\n";	
		}
		else if (encoding == BINARY) {
			toDecoder += "\t\t"+indent(dimension)+LanguageTranslator.affectValueToVar(
					language,
					nTable+accessTable(language,dimension),
					LanguageTranslator.readBinaryType(language,type,"dis")
			)+"\n";
		}
	}

	toDecoder+= "\t"+closeQuote(language,dimension);

	if (objectsApproachMode){
		String[] params = {nTable};
		toDecoder+= "\t\t"+LanguageTranslator.callMethod(language,true,"customType","set"+upCaseFirstLetter(type),params)+LanguageTranslator.endOfLineMarker(language)+"\n";
		//toDecoder+="\t\t customType.set"+upCaseFirstLetter(type)+"("+nTable+");\n";
	}
	if (hashtablesApproachMode){
		String[] params = {"\""+type+"\"",nTable};
		toDecoder += "\t\t"+LanguageTranslator.callMethod(language,true,"h","put",params)+LanguageTranslator.endOfLineMarker(language)+"\n";

		//toDecoder+="\t\t h.put(\""+type+"\","+nTable+");\n";
	}
	return toDecoder;        
}

public static int getIndexInt(){
	return indexType;
}

/**
 * Generate string : ("}")*
 * 
 * @param dimension
 */
private static String closeQuote(Language language,int d) {
	int i = 0;
	String ind = "";
	while (i<d){
		ind+=indent(d-i+1)+LanguageTranslator.closeBlock(language)+"\n";
		i++;
	}
	return ind;
}


/**
 * Generate string : ("[ii]")*
 * 
 * @param d dimension
 * @return
 */
private static String accessTable(Language language, int d) {
	int i = 0;
	String form = "";
	while (i<d){
		form+=LanguageTranslator.arrayAccess(language,"i"+i);
		i++;
	}
	return form;
}


/**
 * Generate string : "table"("[]")*
 * 
 * @param d dimension
 * @return
 */
private static String tableLength(Language language, int d) {
	String form = "";
	int i = 0;
	while (i<d){
		form+=LanguageTranslator.arrayAccess(language,"i"+i);
		i++;
	}
	return form;
}


/**
 * Generate string : ("\t")*
 * 
 * @param d dimension
 * @return
 */
private static String indent(int d) {
	int i = 0;
	String ind = "";
	while (i<d){
		ind+="\t";
		i++;
	}
	return ind;
}


/**
 * Generate table dimension code.
 * 
 * @param dimension
 * @return the string of the dimension code
 */
public static String getDimension(int dimension) {
	String dim="";

	while(dimension>0){ 
		dim+="[]";
		dimension--;
	}
	return dim;
}

/**
 * @param s
 * @return
 */
public static String upCaseFirstLetter(String s) {
	char[] cTab = s.toCharArray();
	cTab[0] = (cTab[0]+"").toUpperCase().charAt(0);
	return new String(cTab);
}

public static String lowCaseFirstLetter(String s) {
	char[] cTab = s.toCharArray();
	cTab[0] = (cTab[0]+"").toLowerCase().charAt(0);
	return new String(cTab);
}


public static boolean copyFile(File srcFile, File destFile ) {
	boolean result = false;

	java.io.FileInputStream sourceFile = null;
	java.io.FileOutputStream destinationFile = null;
	try {
		destFile.createNewFile();
		sourceFile = new java.io.FileInputStream(srcFile);
		destinationFile = new java.io.FileOutputStream(destFile);

		byte buffer[] = new byte[512 * 1024];
		int nbReads;
		while ((nbReads = sourceFile.read(buffer)) != -1) {
			destinationFile.write(buffer, 0, nbReads);
		}

		result = true;
	} catch (java.io.FileNotFoundException f) {
		System.out.println("FileNotFoundException: "+f.getMessage());
	} catch (java.io.IOException e) {
		System.out.println("IOException: "+e.getMessage());
	} finally {
		try {
			sourceFile.close();
		} catch (Exception e) {
			System.out.println("Exception during source file close: "+e.getMessage());
		}
		try {
			destinationFile.close();
		} catch (Exception e) {
			System.out.println("Exception during dest file close: "+e.getMessage());
		}
	}
	return (result);
}

public static String getPath(Language language, boolean isACommonClass){
	String path = _package+"/";
	if (language.equals(Language.FLASH) )
		path += "client/flash/";	
	if (language.equals(Language.CPP) )
		path += "client/c++/";
	if (language.equals(Language.CSHARP) )
		path += "client/c#/";
	if (language.equals(Language.JAVA)  && !isACommonClass){
		if (generateServletLogic) path += "servlet/";
		else if (generateSqlLogic) path += "sql/";
		else path += (isClient? "client/":"server/");
	}
	return path;
}

private static int currentTypeCommunicationModel = 0;

public static void setCommunicationModelForElement(ComplexType type){
	try {
		String communicationModel = type.getAnnotation().getAppinfo().getOptions().getCommunicationModel().getValue().toLowerCase();

		if (communicationModel.equals("unicast")) currentTypeCommunicationModel = UNICAST;
		else if (communicationModel.equals("multicast")) currentTypeCommunicationModel = MULTICAST;
		else if (communicationModel.equals("broadcast"))currentTypeCommunicationModel = BROADCAST;
		else currentTypeCommunicationModel = SERVER;

	}
	catch(Exception e){
		currentTypeCommunicationModel = SERVER;
	}

}

private static boolean validatePackageName(String packageName) {
	char[] chars = new char[packageName.length()];
	packageName.getChars(0, chars.length, chars, 0);
	
	if(!(Character.isDigit(chars[0]) || Character.isLetter(chars[0]))){
		System.out.println("bad first char");
		return false;
	}
	
	if(!(Character.isDigit(chars[chars.length-1]) || Character.isLetter(chars[chars.length-1]))){
		System.out.println("bad end char");
		return false;
	}
	

	for (int i=1; i<chars.length-1; i++){
		if (!(Character.isDigit(chars[i]) || Character.isLetter(chars[i]))){
			if ((chars[i] == '.')){
				if (chars[i+1] == '.'){
					System.out.println("char "+(i+1)+": '.' unauthorized after a '.'");
					return false;
				}
			}
			else{
				System.out.println("char n°"+(i+1)+" invalid");
				return false;
			}
			
		}
	}	
	return true;
}

public static boolean isGenerationForUGaspLogic(){
	return generateUGaspLogic;
}

}
