/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import moods._2006.generation.tools.semanticactionsforgasp.OptionsType;

import org.mega.moods.generator.LanguageTranslator.Language;
import org.w3._2001.xmlschema.ComplexType;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class ServerGenerator {

    public static final int SERVER_GASPONLINE = 0;
    public static final int SERVER_UGASP = 1;
    
	private static Vector classFiles = new Vector();
    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    private static int serverType;
 
    // t: strings array to write
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: write required implemented methods: init, start, end, onStartEvent, onJoinEvent, onEndEvent, onQuitEvent, onDataEvent
    // t[7]: treatMessage method (specific because handles custom types objects)
    // t[8]: traditional utils methods
    // t[9]: end of class
  
    public static void init(int _serverType){
    	classFiles = new Vector();
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    	
    	serverType = _serverType;
    	
    }
    
    public static void startGenerateServer(String _package) throws IOException{
    	System.out.println("ServerGenerator: start to generate custom types for language: "+Language.JAVA.toString());
    	String[] strings = new String[]{"","","","","","","","","",""};
    	String path = Generator.getPath(Language.JAVA,false);
    	String linkClass = path+"Server";
    	ClassMaker.createClassEntry(Language.JAVA,linkClass,(serverType == SERVER_GASPONLINE)? "Server": "GameLogic",strings,false);

	    classNames.add(linkClass);
    	
    	// strings to write table for that class
    	
    	
    	// init strings table

    	strings[0] = "/** \n"
			+" * MooDS Game Server Logic for Java.\n"
	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	 		+" */\n\n";
    	strings[0] += LanguageTranslator.writeComments(Language.JAVA,"This class has generated for the "+(Generator.isClient? "client":"server")+" logic.")+"\n";
    	strings[0] += LanguageTranslator.writePackage(Language.JAVA,_package+".server")+"\n";
        
    	if (serverType == SERVER_GASPONLINE){
	    	// required imports
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Enumeration")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Hashtable")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Iterator")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Vector")+"\n\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.apache.log4j.Category")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.DataEvent")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.EndEvent")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.JoinEvent")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.QuitEvent")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.StartEvent")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.event.impl.DataEventImpl")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.gasp.server.GASPServer")+"\n\n"; 
	
	    	// class declaration
	    	strings[2] += LanguageTranslator.writeClassDeclaration(Language.JAVA,"Server",false,true,"GASPServer");
	    	
	    	// global attributes
	    	strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"GASP logger")+"\n";
	    	strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,true,false,false,false,"","Category","cat")+"\n";
	    	strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"hashtable to store player informations, key aSID, value an object representing player")+"\n";
	    	strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,true,false,false,false,"","Hashtable","players")+"\n";
	    	
	    	// constructor
	    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,"Server",null)+"\n";
	    	strings[5] += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","super",null)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings[5] += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","init",null)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n"; 	
	    	String value = LanguageTranslator.callMethod(Language.JAVA,true,"Category","getInstance", new String[]{"\"GASPLogging\""});
	    	strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(Language.JAVA, "cat", value)+"\n";
	    	strings[5] += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug", new String[]{"\"Log4J Server Category instanciated\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	
	    	// required interface methods 
	     	strings[6] = writeRequiredMethodsImplementations(Language.JAVA, strings[6]);
	    	
	     	// treatMessage method
	    	strings[7] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
	    																	true,
	    																	false,
	    																	true,
	    																	true,
	    																	"",
	    																	"treatMessage",
	    																	new String[][]{ //(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1],
	    																					{"Object","o"},
	    																					{"int","senderASID"}
																						}
	    																	,false,null)+"\n";
	    	
	    	if (Generator.hashtablesApproachMode) strings[7] += "\t\t"+
		    	LanguageTranslator.addLocalVar(
						Language.JAVA,
						true,
						LanguageTranslator.cast(Language.JAVA,"Hashtable","o"),
						"Hashtable",
						"hs"
					)+"\n";
	
	    	strings[8] = writeUtilMethods(strings[8]);
    	
    	
    	}
    	else if (serverType == SERVER_UGASP){
	    	// required imports
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Enumeration")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Hashtable")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.core.ApplicationInstance")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.protocol.rmai.RMAIUtils")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.impl.gamelogic.GameLogicImpl")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.eventmodel.event.*")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.eventmodel.event.impl.*")+"\n"; 
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.eventmodel.listener.*")+"\n";
	    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.ugasp.protocol.moods.CustomTypes")+"\n";
	
	    	// class declaration
	    	strings[2] += LanguageTranslator.writeClassDeclaration(Language.JAVA,"GameLogic",true,true,"OnRMAIEvent, OnMooDSEvent, OnJoinEvent, OnStartEvent, OnEndEvent, OnQuitEvent", "GameLogicImpl");
	    	
	    	// comments
	    	//strings[3] +="/*****************************************************/"+"\n";
	    	//strings[3] +="/** Mandatory code for all uGASP based game servers **/"+"\n";
	    	//strings[3] +="/*****************************************************/"+"\n";
	    	
	    	// global attributes
	    	//strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"iPOJO injection field for RMAI Utils")+"\n";
	    	//strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,false,true,false,false,false,"","RMAIUtils","rmaiUtils")+"\n\n";
	    	
	    	
	    	
	    	//strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,false,true,false,false,false,"","ApplicationInstance","appIns")+"\n\n";
	    	//strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"parent application instance")+"\n";
	    	//strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,false,true,false,false,false,"","ApplicationInstance","appIns")+"\n\n";
	    	//strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"moods protocol encoder/decoder")+"\n";
	    	//strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,false,false,false,false,"","org.ugasp.protocol.moods.CustomTypes","customTypes")+"\n";
	    	//TODO: strings[3] += "\t"+LanguageTranslator.addGlobalVar(Language.JAVA,false,false,false,false,"","org.ugasp.protocol.moods.string.CustomTypes","customTypes4StringEncoding")+"\n\n";
	
	    	
	    	// constructor
	    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,"GameLogic",null)+"\n";	
	    	//strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(Language.JAVA,"customTypes", LanguageTranslator.newType(Language.JAVA,"CustomTypesImpl",""))+"\n";
	    	strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(Language.JAVA,"CUSTOMTYPESIMPL_CLASS_NAME", "\""+_package+".CustomTypesImpl\"")+"\n"; 
	    	
	    	// required interface methods 
	     	strings[6] = writeRequiredMethodsImplementations(Language.JAVA, strings[6]);
	    	
	     	// treatMessage method
	    	strings[7] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
	    																	false,
	    																	true,
	    																	false,
	    																	false,
	    																	true,
	    																	true,
	    																	"",
	    																	"treatMooDSMessage",
	    																	new String[][]{ //(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1],
	    																					{"Object","moodsObject"},
	    																					{"short","senderActorSessionID"}
																						}
	    																	,false,null)+"\n";
	    	
	    	if (Generator.hashtablesApproachMode) strings[7] += "\t\t"+
		    	LanguageTranslator.addLocalVar(
						Language.JAVA,
						true,
						LanguageTranslator.cast(Language.JAVA,"Hashtable","moodsObject"),
						"Hashtable",
						"hs"
					)+"\n";
	    	
	    	strings[8] = writeOverridingMethods(strings[8]);
	    	
    	}
    	
    	strings[9] += LanguageTranslator.closeBlock(Language.JAVA);
    	
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("ServerGenerator: start to generate server at "+ path+"Server");
    }
    
    
    private static String writeRequiredMethodsImplementations(Language language, String strings) {	
    	if (serverType == SERVER_GASPONLINE){
	    	// init()
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","init",null,false,null)+"\n";
	    	strings += "\t\t"+LanguageTranslator.affectValueToVar(Language.JAVA,"players", LanguageTranslator.newType(Language.JAVA,"Hashtable",""))+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	// start()
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","start",null,false,null)+"\n";
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"Server START running!\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	// end()
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","end",null,false,null)+"\n";
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"Server STOP running!\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	strings += "\t"+LanguageTranslator.writeComments(Language.JAVA,"GASP event listeners")+"\n";
	    	
	    	
	    	// onStartEvent
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onStartEvent",new String[][]{{"StartEvent","e"}},false,null)+"\n";
	    	String message = LanguageTranslator.addValueToValue(Language.JAVA,true, new String[]{
	    																			"\"STARTEVENT received by server: aSID>\"",
	    																			LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)
	    																			});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{message})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","start",null)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	// onEndEvent
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onEndEvent",new String[][]{{"EndEvent","e"}},false,null)+"\n";
	    	message = LanguageTranslator.addValueToValue(Language.JAVA,true, new String[]{
	    																			"\"ENDEVENT received by server: aSID>\"",
	    																			LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)
	    																			});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{message})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","end",null)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	//onJoinEvent
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onJoinEvent",new String[][]{{"JoinEvent","e"}},false,null)+"\n";
	    	message = LanguageTranslator.addValueToValue(Language.JAVA,true, new String[]{
	    																			"\"JOINEVENT received by server: aSID>\"",
	    																			LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null),
	    																			"\" username>\"",
	    																			LanguageTranslator.callMethod(Language.JAVA,true,"e","getUsername",null)
	    																			});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{message})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	String[] params = {LanguageTranslator.addValueToValue(Language.JAVA,true,new String[]{"\"\"",LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)})
	    					   , LanguageTranslator.newType(Language.JAVA,"Object","")
	    						};
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"players","put",params)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	    	
	    	//onQuitEvent
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onQuitEvent",new String[][]{{"QuitEvent","e"}},false,null)+"\n";
	    	message = LanguageTranslator.addValueToValue(Language.JAVA,true, new String[]{
	    																			"\"QUITEVENT received by server: aSID>\"",
	    																			LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)
	    																			});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{message})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	String param = LanguageTranslator.addValueToValue(Language.JAVA,true,new String[]{"\"\"",LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"players","remove",new String[]{param})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
	
	    	//onDataEvent
	    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onDataEvent",new String[][]{{"DataEvent","e"}},false,null)+"\n";
	    	message = LanguageTranslator.addValueToValue(Language.JAVA,true, new String[]{
					"\"DATAEVENT received by server: aSID>\"",
					LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null),
					"\" Hashtable size>\"",
					LanguageTranslator.callMethod(Language.JAVA,true,LanguageTranslator.callMethod(Language.JAVA,true,"e","getData",null),"size",null)
					});
	    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{message})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t\t"+LanguageTranslator.addLocalVar(
	    														Language.JAVA,
	    														true,
	    														LanguageTranslator.callMethod(Language.JAVA,true,"e","getData",null),
	    														"Hashtable",
	    														"h1"
	    													)+"\n";
	    	strings += "\t\t"+LanguageTranslator.addLocalVar(
					Language.JAVA,
					true,
					LanguageTranslator.callMethod(Language.JAVA,true,"h1","size",null),
					"int",
					"nbObj"
				)+"\n";
	    	strings += "\t\t"+LanguageTranslator.forLoopDeclaration(Language.JAVA,"i","0","nbObj")+"\n";
	    	strings += "\t\t\t"+LanguageTranslator.addLocalVar(
					Language.JAVA,
					true,
					LanguageTranslator.callMethod(Language.JAVA,true,"h1","get",new String[]{LanguageTranslator.addValueToValue(Language.JAVA,true,new String[]{"i","\"\""})}),
					"Object",
					"o"
				)+"\n";
	    	String[] params2 = {"o",LanguageTranslator.callMethod(Language.JAVA,true,"e","getActorSessionID",null)};
	    	strings += "\t\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","treatMessage",params2)+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
	    	strings += "\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
	    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	}
    	else if (serverType == SERVER_UGASP){
    		
    	}
    	
    	
    	
    	return strings;
	}

    
    public static void addImport(String _import) {	
    	String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+"Server");
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA, _import)+"\n";
    	stringsToWrite.put(Generator.getPath(Language.JAVA,false)+"Server",strings);
    }
    
    
    public static void addCustomTypeTreatment(Language language, int encoding, String customType, int currentTypeCommunicationModel) {	
    	String[] strings = (String[]) stringsToWrite.get(Generator.getPath(language,false)+"Server");
    	
    	if (Generator.objectsApproachMode)
    		strings[7] += "\t\t"+LanguageTranslator.ifDeclaration(language,LanguageTranslator.testInstanceOfType(language,(serverType == SERVER_GASPONLINE)?"o":"moodsObject",customType))+"\n";
    	
    	if (Generator.hashtablesApproachMode){
    		 String[] params3 = {"\"type\""};
             String[] items21 = {
             					LanguageTranslator.callMethod(language,true,"hs","get",params3),
             					"\""+customType+"\""
             					};
             String cond1 = LanguageTranslator.testEqualType(language, true, customType,items21);
             strings[7] += "\t\t"+LanguageTranslator.ifDeclaration(language,cond1)+"\n";
    	}
    	
    	if (currentTypeCommunicationModel == Generator.BROADCAST){
    		strings[7] += "\t\t\t"+LanguageTranslator.writeComments(language,"BROADCAST communication model setted")+"\n";
    		strings[7] += "\t\t\t"+LanguageTranslator.callMethod(language,false,"","notifyToAllOtherPlayers",new String[]{(serverType == SERVER_GASPONLINE)?"o":"moodsObject","senderASID"})+LanguageTranslator.endOfLineMarker(language)+"\n";	
    	}
    	else if (currentTypeCommunicationModel == Generator.MULTICAST){
    		strings[7] += "\t\t\t"+LanguageTranslator.writeComments(language,"MULTICAST communication model setted")+"\n";
    		
    		String[] params = null;
    		if (Generator.objectsApproachMode) params = new String[]{(serverType == SERVER_GASPONLINE)?"o":"moodsObject",LanguageTranslator.callMethod(language,true,LanguageTranslator.cast(language,customType,(serverType == SERVER_GASPONLINE)?"o":"moodsObject"),"getRecipients",null)};
    		if (Generator.hashtablesApproachMode) params = new String[]{(serverType == SERVER_GASPONLINE)?"o":"moodsObject", LanguageTranslator.cast(language, "short[]",LanguageTranslator.callMethod(language,true,"hs","get",new String[]{"\"recipients\""}))};
    		
    		strings[7] += "\t\t\t"+LanguageTranslator.callMethod(language,false,"","notifyToRecipients",params)+LanguageTranslator.endOfLineMarker(language)+"\n";	
        } 	
    	else if (currentTypeCommunicationModel == Generator.UNICAST){
    		strings[7] += "\t\t\t"+LanguageTranslator.writeComments(language,"UNICAST communication model setted")+"\n";
    		
    		if (Generator.objectsApproachMode) strings[7] += "\t\t\t"+LanguageTranslator.callMethod(language,false,"","sendMessageToPlayer",new String[]{(serverType == SERVER_GASPONLINE)?"o":"moodsObject",LanguageTranslator.callMethod(language,true,LanguageTranslator.cast(language,customType,(serverType == SERVER_GASPONLINE)?"o":"moodsObject"),"getRecipient",null)})+LanguageTranslator.endOfLineMarker(language)+"\n";	
    		if (Generator.hashtablesApproachMode){
    			String[] params = {"\""+""+"\""};
        		
    			//TODO:
//    			String toWrite = LanguageTranslator.getObjectTypeValue(
//							language,
//							customType,
//							LanguageTranslator.cast(
//								language, 
//								customType,
//								LanguageTranslator.callMethod(language,true,"h","get",params)));
    			strings[7] += "\t\t\t"+LanguageTranslator.callMethod(language,false,"","sendMessageToPlayer",new String[]{(serverType == SERVER_GASPONLINE)?"o":"moodsObject",
    					LanguageTranslator.getObjectTypeValue(
    							language,
    							"int", LanguageTranslator.cast(language, "int",LanguageTranslator.callMethod(language,true,"hs","get",new String[]{"\"recipient\""})))})+LanguageTranslator.endOfLineMarker(language)+"\n";	
    		}

    	}
    	else // no gasp options for this custom type
    		 strings[7] += "\t\t\t"+LanguageTranslator.writeComments(language,"SERVER communication model: write treatments to "+customType+" object received")+"\n";
    	strings[7] += "\t\t"+LanguageTranslator.closeBlock(language)+"\n";
    	stringsToWrite.put(Generator.getPath(language,false)+"Server",strings);
    }
    
    private static String writeUtilMethods(String strings) {	
    	
    	/**
    	 * Sending a Message Object to all players
    	 * @param o
    	 *  
    	 */
    	
    	// notifyToAllPlayers
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,false,false,true,"","notifyToAllPlayers",new String[][]{{"Object","o"}/*(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1]*/},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"notifyAllPlayers called\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","Hashtable","h")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"h","put",new String[]{"\"0\"","o"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.newType(Language.JAVA,"DataEventImpl",new String[]{"0","h"}),"DataEvent","de")+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.callMethod(Language.JAVA,true,"players","keys",null),"Enumeration","e")+"\n";
    	strings += "\t\t"+LanguageTranslator.whileLoopDeclaration(Language.JAVA,LanguageTranslator.callMethod(Language.JAVA,true,"e","hasMoreElements",null))+"\n";
    	String var = LanguageTranslator.callMethod(Language.JAVA,true,"e","nextElement",null);
    	String string = LanguageTranslator.cast(Language.JAVA,"String",var);
    	strings += "\t\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.parseStringType(Language.JAVA,"int",string), "int", "pid")+"\n";
    	strings += "\t\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","sendDataTo",new String[]{"pid","de"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
    	strings += "\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
    	
    	
    	/**
    	 * Sending a Message object to all other players.
    	 * 
    	 * @param o, idSender
    	 */
    	
    	//notifyToAllOtherPlayers
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,false,false,true,"","notifyToAllOtherPlayers",new String[][]{{"Object","o"}/*(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1]*/,{"int","idSender"}},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"notifyToAllOtherPlayers called\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","Hashtable","h")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"h","put",new String[]{"\"0\"","o"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.newType(Language.JAVA,"DataEventImpl",new String[]{"0","h"}),"DataEvent","de")+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.callMethod(Language.JAVA,true,"players","keys",null),"Enumeration","e")+"\n";
    	strings += "\t\t"+LanguageTranslator.whileLoopDeclaration(Language.JAVA,LanguageTranslator.callMethod(Language.JAVA,true,"e","hasMoreElements",null))+"\n";
    	var = LanguageTranslator.callMethod(Language.JAVA,true,"e","nextElement",null);
    	string = LanguageTranslator.cast(Language.JAVA,"String",var);
    	strings += "\t\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.parseStringType(Language.JAVA,"int",string), "int", "pid")+"\n";
    	strings += "\t\t\t"+LanguageTranslator.ifDeclaration(Language.JAVA,LanguageTranslator.testEqualType(Language.JAVA,true,"int",new String[]{"idSender","pid"}))+"\n";
    	strings += "\t\t\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","sendDataTo",new String[]{"pid","de"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
    	strings += "\t\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
    	strings += "\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";

    	/**
    	 * Sending a Message object to recipient of a multicast object.
    	 * 
    	 * @param o, playerASID
    	 */
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,false,false,true,"","notifyToRecipients",new String[][]{{"Object","o"}/*(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1]*/,{LanguageTranslator.arrayDeclaration(Language.JAVA,"short",1),"recipients"}},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"notifyToRecipients called\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","Hashtable","h")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"h","put",new String[]{"\"0\"","o"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.newType(Language.JAVA,"DataEventImpl",new String[]{"0","h"}),"DataEvent","de")+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","int","i")+"\n";
      	strings += "\t\t"+LanguageTranslator.whileLoopDeclaration(Language.JAVA,
      						LanguageTranslator.testSup(Language.JAVA,true,new String[]{LanguageTranslator.arrayLength(Language.JAVA,"recipients"),"i"}))+"\n";
      	String[] items = {LanguageTranslator.callMethod(Language.JAVA,true,"players","get",new String[]{LanguageTranslator.addValueToValue(Language.JAVA,true,new String[]{"\"\"",LanguageTranslator.arrayAccess(Language.JAVA,"recipients","i")})}),"null"};
      	strings += "\t\t\t"+LanguageTranslator.ifDeclaration(Language.JAVA,LanguageTranslator.testEqualType(Language.JAVA,false,"int",items))+"\n";
      	strings += "\t\t\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","sendDataTo",new String[]{LanguageTranslator.arrayAccess(Language.JAVA,"recipients","i"),"de"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
      	strings += "\t\t\t"+LanguageTranslator.increment(Language.JAVA, true, "i")+"\n";
    	strings += "\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
    	
    	/**
    	 * Sending a Message object to a specific player.
    	 * 
    	 * @param o, playerASID
    	 */
    	
    	// sendMessageToPlayer   	
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,false,false,true,"","sendMessageToPlayer",new String[][]{{"Object","o"}/*(Generator.objectsApproachMode) ? messageTypeVars[0] : messageTypeVars[1]*/,{"int","playerASID"}},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"sendMessageToPlayer called\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","Hashtable","h")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"h","put",new String[]{"\"0\"","o"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.newType(Language.JAVA,"DataEventImpl",new String[]{"0","h"}),"DataEvent","e")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","sendDataTo",new String[]{"playerASID","e"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
    	
    	/**
    	 * Sending few Messages to a specific player.
    	 * 
    	 * @param v, playerASID
    	 */

    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,false,false,true,"","sendMessageToPlayer",new String[][]{{"Vector","v"},{"int","playerASID"}},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"cat","debug",new String[]{"\"sendMessageToPlayer called\""})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,"","Hashtable","h")+"\n";
      	strings += "\t\t"+LanguageTranslator.forLoopDeclaration(Language.JAVA,"i","0",LanguageTranslator.callMethod(Language.JAVA,true,"v","size",null))+"\n";	
      	String iString = LanguageTranslator.addValueToValue(Language.JAVA,true,new String[]{"\"\"","i"});
      	strings += "\t\t\t"+LanguageTranslator.callMethod(Language.JAVA,true,"h","put",new String[]{iString,LanguageTranslator.callMethod(Language.JAVA,true,"v","get",new String[]{"i"})})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n";
      	strings += "\t\t"+LanguageTranslator.addLocalVar(Language.JAVA,true,LanguageTranslator.newType(Language.JAVA,"DataEventImpl",new String[]{"0","h"}),"DataEvent","e")+"\n";
      	strings += "\t\t"+LanguageTranslator.callMethod(Language.JAVA,false,"","sendDataTo",new String[]{"playerASID","e"})+LanguageTranslator.endOfLineMarker(Language.JAVA)+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
    	
    	
    	return strings;
	}

    
    private static String writeOverridingMethods(String strings) {	
    	
    	/*protected RMAIUtils getRMAIUtils() {
    		// TODO Auto-generated method stub
    		return null;
    		}	
    	 */
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,true,false, false,false,false,"RMAIUtils","getRMAIUtils",null,false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.returnValue(Language.JAVA, "null")+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	

    	/*@Override
    	protected Object newRMAIGameLogicManager(short senderASID,
    			short recipientASID) {
    		// TODO Auto-generated method stub
    		return null;
    	}*/
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,false,true,false,false,false,false,"Object","newRMAIGameLogicManager",new String[][]{{"short","senderASID"},{"short","recipientASID"}},false,null)+"\n";
    	strings += "\t\t"+LanguageTranslator.returnValue(Language.JAVA, "null")+"\n";
    	strings += "\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	

    	/*@Override
    	public void onStartEvent(StartEvent se) {
    		// TODO Auto-generated method stub
    		
    	}*/
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onStartEvent",new String[][]{{"StartEvent","se"}},false,null);
    	strings += LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	/*@Override
    	public void onJoinEvent(JoinEvent se) {
    		// TODO Auto-generated method stub
    		
    	}*/
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onJoinEvent",new String[][]{{"JoinEvent","je"}},false,null);
    	strings += LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	/*@Override
    	public void onEndEvent(EndEvent ee) {
    		// TODO Auto-generated method stub
    		
    	}*/
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onEndEvent",new String[][]{{"EndEvent","ee"}},false,null);
    	strings += LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	/*@Override
    	public void onQuitEvent(QuitEvent qe) {
    		// TODO Auto-generated method stub
    		
    	}*/
    	strings += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,true,false,false,true,"","onQuitEvent",new String[][]{{"QuitEvent","qe"}},false,null);
    	strings += LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	
    	
    	return strings;
	}
    
    // t: tableau de strings a ecrire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: write required implemented methods: init, start, end, onStartEvent, onJoinEvent, onEndEvent, onQuitEvent, onDataEvent
    // t[7]: treatMessage method (specific because handles custom types objects)
    // t[8]: traditional utils methods
    // t[9]: end of class
    public static void finalizeServer() throws IOException{
   	 	System.out.println("ServerGenerator: finalize server file...");
    	String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+"Server");
    	
    	if (!strings[0].equals("")) strings[0] += "\n";
    	if (!strings[1].equals("")) strings[1] += "\n";
    	if (!strings[2].equals("")) strings[2] += "\n\n";
    	if (!strings[3].equals("")) strings[3] += "\n";
    	if (!strings[4].equals("")) strings[4] = "\t// attributes\n"+strings[4]+"\n";
    	
    	if (!strings[5].equals(""))
    		strings[5] = "\t// constructor\n"+strings[5]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[6].equals(""))
    		strings[6] = "\t// required methods\n"+strings[6]+"\n";

    	if (!strings[7].equals(""))
    		strings[7] = "\t// treat specifically all custom types objects\n"+strings[7]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n\n";
    	
    	if (!strings[8].equals(""))
    		strings[8] = "\t//"+((serverType == SERVER_GASPONLINE)?
    								" traditional methods used":
    								" inherited methods to implement")+"\n"+strings[8];

    	if (!strings[9].equals(""))
    		strings[9] = strings[9]+"\n";
    	
   	 	System.out.println("ServerGenerator: server file finalized.");
    }
}
