/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import org.mega.moods.generator.LanguageTranslator.Language;


/**
 * MooDS Generator v2.0
 * @author Chanthy LIM
 */
public class ServletGenerator {

    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    
    private static final String SERVLET_SUFFIX = "Servlet";
    private static String servlet_complete_classname = "";
    private static String servlet_complete_path = "";

    // t: strings array to write
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: comments
    // t[4]: init method
    // t[5]: doGet method
    // t[6]: doPost method
  
    public static void init(){
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    }
    
    public static void startGenerateServlet(String _package, String className) throws IOException{
    	System.out.println("ServletGenerator: start to generate custom types for Language.JAVA: "+Language.JAVA.toString());
    	
    	String[] strings = new String[]{"","","","","","","", ""};
    	servlet_complete_path = _package + "/servlet/";
    	servlet_complete_classname = className + SERVLET_SUFFIX;
    	String linkClass = servlet_complete_path+servlet_complete_classname;
    	ClassMaker.createClassEntry(Language.JAVA,linkClass,servlet_complete_classname,strings,false);

	    classNames.add(linkClass);
    	
    	// strings to write table for that class
    	
    	
    	// init strings table

    	strings[0] = "/** \n"
			+" * MooDS Game Server Logic for Java.\n"
	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	 		+" */\n\n";
    	strings[0] += LanguageTranslator.writeComments(Language.JAVA,"This class has generated for the Web server.")+"\n";
    	strings[0] += LanguageTranslator.writePackage(Language.JAVA,_package+".servlet")+"\n";
        
    	// required imports
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.io.DataInputStream")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.io.DataOutputStream")+"\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.io.IOException")+"\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Hashtable")+"\n\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"javax.servlet.ServletContext")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"javax.servlet.ServletException")+"\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"javax.servlet.http.HttpServlet")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"javax.servlet.http.HttpServletRequest")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"javax.servlet.http.HttpServletResponse")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,_package+".server.*")+"\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,_package+".sql.*")+"\n";

    	// class declaration
    	strings[2] += LanguageTranslator.writeClassDeclaration(Language.JAVA,servlet_complete_classname,false,true,"HttpServlet");
    	
    	// global attributes
    	strings[3] += "\tprivate static final String SELECT = \"select\";\n";
    	strings[3] += "\tprivate static final String UPDATE = \"update\";\n";
    	strings[3] += "\tprivate static final String INSERT = \"insert\";\n";
    	strings[3] += "\tprivate static final String DELETE = \"delete\";\n";
    	strings[3] += "\tCustomTypes ct = null;\n";
    	strings[3] += "\tSqlStorageManager sqlManager = null;\n";
    	
    	// constructor
    	strings[4] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
				true,
				false,
				true,
				true,
				"",
				"init",
				new String[][]{},
				false,
				null)+"\n";
    	strings[4] += "\t\tServletContext app = getServletContext();\n";
    	strings[4] += "\t\tct = new CustomTypes();\n";
    	strings[4] += "\t\ttry {\n";
    	strings[4] += "\t\t\tboolean useDataSource = app.getInitParameter(\"moods.use.datasource\")!=null ? Boolean.valueOf(app.getInitParameter(\"moods.use.datasource\")).booleanValue() : false;\n";
    	strings[4] += "\t\t\tString webProjectName = app.getInitParameter(\"moods.web.home\") != null ? app.getInitParameter(\"moods.web.home\") : \"project\";\n";
    	strings[4] += "\t\t\tsqlManager = new SqlStorageManager(useDataSource, webProjectName);\n";
    	strings[4] += "\t\t} catch (Exception e) {\n";
    	strings[4] += "\t\t\te.printStackTrace();\n";
    	strings[4] += "\t\t}\n";
    	
     	// doGet method
    	strings[5] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
    																	true,
    																	false,
    																	true,
    																	true,
    																	"",
    																	"doGet",
    																	new String[][]{	{"HttpServletRequest","request"},
				 																		{"HttpServletResponse","response"}
																					},
																		true,
																		new String[]{"IOException", "ServletException"})+"\n";
    	strings[5] += "\t\tdoPost(request, response);\n";

     	// doPost method
    	strings[6] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
    																	true,
    																	false,
    																	true,
    																	true,
    																	"",
    																	"doPost",
    																	new String[][]{	{"HttpServletRequest","request"},
				 																		{"HttpServletResponse","response"}
																					},
																		true,
																		new String[]{"IOException", "ServletException"})+"\n";

    	strings[7] += LanguageTranslator.closeBlock(Language.JAVA);
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("ServerGenerator: start to generate server at "+ servlet_complete_path+"Servlet");
    }
    
    
    public static void generateDoPostMethod(String customType, TypesJAXBReader reader) throws Exception {	
    	generateDoPostBody(customType, reader);
    }
    
    public static void generateDoPostBody(String customType, TypesJAXBReader reader) throws Exception {	
    	boolean selectPresent = reader.isRequestPresent(TypesJAXBReader.SELECT_TYPE);
    	boolean updatePresent = reader.isRequestPresent(TypesJAXBReader.UPDATE_TYPE);
    	boolean insertPresent = reader.isRequestPresent(TypesJAXBReader.INSERT_TYPE);
    	boolean deletePresent = reader.isRequestPresent(TypesJAXBReader.DELETE_TYPE);
    	if (selectPresent || updatePresent || insertPresent || deletePresent) {    		
	    	String[] strings = (String[]) stringsToWrite.get(servlet_complete_path+servlet_complete_classname);
	    	strings[6] += "\t\tDataInputStream dis = new DataInputStream(request.getInputStream());\n";
	    	strings[6] += "\t\tDataOutputStream dos = null;\n";
	    	strings[6] += "\t\ttry {\n";
	    	strings[6] += "\t\t\tdos = new DataOutputStream(response.getOutputStream());\n";
	    	strings[6] += "\t\t\tString requestType = request.getParameter(\"type\");\n";
	
	    	strings[6] += "\t\t\tif (SELECT.equals(requestType)) {\n";
	    	if (selectPresent) {
		    	strings[6] += "\t\t\t\tct.encodeData(sqlManager.select(), dos);\n";
	    	}
	    	strings[6] += "\t\t\t}\n";
	    	
	    	strings[6] += "\t\t\tif (UPDATE.equals(requestType)) {\n";
	    	if (updatePresent) {
		    	strings[6] += "\t\t\t\tsqlManager.update(ct.decodeData(dis));\n";
	    	}
	    	strings[6] += "\t\t\t}\n";
	    	
	    	strings[6] += "\t\t\tif (INSERT.equals(requestType)) {\n";
	    	if (insertPresent) {
		    	strings[6] += "\t\t\t\tsqlManager.insert(ct.decodeData(dis));\n";
	    	}
	    	strings[6] += "\t\t\t}\n";
	    	
	    	strings[6] += "\t\t\tif (DELETE.equals(requestType)) {\n";
	    	if (deletePresent) {
		    	strings[6] += "\t\t\t\tsqlManager.delete(ct.decodeData(dis));\n";
	    	}
	    	strings[6] += "\t\t\t}\n";
	    	
		    strings[6] += "\t\t} catch (Exception e) {\n";
	    	strings[6] += "\t\t\te.printStackTrace();\n";
			strings[6] += "\t\t}\n";
			strings[6] += "\t\tfinally{\n";
			strings[6] += "\t\t\tif (dis!=null) try{dis.close();}catch (Exception e){}\n";
			strings[6] += "\t\t\tif (dos!=null) try{dos.close();}catch (Exception e){}\n";
			strings[6] += "\t\t}\n";
	    	stringsToWrite.put(servlet_complete_path+servlet_complete_classname,strings);
    	}
    }
    
    // t: tableau de strings a ecrire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: write required implemented methods: init, start, end, onStartEvent, onJoinEvent, onEndEvent, onQuitEvent, onDataEvent
    // t[7]: treatMessage method (specific because handles custom types objects)
    // t[8]: traditional utils methods
    // t[9]: end of class
    public static void finalizeServlet() throws IOException{
   	 	System.out.println("ServerGenerator: finalize server file...");
    	String[] strings = (String[]) stringsToWrite.get(servlet_complete_path+servlet_complete_classname);
    	
    	if (!strings[0].equals("")) strings[0] += "\n";
    	if (!strings[1].equals("")) strings[1] += "\n";
    	if (!strings[2].equals("")) strings[2] += "\n\n";
    	if (!strings[3].equals("")) strings[3] += "\n";
    	if (!strings[4].equals("")) strings[4] = "\t// init method\n"+strings[4]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	
    	if (!strings[5].equals(""))
    		strings[5] = "\t// doGet method\n"+strings[5]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[6].equals(""))
    		strings[6] = "\t// doPost method\n"+strings[6]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[7].equals(""))
    		strings[7] = strings[7]+"\n";
    	
   	 	System.out.println("ServletGenerator: server file finalized.");
    }
}