/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import org.mega.moods.generator.LanguageTranslator.Language;
import org.mega.moods.generator.sql.SqlTransform;
import org.w3._2001.xmlschema.Attribute;
import org.w3._2001.xmlschema.ComplexType;
import org.w3._2001.xmlschema.ElementRef;


/**
 * MooDS Sql Generator v2.0.
 * This class generates the SqlStorageManager tool. This tool manage the data
 * persistence associated with an encoder/decoder of a mobile game.
 * @author Chanthy LIM
 */
public class SqlStorageManagerGenerator {

    private static boolean objectsApproachMode = false;
    private static boolean hashtablesApproachMode = false;
    
    private static String SQL_CLASS_NAME = "SqlStorageManager";

    private static Vector classNames = null;
    private static Hashtable stringsToWrite = null;
    // Instantiate the SqlTransform utility class
    private static SqlTransform transform = null;
	private static Hashtable elementHash_ = null;
	
	private static String updateRequestValue = "";
	private static String insertRequestValue = "";
	private static String deleteRequestValue = "";
	
	private static boolean selectRequestPresent = false;
	private static boolean updateRequestPresent = false;
	private static boolean insertRequestPresent = false;
	private static boolean deleteRequestPresent = false;
    
    // t: strings array to write
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: select method
    // t[7]: update method
    // t[8]: insert method
    // t[9]: delete method    
    // t[10]: Class end declaration
  
    public static void init(){
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    	transform = new SqlTransform();
    	elementHash_ = new Hashtable();
    }
    
    /**
     * Start the SqlStorageManager class writing.
     * @param Language.JAVA the lanaguage in which the class will be generated.
     * @param _package the package where the class will be stored.
     * @param hashtableMode is true if the data type is Hashtable of String values. Else the data type is Hashtable of custom user Objects. 
     * @throws IOException
     */
    public static void startGenerateSqlStorageManager(String _package, boolean hashtableMode) throws IOException{
    	System.out.println("SqlStorageManagerGenerator: start to generate custom types for Language.JAVA: "+Language.JAVA.toString());
    	
    	// Set hashtable mode for the client
    	setHashtableMode(hashtableMode);
    	
    	String[] strings = new String[]{"","","","","","","","","","", "", ""};
    	String path = Generator.getPath(Language.JAVA,false);
    	String linkClass = path+SQL_CLASS_NAME;
    	ClassMaker.createClassEntry(Language.JAVA,linkClass,SQL_CLASS_NAME,strings,false);

	    classNames.add(linkClass);
    	
    	// strings to write table for that class
    	
    	
    	// init strings table

    	strings[0] = "/** \n"
			+" * MooDS Sql Storage Manager for Java.\n"
	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	 		+" */\n\n";
    	strings[0] += LanguageTranslator.writeComments(Language.JAVA,"This class has generated for the data persistence management")+"\n";
    	strings[0] += LanguageTranslator.writePackage(Language.JAVA,_package+".sql")+"\n";
        
    	// required imports
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Properties")+"\n";
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.util.Hashtable")+"\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.sql.ResultSet")+"\n\n"; 
    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"java.sql.ResultSetMetaData")+"\n\n"; 

    	strings[1] += LanguageTranslator.writeImport(Language.JAVA,"org.mega.moods.generator.sql.DatabaseHandler")+"\n\n"; 

    	if (!hashtableMode) {
    		strings[1] += LanguageTranslator.writeImport(Language.JAVA,_package+".*")+"\n\n";
    	}

    	// class declaration
    	strings[2] += LanguageTranslator.writeClassDeclaration(Language.JAVA,SQL_CLASS_NAME,false,true,"DatabaseHandler");
    	
    	// global attributes
    	strings[3] += "\t"+LanguageTranslator.writeComments(Language.JAVA,"Sql Storage Manager")+"\n";
    	
    	// constructor
    	strings[5] += "\t//Constructor with default parameters, i.e. with no connection pool management and 'project' as the web project name.\n";
    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,
				  SQL_CLASS_NAME,
				  new String[][]{},
				  true ,
				  new String[]{"Exception"})+"\n";
    	strings[5] += "\t\tsuper();\n";
    	strings[5] += "\t}\n";
		
    	strings[5] += "\t//Constructor with given parameters, useDataSource is true if using the Apache Tomcat connection pool management and project as the web project name.\n";
    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(Language.JAVA,
				  SQL_CLASS_NAME,
				  new String[][]{{"boolean", "useDataSource"}, {"String", "project"}},
				  true,
				  new String[]{"Exception"})+"\n";
    	strings[5] += "\t\tsuper(useDataSource, project);\n";
    	
    	strings[10] += LanguageTranslator.closeBlock(Language.JAVA);
    	stringsToWrite.put(linkClass,strings);
    	System.out.println("SqlStorageGenerator: start to generate sql manager at "+ path+"Sql");
    }
    
    public static void generateRequestMethodStart(ComplexType msgElement,
    											  TypesJAXBReader reader) throws IOException{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
		selectRequestPresent = reader.isRequestPresent(msgElement, TypesJAXBReader.SELECT_TYPE);
		updateRequestPresent = reader.isRequestPresent(msgElement, TypesJAXBReader.UPDATE_TYPE);
		insertRequestPresent = reader.isRequestPresent(msgElement, TypesJAXBReader.INSERT_TYPE);
		deleteRequestPresent = reader.isRequestPresent(msgElement, TypesJAXBReader.DELETE_TYPE);
		// select method
		if (selectRequestPresent) {
	    	strings[6] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
	    																	true,
	    																	false,
	    																	true,
	    																	false,
	    																	"Hashtable",
	    																	"select",
	    																	new String[][]{}
	    																	,true,
	    																	new String[] {"Exception"}) + "\n";
	    	strings[6] += generateSelectDatabaseConnectionStart("");
    	}    	
		
    	// update method
		if (updateRequestPresent) {
			strings[7] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
					true,
					false,
					true,
					false,
					"void",
					"update",
					new String[][]{{"Hashtable", "hash"}},
					true,
					new String[] {"Exception"}) + "\n";
			strings[7] += generateSelectDatabaseConnectionStart("");
			strings[7] += generateUpdateStartHashBefore(transform, "", 1, "");
			strings[7] += generatePropertiesOnlyDeclaration(1, "");
		}
			
    	// insert method
		if (insertRequestPresent) {
	    	strings[8] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
					true,
					false,
					true,
					false,
					"void",
					"insert",
					new String[][]{{"Hashtable", "hash"}},
					true,
					new String[] {"Exception"}) + "\n";
	    	strings[8] += generateSelectDatabaseConnectionStart("");
	    	strings[8] += generateUpdateStartHashBefore(transform, "", 1, "");
			strings[8] += generatePropertiesOnlyDeclaration(1, "");
		}
		
    	// delete method
		if (deleteRequestPresent) {
	    	strings[9] += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.JAVA,	
					true,
					false,
					true,
					false,
					"void",
					"delete",
					new String[][]{{"Hashtable", "hash"}},
					true,
					new String[] {"Exception"}) + "\n";
	    	strings[9] += generateSelectDatabaseConnectionStart("");
	    	strings[9] += generateUpdateStartHashBefore(transform, "", 1, "");
			strings[9] += generatePropertiesOnlyDeclaration(1, "");
		}    	
    }
    
    public static void generateSqlBodyStart(ComplexType msgElement,String xsdElement,
    										int parentIndex,int elementIndex, 
    										TypesJAXBReader reader) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
					
			// Generates request and variables declaration
		if (reader.isRequestPresent(msgElement, TypesJAXBReader.SELECT_TYPE)) {
			// Generate context properties declaration
			strings[6] += generatePropertiesDeclaration(elementIndex, "");
			generateSelectElementHashStart(reader, msgElement, xsdElement, "", msgElement.getAttribute(), parentIndex, elementIndex, "", true);
		}
		if (updateRequestPresent)
			generateUpdateElementHashStart(reader, msgElement, xsdElement, "", msgElement.getAttribute(), parentIndex, elementIndex, "", true);		
		if (insertRequestPresent)
			generateInsertElementHashStart(reader, msgElement, xsdElement, "", msgElement.getAttribute(), parentIndex, elementIndex, "", true);		
		if (deleteRequestPresent)
			generateDeleteElementHashStart(reader, msgElement, xsdElement, "", msgElement.getAttribute(), parentIndex, elementIndex, "", true);	
    }

    
    public static void generateSqlBody(ComplexType msgElement, String realXsdElementName,
    									String xsdElementName, int elementIndex, 
    									TypesJAXBReader reader) throws Exception {
    	if (elementHash_.containsKey(msgElement.getName())) return;
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
		List attList = msgElement.getAttribute();
		// Generate the select body
		generateSelectElementHashStart(reader, msgElement, xsdElementName, "", attList, 1, elementIndex, "", true);
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		generateSelectElementHashBodyAttribute(reader, msgElement, realXsdElementName, "", (Attribute)attElements.next(), 1, elementIndex, "", true);
    	}
		generateSelectElementHashBodyAttributeEnd(reader, msgElement, "", attList, 1, elementIndex, "", true);
    	if (msgElement.getSequence() != null && !msgElement.getSequence().getElement().isEmpty()) {
			Iterator sequenceElements = msgElement.getSequence().getElement().iterator();
			while (sequenceElements.hasNext()) {		
				generateSelectElementHashBodySequenceBody(reader, msgElement, xsdElementName, "", (ElementRef)sequenceElements.next(), 1, elementIndex, "", true);
			}
			generateSelectElementHashBodySequenceEnd(reader, msgElement, "", 1, elementIndex, "", true);
    	}
		transform.resetValues();
		strings[7] += generateUpdateElementHash(reader, msgElement, realXsdElementName, xsdElementName, "", attList, 1, elementIndex, "", true, transform);
		getElementsHashtable().put(msgElement.getName(), msgElement);
		transform.resetValues();
    }

    public static boolean alreadyElementExist(ComplexType msgElement) {
    	if (elementHash_.containsKey(msgElement.getName())) return true;
    	return false;
    }
    
    public static void generateSqlEnd(TypesJAXBReader reader,
    								  int parentIndex, 
    								  String tabString) throws Exception {
    	String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);

		if (selectRequestPresent) {
	        // Close database connection
	        strings[6] += generateSelectDatabaseConnectionEnd(tabString);
	        // Method return for select
	        strings[6] += generateSelectReturn(tabString, parentIndex);
		}	
		if (updateRequestPresent) {
	        strings[7] += generateElementEnd(tabString);
	        strings[7] += generateSelectDatabaseConnectionEnd(tabString);
		}
		if (insertRequestPresent) {
	        strings[8] += generateElementEnd(tabString);
	        strings[8] += generateSelectDatabaseConnectionEnd(tabString);
		}	
		if (deleteRequestPresent) {
	        strings[9] += generateElementEnd(tabString);
	        strings[9] += generateSelectDatabaseConnectionEnd(tabString);
		}
    }
    
    public static boolean canGenerate(ComplexType element) {
    	if (!elementHash_.containsKey(element.getName())) return true;
    	return false;
    }
    
    public static void generateSelectElementHashStart(TypesJAXBReader reader, 
    												ComplexType element, 
    												String xsdElementName,
    												String elementTypeParam,
    												List attList, 
    												int parentIndex,
    												int elementIndex,
    												String tabString,
    												boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
    	
    	String request = reader.getRequestFromName(elementType, TypesJAXBReader.SELECT_TYPE);   
    	
    	strings[6] += generateSelectStartHash(transform, request, xsdElementName, elementIndex, tabString);
    	
    	// generate field declaration
    	strings[6] += generateSelectHashtableTypeField(xsdElementName, elementIndex, tabString);
    	    	
    	System.out.println("Method String generateSelectElementHashStart() generated");
    }

    public static void generateElementHashBodyAttributeStart(TypesJAXBReader reader, 
    														ComplexType element,
    														String xsdElementName,
    														String elementTypeParam,
    														int parentIndex,
    														int elementIndex,
    														String tabString,
    														boolean forceId) throws Exception {
		if (selectRequestPresent)
			generateSelectElementHashBodyAttributeListFromRequest(reader, element, xsdElementName, "", parentIndex, elementIndex, "", true);    	
    }
    
    public static void generateElementHashBodyAttribute(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String realXsdElementName,
    		String xsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		if (selectRequestPresent)
			generateSelectElementHashBodyAttribute(reader, element, realXsdElementName, "", attribute, parentIndex, elementIndex, tabString, true);
		if (updateRequestPresent)
			generateUpdateElementHashBodyAttribute(reader, element, realXsdElementName, "", attribute, parentIndex, parentIndex, tabString+"\t", true);
		if (insertRequestPresent)
			generateInsertElementHashBodyAttribute(reader, element, realXsdElementName, "", attribute, parentIndex, parentIndex, tabString+"\t", true);
		if (deleteRequestPresent)
			generateDeleteElementHashBodyAttribute(reader, element, realXsdElementName, "", attribute, parentIndex, parentIndex, tabString+"\t", true);
    }

    public static void generateSelectElementHashBodyAttributeListFromRequest(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
    	
    	
		//String attName = attribute.getName();
		//String attType = attribute.getType();
		strings[6] += tabString+"\t\t\t ResultSetMetaData rsmd"+elementIndex+" = rs"+elementIndex+".getMetaData();\n";
		strings[6] += tabString+"\t\t\t int numberOfColumns"+elementIndex+" = rsmd"+elementIndex+".getColumnCount();\n";
		strings[6] += tabString+"\t\t\t for (int columnIndex"+elementIndex+"=1; columnIndex"+elementIndex+" <= numberOfColumns"+elementIndex+"; columnIndex"+elementIndex+"++) {\n";
		String columnName = "rsmd"+elementIndex+".getColumnLabel(columnIndex"+elementIndex+")";
		//strings[6] += generateSelectFieldFromTable(columnName, columnName, "rsmd"+elementIndex+".getColumnTypeName(columnIndex"+elementIndex+")", elementIndex, tabString+"\t");
		//strings[6] += generateSelectFieldFromTable("rsmd"+elementIndex+".getTableName(columnIndex"+elementIndex+")"+"+\".\"+"+columnName, columnName, "rs" + elementIndex + ".getObject(columnIndex" + elementIndex + ")", elementIndex, tabString+"\t");
		generateSelectPropertiesField(transform, "rsmd"+elementIndex+".getTableName(columnIndex"+elementIndex+")", elementType, columnName, "columnIndex"+elementIndex, elementIndex, tabString+"\t");
		strings[6] += tabString+"\t\t\t }\n\n";
    }

    public static void generateSelectElementHashBodyAttribute(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String realXsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
    	
		String attName = attribute.getName();
		String attType = attribute.getType();
		strings[6] += generateSelectField(elementType, realXsdElementName, attName, getType(attType), elementIndex, tabString);
    }

    public static void generateElementHashBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		if (selectRequestPresent)
			generateSelectElementHashBodyAttributeEnd(reader, element, "", attList, parentIndex, elementIndex, tabString, true);
		if (updateRequestPresent)
			generateUpdateElementHashBodyAttributeEnd(reader, element, "", attList, parentIndex, parentIndex, tabString+"\t", true);
		if (insertRequestPresent)
			generateInsertElementHashBodyAttributeEnd(reader, element, "", attList, parentIndex, parentIndex, tabString+"\t", true);
		if (deleteRequestPresent)
			generateDeleteElementHashBodyAttributeEnd(reader, element, "", attList, parentIndex, parentIndex, tabString+"\t", true);
    }

    public static void generateSelectElementHashBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	
    	// Generete field add for properties that are not present in the element attribute list
    	String elementType = element.getName();
    	strings[6] += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, tabString);
    	
    	if (attList.size() > 0) {
    		if (parentIndex > 1) {            	
    			strings[6] += generateSelectFieldIndex(elementIndex, tabString);
    		} else {
    			strings[6] += generateSelectFieldIndexWithPrecedingIndex(elementIndex, tabString);
    		}
    	}
    }

    public static void generateSelectElementTabBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	
    	// Generete field add for properties that are not present in the element attribute list
    	String elementType = element.getName();
    	strings[6] += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, tabString);
    	
    	if (attList.size() > 0) {
   			strings[6] += generateSelectFieldIndex(elementIndex, tabString);
    	}
    }

    public static void generateElementHashBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		if (selectRequestPresent) {
			generateSelectElementHashBodySequenceBody(reader, element, xsdElementName, "", ref, parentIndex, elementIndex, tabString, true);
			transform.resetValues();
		}
		if (updateRequestPresent) {
			generateUpdateElementHashBodySequenceBody(reader, element, xsdElementName, "", ref, parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
		if (insertRequestPresent) {
			generateInsertElementHashBodySequenceBody(reader, element, xsdElementName, "", ref, parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
		if (deleteRequestPresent) {
			generateDeleteElementHashBodySequenceBody(reader, element, xsdElementName, "", ref, parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
    }

    public static void generateElementHashBodySequenceBodyEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		if (selectRequestPresent) {
	    	generateSelectElementTabBodyEnd(reader, element, "", parentIndex, elementIndex, tabString, true);
			transform.resetValues();
		}
		if (updateRequestPresent) {
			generateUpdateElementTabBodySequenceEnd(reader, element, "", parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
		if (insertRequestPresent) {
			generateInsertElementHashSequenceEnd(reader, element, "", parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
		if (deleteRequestPresent) {
			generateDeleteElementHashSequenceEnd(reader, element, "", parentIndex, parentIndex, tabString+"\t", true);
			transform.resetValues();
		}
    }

    public static void generateSelectElementHashBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
    	ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
    	List subElemAttList = subElement.getAttribute();	                
		generateSelectElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, "\t", false);
		strings[6] += generateSelectSubHashSet(seqElementType, elementIndex, 1 + elementIndex, tabString);
		getElementsHashtable().put(subElement.getName(), subElement);
		generateSelectElementHashBodySequenceEnd(reader, element, "", parentIndex, elementIndex, "", true);
    }

    public static void generateSelectElementTabBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
    	ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
    	List subElemAttList = subElement.getAttribute();	                
		generateSelectElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, tabString, false);
		strings[6] += generateSelectSubHashSet(seqElementType, elementIndex, 1 + elementIndex, tabString);
		getElementsHashtable().put(subElement.getName(), subElement);
    }

    public static void generateSelectElementHashBodySequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	
    	// Generate the select set method
		strings[6] += generateSelectHashSubSet(elementTypeParam, parentIndex, elementIndex, tabString, forceId);
   		
    	// Generate the select end method
    	strings[6] += generateElementEnd(tabString);    
    	
    	// Close Statement and ResultSet
    	strings[6] += generateSelectDatabaseRequestEnd(elementIndex, tabString);
    	
    	// Update index
    	if (elementIndex < 2) {
    		strings[6] += generateSelectFieldIndexUpdate(elementIndex, tabString);
    	}
    	System.out.println("Method Hashtable generateSelectElementHashBodySequenceEnd() generated");
    }

    public static void generateSelectElementTabBodySequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	
    	// Generate the select set method
		strings[6] += generateSelectHashTabSubSet(elementIndex, tabString);
   		
    	// Generate the select end method
    	strings[6] += generateElementEnd(tabString);    
    	
    	// Close Statement and ResultSet
    	strings[6] += generateSelectDatabaseRequestEnd(elementIndex, tabString);
    	
    	// Update index
    	if (elementIndex < 2) {
    		strings[6] += generateSelectFieldIndexUpdate(elementIndex, tabString);
    	}
    	System.out.println("Method Hashtable generateSelectElementHashBodySequenceEnd() generated");
    }

    
    public static void generateSelectElementTabBodyEnd(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// Generate the select set method
		strings[6] += generateSelectHashSubSet(elementTypeParam, parentIndex, elementIndex, tabString, forceId);
    	// Generate the select end method
    	strings[6] += generateElementEnd(tabString);    
    	System.out.println("Method Hashtable generateSelectElementTabBodyEnd() generated");
    }

    
    public static Hashtable getElementsHashtable() {
    	return elementHash_;
    }
    
    private static void generateSelectElementTab(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		// Generate the select body
		generateSelectElementTabStart(reader, element, xsdElementName, "", attList, 1, elementIndex, tabString, true);
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		generateSelectElementHashBodyAttribute(reader, element, "", "", (Attribute)attElements.next(), 1, elementIndex, tabString, true);
    	}
    	generateSelectElementTabBodyAttributeEnd(reader, element, "", attList, 1, elementIndex, tabString, true);
    	if (element.getSequence() != null && !element.getSequence().getElement().isEmpty()) {
			Iterator sequenceElements = element.getSequence().getElement().iterator();
			while (sequenceElements.hasNext()) {		
    			generateSelectElementTabBodySequenceBody(reader, element, xsdElementName, "", (ElementRef)sequenceElements.next(), 1, elementIndex, tabString, true);
			}
    	}
    	generateSelectElementTabBodySequenceEnd(reader, element, "", 1, elementIndex, tabString, true);
    	System.out.println("Method Hashtable generateSelectElementTab() generated");
    }
        
    public static void generateSelectElementTabStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
    	
    	String request = reader.getRequestFromName(elementType, TypesJAXBReader.SELECT_TYPE);   
    	
    	strings[6] += generateSelectStartTab(transform, request, xsdElementName, elementIndex, tabString);
    	
    	// generate field declaration
    	strings[6] += generateSelectHashtableTypeField(xsdElementName, elementIndex, tabString);
    	    	
    	System.out.println("Method String generateSelectElementHashStart() generated");
    }
    
    private static String generateSelectStartHash(SqlTransform transform, 
    		String request, 
    		String elementType, 
    		int index, 
    		String tabString) {
    	// Parse the select request field
    	request = transform.parseSelect(request);
    	// Add local variable initialization
    	String stringParam = tabString+"\t\t String request" + index + " = "+ "\"" + request + "\";\n";
    	
    	stringParam += tabString+"\t\t ResultSet rs" + index + " = execute(request" + index + ");\n";
    	if (index < 2) {
    		stringParam += tabString+"\t\t int index = 0;\n";
    	}
    	stringParam += tabString+"\t\t int index" + index + " = 0;\n";
    	// Add the hashtable where to add the values hashtables
    	stringParam += tabString+"\t\t Hashtable hash" + index + " = new Hashtable();\n";
    	if (objectsApproachMode) stringParam += tabString+"\t\t " + elementType + " element" + index + " = null;\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable subElemHash" + index + " = null;\n";
    	// Start of while iteration
    	stringParam += tabString+"\t\t while (rs" + index + ".next()) {\n";
    	
    	// Declaration of object or hashtable
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + index + " = new " + elementType + "();\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + index + " = new Hashtable();\n";
    	
    	System.out.println("Declaration and initialization of method public Hashtable select() generated");
    	return stringParam;
    }
    
    private static String generateSelectStartTab(SqlTransform transform, String request, String elementType, int index, String tabString) {
    	// Parse the select request field
    	request = transform.parseSelect(request);
    	// Add local variable initialization
    	String stringParam = tabString+"\t\t String request" + index + " = "+ "\"" + request + "\";\n";
    	
    	stringParam += tabString+"\t\t ResultSet rs" + index + " = execute(request" + index + ");\n";
    	// Moves cursor of the ResultSet to the last row
    	stringParam += tabString+"\t\t rs" + index + ".last();\n";
    	// Get the index of this last row that gives the row number
    	stringParam += tabString+"\t\t int tabSize" + index + " = rs" + index + ".getRow();\n";
    	// Moves the cursor to the front of this ResultSet object, just before the first row.
    	stringParam += tabString+"\t\t rs" + index + ".beforeFirst();\n";
    	stringParam += tabString+"\t\t int index" + index + " = 0;\n";
    	// Add the hashtable where to add the values hashtables
    	if (objectsApproachMode) stringParam += tabString+"\t\t "+elementType+"[] hashTab" + index + " = new "+elementType+"[tabSize" + index + "];\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable[] hashTab" + index + " = new Hashtable[tabSize" + index + "];\n";
    	if (objectsApproachMode) stringParam += tabString+"\t\t " + elementType + " element" + index + " = null;\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable subElemHash" + index + " = null;\n";
    	// Start of while iteration
    	stringParam += tabString+"\t\t while (rs" + index + ".next()) {\n";
    	
    	// Declaration of object or hashtable
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + index + " = new " + elementType + "();\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + index + " = new Hashtable();\n";
    	
    	System.out.println("Declaration and initialization of method public Hashtable select() generated");
    	return stringParam;
    }
    
    private static String generateSelectHashtableTypeField(String elementType, int index, String tabString) {
    	String stringParam = "";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\"type\", \"" + elementType + "\");\n\n";
    	return stringParam;    	
    }
    
    private static String generateUpdateHashtableTypeField(String elementType, String tabString) {
    	String stringParam = "";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t context.setProperty(\"type\", \"" + elementType +  "\");\n\n";
    	return stringParam;    	
    }
    
    private static String generateSelectField(String elementTypeName, String elementName, String fieldName, String fieldType,int index, String tabString) {
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + index + ".set"+upCaseFirstLetter(fieldName)+"(rs" + index + ".get" + fieldType + "(\"" + fieldName + "\"));\n";
    	if (hashtablesApproachMode) {
    		if ("String".equals(fieldType)) {
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+fieldName+"\", (String)rs" + index + ".get" + fieldType + "(\"" + fieldName + "\"));\n";
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+elementTypeName+"."+fieldName+"\", (String)rs" + index + ".get" + fieldType + "(\"" + fieldName + "\"));\n";
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+elementName+"."+fieldName+"\", (String)rs" + index + ".get" + fieldType + "(\"" + fieldName + "\"));\n\n";
    		}
    		else {
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+fieldName+"\", new "+getCastType(fieldType)+"(rs" + index + ".get" + fieldType + "(\"" + fieldName + "\")));\n";
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+elementTypeName+"."+fieldName+"\", new "+getCastType(fieldType)+"(rs" + index + ".get" + fieldType + "(\"" + fieldName + "\")));\n";
    			stringParam += tabString+"\t\t\t subElemHash" + index + ".put(\""+elementName+"."+fieldName+"\", new "+getCastType(fieldType)+"(rs" + index + ".get" + fieldType + "(\"" + fieldName + "\")));\n\n";
    		}
    		
    	}
    	return stringParam;
    }
    
    private static String generateSelectFieldFromTable(String fieldName, String fieldValue, String fieldType,int index, String tabString) {
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + index + ".set"+upCaseFirstLetter(fieldValue)+"(rs" + index + ".get" + fieldType + "(\"" + fieldValue + "\"));\n";
    	if (hashtablesApproachMode) {
    		if ("String".equals(fieldType))  stringParam += tabString+"\t\t\t subElemHash" + index + ".put("+fieldName+", (String)rs" + index + ".getObject(columnIndex" + index + "));\n";
    		else stringParam += tabString+"\t\t\t subElemHash" + index + ".put("+fieldName+", rs" + index + ".getObject(columnIndex" + index + "));\n";
    		
    	}
    	return stringParam;    	
    }
    
    private static void generateUpdatePropertiesField(
    		String xsdElementName,
    		String elementName, 
    		String fieldName,
    		String fieldType,
    		int index, 
    		String tabString) {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += "String.valueOf(element" + index + ".get"+upCaseFirstLetter(fieldName)+"())";
    	if (hashtablesApproachMode) stringParam += "String.valueOf(subElemHash" + index + ".get(\""+fieldName+"\"))";        
    	String completeName = elementName.toLowerCase() + "." + fieldName;
    	String value = transform.getValue(completeName);
    	if (value != null && !value.equalsIgnoreCase(completeName)) {
    		completeName = value;
    	}
    	String viewableName = elementName + "." + fieldName;
    	String xsdElementCompleteName = xsdElementName + "." + fieldName;
    	strings[7] += tabString+"\t\t\t context.setProperty(\""+fieldName+"\", " + stringParam + ");\n";
    	strings[7] += tabString+"\t\t\t context.setProperty(\""+viewableName+"\", " + stringParam + ");\n";
    	strings[7] += tabString+"\t\t\t context.setProperty(\""+xsdElementCompleteName+"\", " + stringParam + ");\n\n";
    }
    
    private static void generateInsertPropertiesField(
    		String xsdElementName,
    		String elementName, 
    		String fieldName,
    		String fieldType,
    		int index, 
    		String tabString) {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += "String.valueOf(element" + index + ".get"+upCaseFirstLetter(fieldName)+"())";
    	if (hashtablesApproachMode) stringParam += "String.valueOf(subElemHash" + index + ".get(\""+fieldName+"\"))";        
    	String completeName = elementName.toLowerCase() + "." + fieldName;
    	String value = transform.getValue(completeName);
    	if (value != null && !value.equalsIgnoreCase(completeName)) {
    		completeName = value;
    	}
    	String viewableName = elementName + "." + fieldName;
    	String xsdElementCompleteName = xsdElementName + "." + fieldName;
    	strings[8] += tabString+"\t\t\t context.setProperty(\""+fieldName+"\", " + stringParam + ");\n";
    	strings[8] += tabString+"\t\t\t context.setProperty(\""+viewableName+"\", " + stringParam + ");\n";
    	strings[8] += tabString+"\t\t\t context.setProperty(\""+xsdElementCompleteName+"\", " + stringParam + ");\n\n";
    }
    
    private static void generateDeletePropertiesField(
    		String xsdElementName,
    		String elementName, 
    		String fieldName,
    		String fieldType,
    		int index, 
    		String tabString) {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += "String.valueOf(element" + index + ".get"+upCaseFirstLetter(fieldName)+"())";
    	if (hashtablesApproachMode) stringParam += "String.valueOf(subElemHash" + index + ".get(\""+fieldName+"\"))";        
    	String completeName = elementName.toLowerCase() + "." + fieldName;
    	String value = transform.getValue(completeName);
    	if (value != null && !value.equalsIgnoreCase(completeName)) {
    		completeName = value;
    	}
    	String viewableName = elementName + "." + fieldName;
    	String xsdElementCompleteName = xsdElementName + "." + fieldName;
    	strings[9] += tabString+"\t\t\t context.setProperty(\""+fieldName+"\", " + stringParam + ");\n";
    	strings[9] += tabString+"\t\t\t context.setProperty(\""+viewableName+"\", " + stringParam + ");\n";
    	strings[9] += tabString+"\t\t\t context.setProperty(\""+xsdElementCompleteName+"\", " + stringParam + ");\n\n";
    }
    
    private static String generateSelectFieldIndex(int index, String tabString) {
    	// -1 because the ResultSet.getRow() begins at 1 and the array at 0
    	return tabString+"\t\t\t index" + index + " = rs" + index + ".getRow() - 1;\n";    	
    	
    }
    
    private static String generateUpdateFieldIndex(int index, String tabString) {
    	return tabString+"\t\t\t index" + index + "++;\n";    	
    	
    }
    
    private static String generateSelectFieldIndexWithPrecedingIndex(int index, String tabString) {
    	return tabString+"\t\t\t index" + index + " = rs" + index + ".getRow() + index - 1;\n";    	
    	
    }
    
    private static String generateSelectFieldIndexUpdate(int index, String tabString) {
    	return tabString+"\t\t index = index" + index + " + 1;\n\n";    	
    	
    }
    
    private static String generateSelectHashSubSet(String subElementName, int parentIndex, int elementIndex, String tabString, boolean forceId) {
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t hash" + (forceId ? parentIndex : elementIndex) + ".put(\"" + subElementName + "\"+index" + elementIndex + ", element" + elementIndex + ");\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t hash" + (forceId ? parentIndex : elementIndex) + ".put(\"" + subElementName + "\"+index" + elementIndex + ", subElemHash" + elementIndex + ");\n";        
    	return stringParam;
    }
    
    private static String generateSelectHashTabSubSet(int elementIndex, String tabString) {
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t hashTab" + elementIndex + "[index" + elementIndex + "] = element" + elementIndex + ";\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t hashTab" + elementIndex + "[index" + elementIndex + "] = subElemHash" + elementIndex + ";\n";        
    	return stringParam;
    }
    
    private static String generateSelectSubHashSet(String subElementName, int parentIndex, int elementIndex, String tabString) {
    	String stringParam = "";
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + parentIndex + ".set"+upCaseFirstLetter(subElementName)+"(hashTab" + elementIndex + ");\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + parentIndex + ".put(\"" + subElementName + "\", hashTab" + elementIndex + ");\n";        
    	return stringParam;
    }
    
    private static String generateElementEnd(String tabString) {
    	return tabString+"\t\t }\n\n";
    }
    
    private static String generateElementEndSimpleTab(String tabString) {
    	return tabString+"\t }\n\n";
    }
    
    public static String generateSelectReturn(String tabString, int index) {
    	return tabString+"\t\t return hash" + index + ";\n\n";
    }
    
    private static String generateSelectDatabaseRequestEnd(int index, String tabString) {
    	return tabString+"\t\t endExecute(rs" + index + ");\n\n";
    }
    
    public static String generateSelectDatabaseConnectionStart(String tabString) {
    	return tabString+"\t\t connect();\n\n";
    }
    
    public static String generateSelectDatabaseConnectionEnd(String tabString) {
    	return tabString+"\t\t disconnect();\n\n";
    }
    
    public static String generatePropertiesDeclaration(int index, String tabString) {
    	if (index == 1) {
    		return tabString + "\t\t Properties context = new Properties();\n";
    	} else {
    		return tabString + "\t\t context = new Properties();\n";
    	}
    }
    
    public static String generatePropertiesOnlyDeclaration(int index, String tabString) {
   		return tabString + "\t\t Properties context = new Properties();\n";
    }
    
    public static String generateTypeCheck(int index, String elementName, String tabString) {
    	String returnValue = "";
    	if (objectsApproachMode) {
    		returnValue += tabString + "\t\t\t if (o"+index+" instanceof "+elementName+") {;\n";
    		returnValue += tabString+"\t\t\t "+elementName+" element" + index + " = (" + elementName + ")o"+index+";\n";

    	}
    	if (hashtablesApproachMode) returnValue += tabString + "\t\t\t if (\""+elementName+"\".equals(subElemHash"+index+".get(\"type\"))) {\n";
    	return returnValue;
    }
    
    public static String generatePropertiesUpdate(String tabString) {
   		return tabString + "\t\t\t context = new Properties();\n";
    }
    
    private static void generateSelectPropertiesField(SqlTransform transform, String tableName, String elementName, String fieldName, String valueName, int index, String tabString) {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String completeName = elementName.toLowerCase() + "." + fieldName;
    	String value = transform.getValue(completeName);
    	if (value != null && !value.equalsIgnoreCase(completeName)) {
    		completeName = value;
    	}
    	String tableCompleteName = tableName + "+\".\"+" + fieldName;
    	// Add 
    	strings[6] += tabString+"\t\t\t context.setProperty("+fieldName+", rs" + index + ".getString(" + valueName + "));\n";
    	strings[6] += tabString+"\t\t\t context.setProperty("+tableCompleteName+", rs" + index + ".getString(" + valueName + "));\n";
    }
    
    private static String generatePropertiesAdditionalField(SqlTransform transform, String fieldAlias, String elementName, String fieldName, int index, String tabString) {
    	return tabString+"\t\t\t context.setProperty(\""+fieldAlias+"\", rs" + index + ".getString(\"" + fieldName + "\"));\n";
    }
    
    private static String generateUpdatePropertiesAdditionalField(SqlTransform transform, String fieldAlias, String elementName, String fieldName, int index, String tabString) {
    	String returnValue = "";
    	if (objectsApproachMode) returnValue += tabString+"\t\t\t context.setProperty(\""+fieldAlias+"\", String.valueOf(element" + index + ".get"+upCaseFirstLetter(fieldName)+"()));\n";
    	if (hashtablesApproachMode) returnValue += tabString+"\t\t\t context.setProperty(\""+fieldAlias+"\", String.valueOf(subElemHash" + index + ".get(\"" + fieldName + "\")));\n";
    	return returnValue;
    }
    
    private static String generatePropertiesAdditionalFieldList(SqlTransform transform, 
    		List attList, 
    		String elementName, 
    		int index,
    		String tabString) {
    	return generatePropertiesAdditionalFieldList(transform, attList, elementName, index, false, tabString);
    }
    private static String generatePropertiesAdditionalFieldList(SqlTransform transform, 
    		List attList, 
    		String elementName, 
    		int index,
    		boolean update,
    		String tabString) {
    	// Remove field keys that are already in the element attribute list
    	Properties props = (Properties)transform.getValues().clone();
    	Attribute attribute = null;
    	String attName = null;
    	for (int i=0; i < attList.size(); i++) {
    		attribute = (Attribute) attList.get(i);
    		attName = elementName.toLowerCase() + "." + attribute.getName();
    		// Remove id that have no aliases
    		if (props.containsKey(attName)) {
    			props.remove(attName);
    		}
    	}
    	
    	// Add the remaining field in the properties
    	String stringParam = "";
    	Iterator it = props.keySet().iterator();  
    	String propKey = null;
    	String fieldValue = null;
    	String fieldElementName = null;
    	int elemIndex = 0;
    	while (it.hasNext()) {
    		attName = (String)it.next();
    		propKey = ((String)props.get(attName));
    		fieldValue = propKey.substring(propKey.indexOf(".") +1, propKey.length());
    		elemIndex = propKey.indexOf(".");
    		if (elemIndex != -1) {
    			fieldElementName = propKey.substring(0, elemIndex);
    		} else {
    			fieldElementName = propKey;
    		}
    		// Add only the properties related to the current element
    		// because the SqlTransform is global to all the elements and their attributes.
    		if (fieldElementName.equalsIgnoreCase(elementName)) {
    			if (update) stringParam += generateUpdatePropertiesAdditionalField(transform, attName,  elementName, fieldValue, index, tabString);
    			else stringParam += generatePropertiesAdditionalField(transform, attName,  elementName, fieldValue, index, tabString);
    		}
    	}
    	return stringParam;
    }
    
      
    public static void generateUpdateElementHashStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	updateRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.UPDATE_TYPE);   
    	
    	strings[7] += generateUpdateStartHash(elementIndex, tabString);
    	
		strings[7] += generateTypeCheck(parentIndex, xsdElementName, tabString);

		strings[7] += generatePropertiesUpdate(tabString+"\t");
    	
    	// generate field declaration
    	strings[7] += generateUpdateHashtableTypeField(xsdElementName, tabString+"\t");
    }
    
    public static void generateUpdateElementHashBodyAttribute(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String realXsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateUpdatePropertiesField(realXsdElementName, elementType, attName, getType(attType), elementIndex, tabString);
    }
    
    public static void generateUpdateElementHashBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();

    	// Generete field add for properties that are not present in the element attribute list
		strings[7] += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, true, tabString);
    	
		strings[7] += generateUpdateRequest(transform, updateRequestValue, elementType, elementIndex, tabString);
    	
		strings[7] += generateUpdateFieldIndex(elementIndex, tabString);                	
    }
    
    public static void generateUpdateElementHashBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateUpdateElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, "\t\t", false);
		getElementsHashtable().put(subElement.getName(), subElement);
		generateUpdateElementHashSequenceEnd(reader, element, "", 1, elementIndex, "\t", true);
    }
    
    public static void generateUpdateElementHashSequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {    	
    	// Generate the select end method
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);    	
		strings[7] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateElementHashSequenceEnd(Hashtable h) generated");
    }
    
    public static String generateUpdateElementHash(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String realXsdElementName,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId,
    		SqlTransform transform) throws Exception {
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	String request = reader.getRequestFromName(elementType, TypesJAXBReader.UPDATE_TYPE);   
    	
    	String selectString = generateUpdateStartHash(elementIndex, tabString);
    	
    	// generate field declaration
    	selectString += generateUpdateHashtableTypeField(xsdElementName, tabString);
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		Attribute attribute = (Attribute) attElements.next();
    		String attName = attribute.getName();
    		String attType = attribute.getType();
    		generateUpdatePropertiesField(realXsdElementName, elementType, attName, getType(attType), elementIndex, tabString);
    	}
    	
    	// Generete field add for properties that are not present in the element attribute list
    	selectString += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, true, tabString);
    	
    	selectString += generateUpdateRequest(transform, request, elementType, elementIndex, tabString);
    	
    	selectString += generateUpdateFieldIndex(elementIndex, tabString);            
    	
    	// generate code for sequence types
    	if (element.getSequence() != null && !element.getSequence().getElement().isEmpty()) {
    		Iterator sequenceElements = element.getSequence().getElement().iterator();
    		ElementRef ref = null;
    		String seqElementType = null;
    		ComplexType subElement = null;
    		List subElemAttList = null;
    		
    		while (sequenceElements.hasNext()) {
    			ref = (ElementRef)sequenceElements.next();
    			seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
    			subElement = reader.getComplexTypeFromName(seqElementType);
    			// Get the ComlexType attribute
    			subElemAttList = subElement.getAttribute();	                
    			generateUpdateElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, "\t", false);
    			getElementsHashtable().put(subElement.getName(), subElement);
    		}
    	}
    	
    	
    	// Generate the select end method
    	selectString += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateStart(Hashtable h) generated");
    	
    	return selectString;
    }
    
    private static String generateUpdateStartHashBefore(SqlTransform transform, 
    		String elementType, 
    		int index, 
    		String tabString) {
    	String stringParam = "";
    	if (index < 2) {
	    	stringParam = tabString+"\t\t int index" + index + " = 0;\n";
	    	// Add the hashtable where to add the values hashtables
	    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable subElemHash" + index + " = null;\n";
    	}
    	return stringParam;
    }
    	
    private static String generateUpdateStartHash(int index, 
		String tabString) {
    	String stringParam = "";
    	if (index < 2) {
	    	// Start of while iteration
	    	stringParam += tabString+"\t\t while (index" + index + " < hash.size()) {\n";
	    	
	    	// Declaration of object or hashtable
	    	if (objectsApproachMode) stringParam += tabString+"\t\t\t Object o" + index + " = hash.get(\"\"+index" + index + ");\n";
	    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + index + " = (Hashtable)hash.get(\"\"+index" + index + ");\n";
	    	
	    	System.out.println("Declaration and initialization of method public Hashtable generateUpdateStartHash() generated");
    	}
    	return stringParam;
    }
    
    private static String generateUpdateRequest(SqlTransform transform,
    		String request, 
    		String elementType, 
    		int index, 
    		String tabString) {
    	// Parse the select request field
    	request = transform.parseUpdate(request);
    	// Add local variable initialization
    	String stringParam = tabString+"\t\t\t String request" + index + " = "+ "\"" + request + "\";\n";
    	
    	stringParam += tabString+"\t\t\t executeUpdate(request" + index + ");\n";
    	return stringParam;
    }
    
    private static String generateUpdateStartTab(SqlTransform transform, 
    		String request, 
    		String elementType,
    		int parentIndex,
    		int index, 
    		String tabString) {
    	String stringParam = tabString+"\t\t int index" + index + " = 0;\n";
    	// Add the hashtable where to add the values hashtables
    	if (objectsApproachMode) stringParam += tabString+"\t\t "+elementType+"[] hashTab" + index + " = ("+elementType+"[])element" + parentIndex + ".get" + upCaseFirstLetter(elementType) + "();\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable[] hashTab" + index + " = (Hashtable[])subElemHash" + parentIndex + ".get(\"" + elementType + "\");\n";
    	if (objectsApproachMode) stringParam += tabString+"\t\t " + elementType + " element" + index + " = null;\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t Hashtable subElemHash" + index + " = null;\n";
    	// Start of while iteration
    	stringParam += tabString+"\t\t while (index" + index + " < hashTab" + index + ".length) {\n";
    	
    	// Declaration of object or hashtable
    	if (objectsApproachMode) stringParam += tabString+"\t\t\t element" + index + " = (" + elementType + ")hashTab" + index + "[index" + index + "];\n";
    	if (hashtablesApproachMode) stringParam += tabString+"\t\t\t subElemHash" + index + " = (Hashtable)hashTab" + index + "[index" + index + "];\n";
    	
    	System.out.println("Declaration and initialization of method public Hashtable select() generated");
    	return stringParam;
    }
    
    private static void generateUpdateElementTabStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	updateRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.UPDATE_TYPE);   
    	
    	strings[7] += generateUpdateStartTab(transform, updateRequestValue, xsdElementName, parentIndex, elementIndex, tabString);
    	// generate field declaration
    	strings[7] += generateUpdateHashtableTypeField(xsdElementName, tabString);
    	
    }
    
    private static void generateUpdateElementTabAttributeBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateUpdatePropertiesField(xsdElementName, elementTypeParam, attName, getType(attType), elementIndex, tabString);
    }
    
    private static void generateUpdateElementTabSequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateUpdateElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, tabString, false);
		getElementsHashtable().put(subElement.getName(), subElement);
    }
    
    private static void generateUpdateElementTabBodySequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// Generate the select end method
		strings[7] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateStart(Hashtable h) generated");
    }
    
    private static void generateUpdateElementTab(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
    	generateUpdateElementTabStart(reader, element, xsdElementName, "", attList, parentIndex, elementIndex, tabString);
    	
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		generateUpdateElementTabAttributeBody(reader, element, xsdElementName, elementTypeParam,(Attribute) attElements.next(), parentIndex, elementIndex, tabString, forceId);
    	}
    	
    	generateUpdateElementHashBodyAttributeEnd(reader, element, elementTypeParam, attList, parentIndex, elementIndex, tabString, forceId);            
    	
    	// generate code for sequence types
    	if (element.getSequence() != null && !element.getSequence().getElement().isEmpty()) {
    		Iterator sequenceElements = element.getSequence().getElement().iterator();
    		while (sequenceElements.hasNext()) {
    			generateUpdateElementTabSequenceBody(reader, element, xsdElementName, elementTypeParam, (ElementRef)sequenceElements.next(), parentIndex, elementIndex, tabString, forceId);
    		}
    	}
    	    	
    	// Generate the select end method
    	generateUpdateElementTabBodySequenceEnd(reader, element, elementTypeParam, parentIndex, elementIndex, tabString, forceId);
    }   
    
    public static void generateInsertElementHashStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	insertRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.INSERT_TYPE);   
    	
    	strings[8] += generateUpdateStartHash(elementIndex, tabString);
    	
		strings[8] += generateTypeCheck(parentIndex, xsdElementName, tabString);

		strings[8] += generatePropertiesUpdate(tabString+"\t");
    	
    	// generate field declaration
    	strings[8] += generateUpdateHashtableTypeField(xsdElementName, tabString+"\t");
    }
       
    public static void generateDeleteElementHashStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();    	                
    	
    	deleteRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.DELETE_TYPE);   
    	
    	strings[9] += generateUpdateStartHash(elementIndex, tabString);
    	
		strings[9] += generateTypeCheck(parentIndex, xsdElementName, tabString);

		strings[9] += generatePropertiesUpdate(tabString+"\t");
    	
    	// generate field declaration
    	strings[9] += generateUpdateHashtableTypeField(xsdElementName, tabString+"\t");
    }
       
    public static void generateInsertElementHashBodyAttribute(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String realXsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateInsertPropertiesField(realXsdElementName, elementType, attName, getType(attType), elementIndex, tabString);
    }
    
    public static void generateDeleteElementHashBodyAttribute(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String realXsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateDeletePropertiesField(realXsdElementName, elementType, attName, getType(attType), elementIndex, tabString);
    }
    
    public static void generateInsertElementHashBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();

    	// Generete field add for properties that are not present in the element attribute list
		strings[8] += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, true, tabString);
    	
		strings[8] += generateInsertRequest(transform, insertRequestValue, elementType, elementIndex, tabString);
    	
		strings[8] += generateUpdateFieldIndex(elementIndex, tabString);                	
    }
    
    public static void generateDeleteElementHashBodyAttributeEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	String elementType = element.getName();

    	// Generete field add for properties that are not present in the element attribute list
		strings[9] += generatePropertiesAdditionalFieldList(transform, attList, elementType, elementIndex, true, tabString);
    	
		strings[9] += generateDeleteRequest(transform, deleteRequestValue, elementType, elementIndex, tabString);
    	
		strings[9] += generateUpdateFieldIndex(elementIndex, tabString);                	
    }
    
    private static String generateInsertRequest(SqlTransform transform,
    		String request, 
    		String elementType, 
    		int index, 
    		String tabString) {
    	// Parse the select request field
    	request = transform.parseInsert(request);
    	// Add local variable initialization
    	String stringParam = tabString+"\t\t\t String request" + index + " = "+ "\"" + request + "\";\n";
    	
    	stringParam += tabString+"\t\t\t executeUpdate(request" + index + ");\n";
    	return stringParam;
    }
    
    private static String generateDeleteRequest(SqlTransform transform,
    		String request, 
    		String elementType, 
    		int index, 
    		String tabString) {
    	// Parse the select request field
    	request = transform.parseDelete(request);
    	// Add local variable initialization
    	String stringParam = tabString+"\t\t\t String request" + index + " = "+ "\"" + request + "\";\n";
    	
    	stringParam += tabString+"\t\t\t executeUpdate(request" + index + ");\n";
    	return stringParam;
    }
    
    public static void generateInsertElementHashBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateInsertElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, "\t\t", false);
		getElementsHashtable().put(subElement.getName(), subElement);
		generateInsertElementHashSequenceEnd(reader, element, "", 1, elementIndex, "\t", true);
    }
    
    public static void generateDeleteElementHashBodySequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateDeleteElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, "\t\t", false);
		getElementsHashtable().put(subElement.getName(), subElement);
		generateDeleteElementHashSequenceEnd(reader, element, "", 1, elementIndex, "\t", true);
    }
    
    public static void generateInsertElementHashSequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {    	
    	// Generate the select end method
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);    	
		strings[8] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateElementHashSequenceEnd(Hashtable h) generated");
    }
    
    public static void generateDeleteElementHashSequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception {    	
    	// Generate the select end method
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);    	
		strings[9] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateElementHashSequenceEnd(Hashtable h) generated");
    }
    
    private static void generateInsertElementTab(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
    	generateInsertElementTabStart(reader, element, xsdElementName, "", attList, parentIndex, elementIndex, tabString);
    	
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		generateInsertElementTabAttributeBody(reader, element, xsdElementName, elementTypeParam,(Attribute) attElements.next(), parentIndex, elementIndex, tabString, forceId);
    	}
    	
    	generateInsertElementHashBodyAttributeEnd(reader, element, elementTypeParam, attList, parentIndex, elementIndex, tabString, forceId);            
    	
    	// generate code for sequence types
    	if (element.getSequence() != null && !element.getSequence().getElement().isEmpty()) {
    		Iterator sequenceElements = element.getSequence().getElement().iterator();
    		while (sequenceElements.hasNext()) {
    			generateInsertElementTabSequenceBody(reader, element, xsdElementName, elementTypeParam, (ElementRef)sequenceElements.next(), parentIndex, elementIndex, tabString, forceId);
    		}
    	}
    	    	
    	// Generate the select end method
    	generateInsertElementTabBodySequenceEnd(reader, element, elementTypeParam, parentIndex, elementIndex, tabString, forceId);
    }   
    
    private static void generateDeleteElementTab(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
    	generateDeleteElementTabStart(reader, element, xsdElementName, "", attList, parentIndex, elementIndex, tabString);
    	
    	for(Iterator attElements = attList.iterator(); attElements.hasNext();){
    		generateDeleteElementTabAttributeBody(reader, element, xsdElementName, elementTypeParam,(Attribute) attElements.next(), parentIndex, elementIndex, tabString, forceId);
    	}
    	
    	generateDeleteElementHashBodyAttributeEnd(reader, element, elementTypeParam, attList, parentIndex, elementIndex, tabString, forceId);            
    	
    	// generate code for sequence types
    	if (element.getSequence() != null && !element.getSequence().getElement().isEmpty()) {
    		Iterator sequenceElements = element.getSequence().getElement().iterator();
    		while (sequenceElements.hasNext()) {
    			generateDeleteElementTabSequenceBody(reader, element, xsdElementName, elementTypeParam, (ElementRef)sequenceElements.next(), parentIndex, elementIndex, tabString, forceId);
    		}
    	}
    	    	
    	// Generate the select end method
    	generateDeleteElementTabBodySequenceEnd(reader, element, elementTypeParam, parentIndex, elementIndex, tabString, forceId);
    }   
    
    private static void generateInsertElementTabStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	insertRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.INSERT_TYPE);   
    	
    	strings[8] += generateUpdateStartTab(transform, insertRequestValue, xsdElementName, parentIndex, elementIndex, tabString);
    	// generate field declaration
    	strings[8] += generateUpdateHashtableTypeField(xsdElementName, tabString);
    	
    }
    
    private static void generateDeleteElementTabStart(
    		TypesJAXBReader reader, 
    		ComplexType element,
    		String xsdElementName,
    		String elementTypeParam,
    		List attList, 
    		int parentIndex,
    		int elementIndex,
    		String tabString) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// start to build update method of the SqlStorageManager
    	String elementType = element.getName();
    	
    	deleteRequestValue = reader.getRequestFromName(elementType, TypesJAXBReader.DELETE_TYPE);   
    	
    	strings[9] += generateUpdateStartTab(transform, deleteRequestValue, xsdElementName, parentIndex, elementIndex, tabString);
    	// generate field declaration
    	strings[9] += generateUpdateHashtableTypeField(xsdElementName, tabString);
    	
    }
    
    private static void generateInsertElementTabAttributeBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateInsertPropertiesField(xsdElementName, elementTypeParam, attName, getType(attType), elementIndex, tabString);
    }
    
    private static void generateDeleteElementTabAttributeBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		Attribute attribute, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
		String attName = attribute.getName();
		String attType = attribute.getType();
		generateDeletePropertiesField(xsdElementName, elementTypeParam, attName, getType(attType), elementIndex, tabString);
    }
    
    private static void generateInsertElementTabBodySequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// Generate the select end method
		strings[8] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateStart(Hashtable h) generated");
    }
    
    private static void generateDeleteElementTabBodySequenceEnd(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String elementTypeParam,
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	// Generate the select end method
		strings[9] += generateElementEnd(tabString);    
    	
    	System.out.println("Method Hashtable generateUpdateStart(Hashtable h) generated");
    }
    
    private static void generateInsertElementTabSequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateInsertElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, tabString, false);
		getElementsHashtable().put(subElement.getName(), subElement);
    }
    
    private static void generateDeleteElementTabSequenceBody(
    		TypesJAXBReader reader, 
    		ComplexType element, 
    		String xsdElementName,
    		String elementTypeParam,
    		ElementRef ref, 
    		int parentIndex,
    		int elementIndex,
    		String tabString,
    		boolean forceId) throws Exception{
		String seqElementType = upCaseFirstLetter(ref.getRef().substring(ref.getRef().indexOf(":")+1, ref.getRef().length()));
		ComplexType subElement = reader.getComplexTypeFromName(seqElementType);
		// Get the ComlexType attribute
		List subElemAttList = subElement.getAttribute();	                
		generateDeleteElementTab(reader, subElement, xsdElementName, seqElementType, subElemAttList, elementIndex, 1 + elementIndex, tabString, false);
		getElementsHashtable().put(subElement.getName(), subElement);
    }
    
    public static void setHashtableMode(boolean hashtableMode) {
    	if (hashtableMode) {
    		objectsApproachMode = false;
    		hashtablesApproachMode = true;
    	} else {
    		objectsApproachMode = true;
    		hashtablesApproachMode = false;
    	}
    }
    
    /**
     * Return the type of elementType.
     * 
     * @param elementType
     * @return id 
     */
    private static String getType(String elementType) {
    	if (elementType.toLowerCase().equals("xsd:string")) 
    		return "String";
    	if (elementType.toLowerCase().equals("xsd:char")) 
    		return "Char";
    	if (elementType.toLowerCase().equals("xsd:int")) 
    		return "Int";
    	if (elementType.toLowerCase().equals("xsd:long")) 
    		return "Long";
    	if (elementType.toLowerCase().equals("xsd:double")) 
    		return "Double";
    	if (elementType.toLowerCase().equals("xsd:short"))
    		return "Short";
    	if (elementType.toLowerCase().equals("xsd:byte")) 
    		return "Byte";
    	if (elementType.toLowerCase().equals("xsd:boolean"))
    		return "Boolean";
    	else return "";
    } 
    
    
    /**  
     *  Get the class cast name for types:
     *  	byte, short, int, double, long, char, boolean 
     */
    private static String getCastType(String type){
    	String tType = type.toLowerCase();
    	if (tType.equals("long")) return "Long";
    	if (tType.equals("int")) return "Integer";
    	if (tType.equals("double")) return "Double";
    	if (tType.equals("short")) return "Short";
    	if (tType.equals("byte")) return "Byte";
    	if (tType.equals("char")) return "Char";
    	if (tType.equals("boolean")) return "Boolean";
    	return type;
    }
    
    /**
     * @param s
     * @return
     */
    private static String upCaseFirstLetter(String s) {
    	char[] cTab = s.toCharArray();
    	if (cTab.length > 0) {
    		cTab[0] = (cTab[0]+"").toUpperCase().charAt(0);
    	}
    	return new String(cTab);
    }
    

    // t: tableau de strings a ecrire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructor
    // t[6]: write required implemented methods: init, start, end, onStartEvent, onJoinEvent, onEndEvent, onQuitEvent, onDataEvent
    // t[7]: treatMessage method (specific because handles custom types objects)
    // t[8]: traditional utils methods
    // t[9]: end of class
    public static void finalizeSqlStorage() throws IOException{
   	 	System.out.println("ServerGenerator: finalize server file...");
    	String[] strings = (String[]) stringsToWrite.get(Generator.getPath(Language.JAVA,false)+SQL_CLASS_NAME);
    	
    	if (!strings[0].equals("")) strings[0] += "\n";
    	if (!strings[1].equals("")) strings[1] += "\n";
    	if (!strings[2].equals("")) strings[2] += "\n\n";
    	if (!strings[3].equals("")) strings[3] += "\n";
    	if (!strings[4].equals("")) strings[4] = "\t// attributes\n"+strings[4]+"\n";
    	
    	if (!strings[5].equals(""))
    		strings[5] = strings[5]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[6].equals(""))
    		strings[6] = "\t// select method\n"+strings[6]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[7].equals(""))
    		strings[7] = "\t// update method\n"+strings[7]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";
    	
    	if (!strings[8].equals(""))
    		strings[8] = "\t// insert method\n"+strings[8]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[9].equals(""))
    		strings[9] = "\t// delete method\n"+strings[9]+"\t"+LanguageTranslator.closeBlock(Language.JAVA)+"\n\n";

    	if (!strings[10].equals(""))
    		strings[10] = strings[10]+"\n";
    	
   	 	System.out.println("SqlStorageGenerator: sql manager file finalized.");
    }
}
