/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.mega.moods.generator.LanguageTranslator.Language;
import org.w3._2001.xmlschema.ComplexType;


/**
 * MooDS Generator v2.0
 * @author Romain PELLERIN
 */
public class TypesGenerator {

    private static Vector classFiles = new Vector();
    private static Vector classNames = new Vector();
    private static Hashtable stringsToWrite = new Hashtable();
    private static boolean needsMulticastCommunicationMethods = false;
    private static boolean isVectorIncluded = false;
    private static int nbComlexArrayUsed = 0;
    private static int nbComlexTypeRetrieveUsed = 0;
     
    
    // t: tableau de strings a ecrire
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
	// t[3]: public attributes
    // t[4]: private attributes
    // t[5]: constructeur
    // t[6]: setters/getters
    // t[7]: end of class
    
    public static void init(){
    	classFiles = new Vector();
    	classNames = new Vector();
    	stringsToWrite = new Hashtable();
    	nbComlexArrayUsed = 0;
    	nbComlexTypeRetrieveUsed = 0;
    	ClassMaker.init();
    }
    
    public static void startGenerateClass(Language language, int encoding, String _package, String name, int currentTypeCommunicationModel) throws IOException{
    	attributesAddedForCompactMode.removeAllElements();
    	
    	if (Generator.isTypesClassesAlreadyGenerated) return;
    	
    	boolean isACommonClass = (language.equals(Language.JAVA)? true:false);
    	String path = Generator.getPath(language,isACommonClass);
    	String linkClass = path+name;
    	String[] strings = null;
	    
	   if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP)  ){
		   
		   
		    // strings to write table for that class
	    	strings = new String[8];
	    	System.out.println("1");
	    	ClassMaker.createClassEntry(language,linkClass,name,strings,isACommonClass);
	    	System.out.println("2");
		    classNames.add(linkClass);
	    
		    if (language.equals(Language.JAVA)  || language.equals(Language.FLASH)  ){
		    	strings[0] = LanguageTranslator.writePackage(language,_package)+"\n";//"package "+ _package+";\n"; // (for java)	
		    }
		    else if (language.equals(Language.CSHARP) )
		    	strings[0] = "";
	    	
		    if (language.equals(Language.JAVA)  || language.equals(Language.FLASH) )
		    	strings[1] = ""; // no imports
		    else if (language.equals(Language.CSHARP) ){
		    	strings[1] = LanguageTranslator.writeImport(language, "System")+"\n";
		    	strings[1] += LanguageTranslator.writeImport(language, "System.Collections")+"\n";
		    }
		    
		    if (language.equals(Language.CSHARP) ){
		    	strings[2] =  LanguageTranslator.writePackage(language,_package)+"\n"; // 
	        }
	        else
	        	strings[2] = ""; 
	        
		    strings[3] = ""; // 
	        
		    strings[4] = ""; // 
	        
		    strings[5] = ""; // 
	        
		    strings[6] = ""; // 
	        
		    strings[7] = ""; // 
	        	
	    	strings[0] += "/** \n"
	    			+" * MooDS Class File for "+((language.equals(Language.JAVA) )? "Java":"Flash")+".\n"
	    	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	    	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	    	 		+" */\n";

	    	strings[2] += LanguageTranslator.writeClassDeclaration(language,name,false,false,"");
	    	
	    	String[][] params = null;
	    	if (currentTypeCommunicationModel == Generator.UNICAST) params = new String[][]{{"short","_recipient"}};
	    	else if (currentTypeCommunicationModel == Generator.MULTICAST) params = new String[][]{{LanguageTranslator.arrayDeclaration(language,"short",1),"_recipients"}};
	    	
	    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(language,name,params)+"\n";//"\tpublic function "+name+"() {\n";
	    
	    	if (currentTypeCommunicationModel == Generator.UNICAST){
	    		strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipient","_recipient")+"\n";
	    	}
	    	else if (currentTypeCommunicationModel == Generator.MULTICAST){
	    		strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipients","_recipients")+"\n";
	    	}
	    	
	    	strings[7] = LanguageTranslator.closeBlock(language);
	    	
	    }
	    if (language.equals(Language.CPP) ){
	    	// strings to write table for that class
	    	strings = new String[15];
	    	ClassMaker.createClassEntry(language,linkClass,name,strings,isACommonClass);
		    classNames.add(linkClass);
	   
	    	strings[0] = ""; //	commentaires + macro
	        strings[1] = ""; // class declaration
	        strings[2] = ""; // private attributes
	        strings[3] = ""; // public methods
	        strings[4] = ""; // type
	        strings[5] = ""; // constructeur
	        strings[6] = ""; // constructeur without args
	        strings[7] = ""; // destructeur
	        strings[8] = ""; // setter & getter methods
	        strings[9] = ""; // equals
	        strings[10] = "";// encode
	        strings[11] = "";// encodeType
	        strings[12] = "";// decode 
	        strings[13] = "";// display
	        strings[14] = "";// end of class
	        	
	        // Commentaires
	    	strings[0] += "/** \n"
	    			+" * MooDS Class File for C++.\n"
	    	 		+" * @author Generated by MooDS Generator v"+Generator.MOODS_VERSION
	    	 		+" - "+Generator.MOODS_YEAR+" "+Generator.MOODS_TEAM+" \n"
	    	 		+" */\n";
	    	
	    	strings[0] += LanguageTranslator.writeMacro(name)+"\n";//#ifndef...#define
	    	strings[0] += LanguageTranslator.writeImport(Language.CPP, "Object.h")+"\n";//#include <Object.h>
	    	
	    	// class declaration
	    	strings[1] += LanguageTranslator.writeClassDeclaration(language,name,false,true,"Object");
	    	
	    	String[][] params = null;
	    	if (currentTypeCommunicationModel == Generator.UNICAST) params = new String[][]{{"short","_recipient"}};
	    	else if (currentTypeCommunicationModel == Generator.MULTICAST) params = new String[][]{{LanguageTranslator.arrayDeclaration(language,"short",1)+"&","_recipients"}};
	    	
	    	// type
	    	strings[4] += "\t"+LanguageTranslator.addGlobalVar(Language.CPP, false, true, true, true, ""+Generator.getIndexInt(), "byte", "TYPE");
	    	// constructor
	    	strings[5] += "\t"+LanguageTranslator.writeConstructorDeclaration(language,name,params)+"\n";
	    	
	    	if (currentTypeCommunicationModel == Generator.UNICAST) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipient","_recipient")+"\n";
	    	else if (currentTypeCommunicationModel == Generator.MULTICAST) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,"recipients","_recipients")+"\n";
	   
	    	// other constructor
	    	if (currentTypeCommunicationModel == Generator.UNICAST ||currentTypeCommunicationModel == Generator.MULTICAST)
	    		strings[6] += "\t"+LanguageTranslator.writeConstructorDeclaration(language,name,null)+"\n";
	    	
	    	// destructor
	    	strings[7] += "\t"+LanguageTranslator.writeDestructorDeclaration(name)+"\n";
	    	// equals method (equals params = MyClass myClass)
	    	String[][] equalsParams = new String[][]{{name,Generator.lowCaseFirstLetter(name)}};
	    	strings[9] += "\t"+LanguageTranslator.writeFunctionDeclaration(language, true, false, false, false,"boolean", "equals", equalsParams, false, null)+"\n";
	    	strings[9] += "\t\tif( ";
	    	// encode method
	    	String[][] attrs = {{"SocketStream&","ss"}};
	    	strings[10]  += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.CPP,
	    											false,false,false,true,"","encode", attrs,false,null)+"\n";
	    	
	    	// encodeType method
	    	// This method has exactly the same implementation for each generated classes, that's why all generated code
	    	// sis written in this method startGenerateClass 
	    	strings[11]  += "\t"+LanguageTranslator.writeFunctionDeclaration(Language.CPP,
	    											false,false,false,true,"","encodeType", attrs,false,null)+"\n";
	    	strings[11] += "\t\t"+ LanguageTranslator.writeBinaryType(Language.CPP, "byte", "TYPE","ss")+"\n";
	    	
	    	// decode method
	    	strings[12]+= CustomTypesGenerator.startToWriteDecodingMethodsForCpp(Generator.BINARY);
		    // pointer declaration
		    //strings[10]+= "\t\t"+LanguageTranslator.addLocalVar(LanguageTranslator.CPP, true, name+"()",true,"", name+"*", Generator.lowCaseFirstLetter(name))+"\n";
	    	// display method
		    strings[13]+= "\t"+LanguageTranslator.writeFunctionDeclaration(Language.CPP, true, false, false, true, "", "display", null, false, null)+"\n";
		    
		    strings[14] = LanguageTranslator.closeBlock(language);
	    	
	    }
    
	    stringsToWrite.put(linkClass,strings);
    	System.out.println("TypesGenerator: start to generate class named "+ path+name);
    }
    
    private static Vector attributesAddedForCompactMode = new Vector();
    
    // t[0]: packages
    // t[1]: imports
    // t[2]: class declaration
    // t[3]: attributes
    // t[4]: constructeur
    // t[5]: setters/getters
    // t[6]: end of class
    public static void addClassAttribute(Language language, boolean isTable, int dimension, boolean isCompactMode, String className, String attributeName, String type) throws IOException {
  
    	boolean isACommonClass = (language.equals(Language.JAVA));
    	String path = Generator.getPath(language,isACommonClass);
    	String[] strings = (String[]) stringsToWrite.get(path+className);
    	System.out.println("path: "+path+className+" strings: "+strings);
    	String lowAttributeName = Generator.lowCaseFirstLetter(attributeName);
    	
    	
    	// attribute already added
    	if (isCompactMode && attributesAddedForCompactMode.contains(attributeName)) return;
    	
	    String languageType = (isTable? LanguageTranslator.arrayDeclaration(language,LanguageTranslator.getType(language,type),dimension):LanguageTranslator.getType(language,type));
	    
	    if(language.equals(Language.JAVA)  ||language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){

		    // attribute
		    strings[3] += "\t"+LanguageTranslator.addGlobalVar(language,false,false,false,false,"",languageType,lowAttributeName)+"\n";//"\t"+javaType+" "+lowAttributeName+";\n";
		    	
		    // constructor
		    if (!isTable && !((attributeName.equals("recipient") || attributeName.equals("recipients")))) strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,LanguageTranslator.getAttributesInitiator(language,languageType))+"\n"; //lowAttributeName+" = "+getJavaAttributesInitiator(javaType)+";\n";
		   
		    
		    // getter
		    strings[6] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,false,languageType,"get"+Generator.upCaseFirstLetter(attributeName),null,false,null)+"\n";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\n";
		    strings[6] += "\t\t"+LanguageTranslator.returnValue(language,lowAttributeName)+"\n";// return "+lowAttributeName+";\n";
		    strings[6] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";// }\n\n";
		    	
		    // setter
		    String[][] params = {{languageType,"_local_var"}};
		    strings[6] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,languageType,"set"+Generator.upCaseFirstLetter(attributeName),params,false,null)+"\n";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\n";
		    strings[6] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,"_local_var")+"\n";// return "+lowAttributeName+";\n";
		    strings[6] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";// }\n\n";

	    }
	    if(language.equals(Language.CPP) ){
	    	// checks if the methods are needed 
	    	if(lowAttributeName.equals("recipients")){
	    		needsMulticastCommunicationMethods = true;
	    	}
	    	// For the multicastRecipientsCommunication case
	    	if(type.contains("vector") && !isVectorIncluded){
	    		//#include <vector>
	    		strings[0] += LanguageTranslator.writeImport(Language.CPP, "vector")+"\n";
	    		isVectorIncluded = true;
	    	}
	    	//imports of vector and complexType
	    	if(languageType.contains("vector") && !isVectorIncluded){
	    		//#include <vector>
	    		strings[0] += LanguageTranslator.writeImport(Language.CPP, "vector")+"\n";
	    		isVectorIncluded = true;
	    		//#include "_.h" if type isn't a simple type 
	    		if(!LanguageTranslator.isPrimitiveCppType(type)){
	    			strings[0] += LanguageTranslator.writeImport(Language.CPP, type+".h")+"\n";	
	    		}
	    	}
	    	//imports of complexType
	    	if(!LanguageTranslator.isPrimitiveCppType(type)){
    			strings[0] += LanguageTranslator.writeImport(Language.CPP, type+".h")+"\n";	
    		}
	    	
		    // attribute
		    strings[3] += "\t"+LanguageTranslator.addGlobalVar(language,false,false,false,false,"",languageType,lowAttributeName)+"\n";
		    	
		    // constructor
		    if (!isTable && LanguageTranslator.isPrimitiveCppType(type) 
		    	&&!((attributeName.equals("recipient") || attributeName.equals("recipients")))){
		    		strings[5] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,LanguageTranslator.getAttributesInitiator(language,languageType))+"\n"; 
		    		//lowAttributeName+" = "+getJavaAttributesInitiator(javaType)+";\n";
		    }
		    	
		    
		    // other constructor
		    // only if strings[6] is used by startGeberateClass()
		    if (!strings[6].equals("") 
		    	&& !((attributeName.equals("recipient") || attributeName.equals("recipients")))){
		    		strings[6] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,LanguageTranslator.getAttributesInitiator(language,languageType))+"\n";
		    		//lowAttributeName+" = "+getJavaAttributesInitiator(javaType)+";\n";
		    }
		    
		    // getter
		    strings[8] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,false,languageType,"get"+Generator.upCaseFirstLetter(attributeName),null,false,null)+"\n";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\n";
		    strings[8] += "\t\t"+LanguageTranslator.returnValue(language,lowAttributeName)+"\n";// return "+lowAttributeName+";\n";
		    strings[8] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";// }\n\n";
		    	
		    // setter
		    // if type is vector or another complex type, we precise that we use reference and that vector stays const 
		    String[][] params = {{(languageType.contains("vector") || !LanguageTranslator.isPrimitiveCppType(languageType))?("const "+languageType+"&"):languageType,"_local_var"}};
		    strings[8] += "\t"+ LanguageTranslator.writeFunctionDeclaration(language,true,false,false,true,languageType,"set"+Generator.upCaseFirstLetter(attributeName),params,false,null)+"\n";//public "+javaType+" get"+Generator.upCaseFirstLetter(attributeName)+"(){\n";
		    strings[8] += "\t\t"+LanguageTranslator.affectValueToVar(language,lowAttributeName,"_local_var")+"\n";// return "+lowAttributeName+";\n";
		    strings[8] += "\t"+LanguageTranslator.closeBlock(language)+"\n\n";// }\n\n";
		    if(needsMulticastCommunicationMethods){
		    	strings[8] +="\t"+CustomTypesGenerator.writeMulticastCommunicationModelMethods(Language.CPP,Generator.BINARY)+"\n";
		    }
		    
		    // equals
		    String[] itemsToEquals = new String[]{lowAttributeName, Generator.lowCaseFirstLetter(className)+"."+lowAttributeName};
		    strings[9] += LanguageTranslator.testEqualType(Language.CPP, true, type, itemsToEquals)+" && ";
		    // encode
		    if(needsMulticastCommunicationMethods){
		    	String[]paramsMulticast = {"ss"};
		    	strings[10] += "\t\t"+LanguageTranslator.callMethod(Language.CPP, false, "", "multicastRecipientsEncoding", paramsMulticast)+";\n";    	
		    }
		    if(!languageType.contains("vector")){
		    	//ss.writeType(value);
		    	if(LanguageTranslator.isPrimitiveCppType(type)){
		    		strings[10] += "\t\t"+LanguageTranslator.writeBinaryType(Language.CPP,type, lowAttributeName, "ss")+"\n";
		    	}else{
		    		//value.encode(ss);
		    		String[]paramsEncode = {"ss"};
		    		strings[10] += "\t\t"+LanguageTranslator.callMethod(Language.CPP, true, lowAttributeName, "encode", paramsEncode)+";\n";
		    	}		    	
		    }else if (!lowAttributeName.equals("recipients")){
		    	// should be done in CustomTypesGenerator.generateEncodingArrayType()
		    	//ss.writeInt(tab.size());
		    	strings[10] += "\t\t"+ LanguageTranslator.writeBinaryType(Language.CPP, "int", LanguageTranslator.arrayLength(Language.CPP, lowAttributeName),"ss")+"\n";
		    	strings[10] += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0",LanguageTranslator.arrayLength(Language.CPP, lowAttributeName))+"\n";
		    	if(LanguageTranslator.isPrimitiveCppType(type)){
		    		//ss.writeType(tab[i]);
		    		strings[10] += "\t\t\t"+LanguageTranslator.writeBinaryType(Language.CPP, type, 
		    								LanguageTranslator.arrayAccess(Language.CPP, lowAttributeName, "i"), "ss")
		    								+"\n\t\t"+LanguageTranslator.closeBlock(Language.CPP)+"\n";
		    	}else{
		    		//tab[i].encode(ss);
		    		String[] ss = {"ss"};
			    	strings[10] += "\t\t\t"+LanguageTranslator.callMethod(Language.CPP, 
			    											   true, 
			    											   	LanguageTranslator.arrayAccess(Language.CPP, lowAttributeName, "i"), 
			    											   "encode",
			    											   ss)+";\n"+"\t\t"+LanguageTranslator.closeBlock(Language.CPP)+"\n";	
		    	}
		    	
		    }
		    
		    
		    // decode
		    String decode = LanguageTranslator.readBinaryType(Language.CPP, type, "ss");
		    if(needsMulticastCommunicationMethods){
		    	// declares _recipients
		    	strings[12]+= "\t\t"+LanguageTranslator.addLocalVar(Language.CPP,
		    														false,
		    														"",
		    														"std::vector<short>", 
		    														"_recipients")+"\n";
		    	String[] param = new String[]{"ss", "_recipients"};
		    	// gets _recipients
		    	strings[12]+= "\t\t"+LanguageTranslator.callMethod(Language.CPP,false,"","multicastRecipientsDecoding", param)+";\n";
		    	// affects _recipients
		    	param = new String[]{"_recipients"};
		    	strings[12]+= "\t\t"+LanguageTranslator.callMethod(Language.CPP,false,"", "setRecipients", param)+";\n";
		    }
		    // pointer declaration

		    if(!attributeName.equals("recipients") && !languageType.contains("vector")){
		    	if(LanguageTranslator.isPrimitiveCppType(type)){
		    		//setAttribute(ss.readType());
		    		strings[12]+= "\t\t"+"set"+Generator.upCaseFirstLetter(attributeName)+"("+decode+");\n";	
		    	}else{
		    		//Object obj_0;
		    		strings[12]+="\t\t"+LanguageTranslator.addLocalVar(Language.CPP, false, "", type, lowAttributeName+"_"+nbComlexTypeRetrieveUsed)+"\n";
		    		//obj_0.decode(ss);
		    		String[] ss = {"ss"};
		    		strings[12]+="\t\t"+LanguageTranslator.callMethod(Language.CPP, true, lowAttributeName+"_"+nbComlexTypeRetrieveUsed, "decode", ss)+";\n";
		    		//setObject(obj_0);
		    		strings[12]+= "\t\t"+"set"+Generator.upCaseFirstLetter(attributeName)+"("+lowAttributeName+"_"+nbComlexTypeRetrieveUsed+");\n";
		    		nbComlexTypeRetrieveUsed++;
		    	}
		    }// TODO Should be placed in CustomTypeGenerator.generateArrayDecodingType()
		    else if(!attributeName.equals("recipients") && languageType.contains("vector")){
		    	strings[12]+= "\t\t"+LanguageTranslator.addLocalVar(Language.CPP,
		    			false,
		    			"",
		    			LanguageTranslator.arrayDeclaration(Language.CPP, type, 1), 
		    			"table_"+nbComlexArrayUsed)+"\n";
		    	String[] readsize = {LanguageTranslator.readBinaryType(Language.CPP, "int", "ss")};
		    	strings[12] += "\t\t"+LanguageTranslator.callMethod(Language.CPP, true,"table_"+nbComlexArrayUsed , "resize", readsize)+";\n";
		    	strings[12] += "\t\t"+LanguageTranslator.forLoopDeclaration(language,"i","0",LanguageTranslator.arrayLength(Language.CPP, "table_"+nbComlexArrayUsed))+"\n";
		    	// for simple type vector
		    	if(LanguageTranslator.isPrimitiveCppType(type)){
		    		//table_[i] = ss.readType();
		    		strings[12] += "\t\t\t"+LanguageTranslator.affectValueToVar(Language.CPP, 	
		    				LanguageTranslator.arrayAccess(Language.CPP, "table_"+nbComlexArrayUsed, "i"),
		    				LanguageTranslator.readBinaryType(Language.CPP, type, "ss"));		
		    		// for complex type vector
		    	}else{
		    		//table_[i].decode(ss);
		    		String[] ss = {"ss"};
		    		strings[12] += "\t\t\t"+LanguageTranslator.callMethod(Language.CPP, 
		    				true, 
		    				LanguageTranslator.arrayAccess(Language.CPP, "table_"+nbComlexArrayUsed, "i"), 
		    				"decode",
		    				ss)+";";
		    	}
		    	strings[12] += "\n"+"\t\t"+LanguageTranslator.closeBlock(Language.CPP)+"\n";
		    	String[] tableName = {"table_"+nbComlexArrayUsed};
		    	strings[12] += "\t\t"+LanguageTranslator.callMethod(Language.CPP, false, "", "set"+CustomTypesGenerator.upCaseFirstLetter(attributeName), tableName)+";\n";
		    	// For the case where at least 2 vectors are presents, so they don't have the same name
		    	nbComlexArrayUsed++;
		    }


		    //Cast in int for byte to display value
		    strings[13]+="\t\t"+"std::cout << \""+attributeName+" = \" << "+((languageType.equals("char"))?"(int) ":"")+lowAttributeName+" << std::endl;\n"; 
		    // reiniatizes booleans
		    needsMulticastCommunicationMethods = false;
		    isVectorIncluded = false;
	    }

    	if (isCompactMode) attributesAddedForCompactMode.add(attributeName);
    	stringsToWrite.put(path+className,strings);
    	System.out.println("TypesGenerator: attribute ("+ type +","+ attributeName +") for "+ className +" class added");
    }
    

	
    public static void finalize(Language language, int i) throws IOException{
    	System.out.println("TypesGenerator: finalize "+classNames.get(i)+"...");
    	String[] strings = (String[]) stringsToWrite.get(classNames.get(i));
    
    	// t[0]: packages
        // t[1]: imports
        // t[2]: class declaration
        // t[3]: attributes
        // t[4]: constructeur
        // t[5]: setters/getters
        // t[6]: end of class
    	if(language.equals(Language.JAVA)  || language.equals(Language.FLASH)  || language.equals(Language.CSHARP) ){
        	if (!strings[0].equals("")) strings[0] += "\n";
        	if (!strings[1].equals("")) strings[1] += "\n";
        	
        	if (!strings[2].equals("")) strings[2] += "\n\n";
        	if (!strings[3].equals("")) strings[3] += "\n";
        	if (!strings[4].equals(""))
        		strings[4] = "\t// attributes\n"+strings[4];
        	if (!strings[5].equals(""))
        		strings[5] = "\t// constructor\n"+strings[5]+"\t"+LanguageTranslator.closeBlock(language)+"\n\n";	
        	if (!strings[6].equals(""))
        		strings[6] = "\t// setter/getter methods\n"+strings[6]+"\n";
        	if (!strings[7].equals("")) {
        		if (language.equals(Language.JAVA)  || language.equals(Language.FLASH) )
        			strings[7] = strings[7]+"\n";
        		else if (language.equals(Language.CSHARP) ) 
        			strings[7] = strings[7]+"\n"+LanguageTranslator.closeBlock(language)+"\n";
        	}
    	}
    	if(language.equals(Language.CPP) ){
        	if (!strings[0].equals("")){
        		// deleteDoubleInclude
        		String delete = LanguageTranslator.deleteDoubleIncludes(strings[0]);
        		strings[0] = delete+"\n";
        	}
        	if (!strings[1].equals("")) strings[1] += "\n{\n";
        		strings[2] += "\t//attributes"+"\n\tprivate:\n";
        	if (!strings[3].equals("")) strings[3] += "\n\t//methods"+"\n\tpublic:\n";
        	if (!strings[4].equals(""))
        		strings[4] = "\t// type\n"+strings[4]+"\n\n";
        	if (!strings[5].equals(""))
        		strings[5] = "\t// constructor\n"+strings[5]+"\t"+LanguageTranslator.closeBlock(language)+"\n\n";
        	if (!strings[6].equals(""))	
        		strings[6] = "\t// other constructor\n"+strings[6]+"\n\t"+LanguageTranslator.closeBlock(language)+"\n\n";
        	if (!strings[7].equals(""))
        		strings[7] = "\t// destructor\n"+strings[7]+"\t{\n"+"\t"+LanguageTranslator.closeBlock(language)+"\n\n";		
        	if (!strings[8].equals(""))
        		strings[8] = "\t// setter/getter methods\n"+strings[8];
        	// TODO handle equals for array and complex type
        	if (!strings[9].equals("")){
        		strings[9]="";/*
        		// Suppression des deux derniers "&&" en trop
        		int newSize = strings[9].length() - 4;
        		String tab = strings[9].substring(0,newSize);
        		// commentaire
        		strings[9] = "\t// equals method\n"+tab;
        		// retour de la fonction
        		strings[9] += ")\n\t\t"+"return true;\n\t}\n\n";*/
        	}	
        	if (!strings[10].equals("")) 
        		strings[10] = "\t// encode methods\n"+strings[10]+"\n\t"+LanguageTranslator.closeBlock(language)+"\n\n";
        	if (!strings[11].equals("")) 
        		strings[11] += "\n\t"+LanguageTranslator.closeBlock(language)+"\n\n";
        	if (!strings[12].equals("")){
        		// get the name of the class with low case for first letter
        		strings[12] = "\t// decode method\n"+strings[12]+"\n\t"+LanguageTranslator.closeBlock(language)+"\n\n";
        	}	
        	// TODO handle display for array and complex type
        	if (!strings[13].equals(""))
        		//strings[12] = "\t// display method\n"+strings[12]+"\n\t"+LanguageTranslator.closeBlock(language)+"\n";
        		strings[13] = "";  
        	
        	strings[14] += (!strings[14].equals(""))?";\n"+LanguageTranslator.writeEndOfFile() :"\n"+LanguageTranslator.writeEndOfFile();
        	
        }
    	System.out.println("TypesGenerator: "+classNames.get(i)+" finalized");
    }
    
    public static void finalizeTypes(Language language) throws IOException{
    	 for (int i=0; i<classNames.size(); i++) {
    		 finalize(language,i);
         }
    	 System.out.println("TypesGenerator: types class finalized.");
    }
}
