#ifndef __SocketStream_h
#define __SocketStream_h

#ifndef WIN32
#	include <sys/types.h>
#	include <sys/socket.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <netdb.h>

#	include <unistd.h>
#else
#	include <WinSock2.h>
#	pragma comment(lib, "ws2_32.lib")
#endif

#include "SocketException.h"
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

#define L_ENDIAN	0
#define B_ENDIAN    1

const int MAXHOSTNAME = 200;
const int MAXCONNECTIONS = 5;
const int MAXRECV = 500;


// This class mix the socket and the stream functions
class SocketStream
{
	public:
	
		// Constructor
		SocketStream();
		// Destructor
		virtual ~SocketStream();
		// Bind the socket
		bool connect ( const std::string host, const int port);
		bool isConnected() { return connected; }
		bool close();
		
		// Advanced data transmission methods for output
		void writeBool(bool);
		void writeByte(char);
		void writeShort(short int);
		void writeInt (int);
		void writeLong(long long);
		void writeFloat(float);
		void writeDouble(double);
		void writeUTF(std::string);
		
		// Advanced data transmission methods for input
		bool readBool() const;
		char readByte() const;
		short readShort() const;
		unsigned short readUnsignedShort() const;
		int readInt() const;
		long long readLong() const;
		float readFloat() const;
		double readDouble() const;
		std::string readUTF() const;
		
		
	private:
		
		// Client initialization
		bool create();
		bool connected;	

		// Basic output data transimission methods (same methods as those used by ostream)
		bool write ( const char* tab, int size ) const;
		bool put ( char ) const;		

		// Basic input data transimission methods (same methods as those used by istream)
		int read (char* tab, int size ) const;	
		char peek() const;

		static int endian();

		// attributes
#ifndef WIN32
		bool is_valid() const { return m_sock != -1; }

		int m_sock;
		sockaddr_in m_addr;	
#else
		static size_t	mNbSockets;
		WSADATA			mWSAData;
		SOCKET			m_sock;
		SOCKADDR_IN		mSocketAddr;
#endif
};
#endif
