/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP
 *
 * Copyright (C) 2004,2005 CNAM/INT - 2006 CNAM/INT/Filao
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin, Chanty Lim
 */
package org.mega.moods.generator.sql;

import java.util.Properties;
import java.util.StringTokenizer;

/**
 * This class transform SQL requests by replacing variable between "[" and "]" by values found in context.
 * @author Chanthy LIM
 *
 */
public class SqlTransform {

	private static Properties properties_ = null;
	private static final String SEPARATOR = " \t\n\r\f=,;";
	private static final String COMMA_SEPARATOR = ",";
	private static final String SELECT_STRING = "select";
	private static final String SET_STRING = "set";
	private static final String FROM_STRING = "from";
	private static final String WHERE_STRING = "where";
	private static final String DOT_COMMA_STRING = ";";
	private static final String BRACKET_START = "[";
	private static final String BRACKET_END = "]";
	private static final String AS_SEPARATOR = "as";
	private static final String EQUAL_SEPARATOR = "=";
	private static final String AND_SEPARATOR = " and ";
	private static final String OR_SEPARATOR = " or ";
	private static final String AND_OR_REGEXP = "(" + AND_SEPARATOR + ")|(" + OR_SEPARATOR + ")";
	
	public SqlTransform() {
		properties_ = new Properties();
	}
	
	public String parseSelect(String inRequest) {
		return parse(inRequest, SELECT_STRING, FROM_STRING, AS_SEPARATOR, "context.getProperty");
	}
	
	public String parseUpdate(String inRequest) {
		return parse(inRequest, SET_STRING, WHERE_STRING, EQUAL_SEPARATOR, "context.getProperty");
	}
	
	public String parseInsert(String inRequest) {
		return parse(inRequest, SET_STRING, DOT_COMMA_STRING, EQUAL_SEPARATOR, "context.getProperty");
	}
	
	public String parseDelete(String inRequest) {
		return parse(inRequest, WHERE_STRING, DOT_COMMA_STRING, EQUAL_SEPARATOR, "context.getProperty");
	}
	
	private String parse(String inRequest, String start, String end, String fieldSeparator, String replaceBracket) {
		String returnRequest = new String(inRequest);
		// Get the column fields from the select statement
		// todo check that the select and from are present
		String fieldSequence = inRequest.substring(inRequest.indexOf(start) + start.length(), inRequest.length());		
		fieldSequence = fieldSequence.substring(0, fieldSequence.indexOf(end));
		StringTokenizer selectSt = new StringTokenizer(fieldSequence, COMMA_SEPARATOR);		
	    String token = null; 
	    String columnName = null;
	    String columnAlias = null;	    
		while (selectSt.hasMoreTokens()) {
			token = selectSt.nextToken().trim();
			// If the token contains an alias ('as') 
			if (containString(token, fieldSeparator)) {
				columnName = token.substring(0, token.indexOf(fieldSeparator)).trim();
				columnAlias = token.substring(token.indexOf(fieldSeparator) + fieldSeparator.length(), token.length()).trim();
				returnRequest = replaceBrack(returnRequest, columnAlias, replaceBracket);
				setValue(columnAlias, columnName);
			} else {
				setValue(token, token);
			}
	     }
		// replace in the where condition values in brackets with internal existing properties if there are
		if (containString(returnRequest, WHERE_STRING) && !WHERE_STRING.equals(start)) {
			String whereSequence = inRequest.substring(inRequest.indexOf(WHERE_STRING) + WHERE_STRING.length(), inRequest.length());		
			String[] whereArray = whereSequence.split(AND_OR_REGEXP);
			int length = whereArray.length;
			String seqParam = null;
			for (int i =0 ; i < length; i++) {
				seqParam = whereArray[i];
				StringTokenizer whereSequenceSt = new StringTokenizer(seqParam, SEPARATOR);		
				String toReplace = null;
				while (whereSequenceSt.hasMoreTokens()) {
					token = whereSequenceSt.nextToken();
					returnRequest = replaceBrack(returnRequest, token, replaceBracket);
				}
			}
		}
		return returnRequest;
	}
	
	private String replaceBrack(String returnRequest, String token, String replaceBracket) {
		if (containString(token, BRACKET_START) && containString(token, BRACKET_END)) {
			String toReplace = replaceBracketToken(token.trim(), replaceBracket);
			returnRequest = returnRequest.substring(0, returnRequest.indexOf(token)) + "\"+" +toReplace + "+\"" +returnRequest.substring(returnRequest.indexOf(token)+token.length(), returnRequest.length());
		}
		return returnRequest;
	}
	
	public String getValue(String param) {
		return properties_.getProperty(param);
	}

	public void setValue(String key, String param) {
		properties_.setProperty(removeBrackets(removeQuotes(key)), removeBrackets(removeQuotes(param)));
	}
	
	public Properties getValues() {
		return properties_;
	}

	public void resetValues() {
		properties_ = new Properties();
	}
	
	private String removeQuotes(String param) {
		if (param.startsWith("'") && param.endsWith("'")) {
			return param.substring(1, param.length()-1);
		} else {
			return param;
		}
	}
	
	private String removeBrackets(String param) {
		if (param.startsWith("[") && param.endsWith("]")) {
			return param.substring(1, param.length()-1);
		} else {
			return param;
		}
	}
	
	private String replaceBracketToken(String token, String replaceParam) {
		if (containString(token, BRACKET_START) && containString(token, BRACKET_END)) {
			int bracketStart = token.indexOf(BRACKET_START);
			int bracketEnd = token.indexOf(BRACKET_END);
			String begin = "";
			String end = "";
			if (bracketStart != 0) {
				begin = token.substring(0, bracketStart);
				if ("'".equals(begin)) {
					begin = "\"" + begin + "\"" + "+";
				}
			}
			if (bracketEnd != 0) {
				end = token.substring(bracketEnd+1, token.length());
				if ("'".equals(end)) {
					end =  "+" + "\"" + end + "\"";
				}
			}
			String toReplace = token.substring(bracketStart+1, bracketEnd);
			return begin +  replaceParam + "("+replaceBracketToken(toReplace, replaceParam)+")" + end;
		} else {
			return "\"" + token + "\"";
		}
	}
	
	boolean containString(String param, String toSearch) {
		if (param.indexOf(toSearch) != -1) {
			return true;
		}
		return false;
	}
}
