package tests;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;


class LittleServer implements Runnable{
	
	private static final LittleServer INSTANCE = new LittleServer();
	private static ServerSocket server;
	private static boolean serverConstructed = false;
	private Socket sock;
	private DataInputStream dis;
	private volatile boolean isRunning = false;
	private volatile boolean stop = false;

	private LittleServer(){}
	
	public static LittleServer getInstance(){
		return INSTANCE;
	}

	/*
	 * Creates server on localhost
	 * (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	synchronized public void run()
	{
		try{
			if(!serverConstructed){
				// Server socket on localhost
				server = new ServerSocket(9090,0,InetAddress.getByName("127.0.0.1"));
				serverConstructed = true;
			}

			synchronized (this) {
				isRunning = true;
				sock = server.accept();
				dis = new DataInputStream(sock.getInputStream());
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
	}

	synchronized DataInputStream getDataInputStream(){
		return dis;
	}
	
	/*
	 * Closes the socket and the attached DataInputStream.
	 * CAUTION : needs to be explicitly call if two successive connection are needed for tests,
	 * 			otherwise, server couldn't accept a new client on the same port.		
	 */
	void stop(){
		if(!stop){
			try{
				dis.close();
				sock.close();
				stop = true;
			} catch (IOException e){
				e.printStackTrace();
			}
		}
	}
		
	/*
	 * Launches server and C++ client, allowing test classes to get server's input stream.
	 * CAUTION : Server needs to be explicitly stop, invoquing stop() method in a try-finally block.
	 */
	void launchClientToServerCommunication(String executableFile) throws SecurityException{
		try{
			// Launchs server in threadServer ...
			launch();
			// then c++ client in a second thread
			launchCppClient(executableFile);
			
		}catch (IOException e1){
			e1.printStackTrace();
			this.stop();
		}catch (InterruptedException e2){			
			e2.printStackTrace();
			this.stop();
		}
		
	}	
	
	/*
	 * Runs this instance in threadServer.
	 */
	private void launch(){
		Object key = new Object();
		synchronized(key)
		{
			try{
				Thread threadServer = new Thread(this);
				threadServer.start();	
				while(!isRunning)		
				{
					key.wait(10);
				}
			}
			catch (InterruptedException e){
				e.printStackTrace();
			}
		}
	}
	
	/*
	 * Launches an executable and bloqs untill end of its execution.			 
	 */
	private static void launchCppClient(String executableFile) throws IOException, InterruptedException{
		try{
			Runtime runtime = Runtime.getRuntime();
			Process process = null;
			process = runtime.exec(executableFile);
	    	// Bloquing thread untill C++ client has finished its execution
			process.waitFor();
		} catch (IOException e1){
			e1.printStackTrace();
		} catch (InterruptedException e2){
			e2.printStackTrace();
		}
	}
}




