package tests;

import junit.framework.*;
import basicTypes.server.*;
import basicTypes.BasicTypes;
import basicTypes.BasicTypesLists;
import java.util.*;
import java.io.*;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;

/*
 * Tests exchange between a C++ client and a java server, both with communication mode set to "objects".
 */
public class TestBasicTypes extends TestCase
{
	// Paths to C++ clients executables compiled by cpptask in pom.xml
	public static final String basicTypesExecutableFile = "./target/execBasicTypes";
	public static final String basicTypesListsExecutableFile = "./target/execBasicTypesLists";

    public void testClientServerBasicTypesExchange () throws Exception{
    	// BasicTypes creation in local ...
        BasicTypes basictypes = makeBasicTypes();
        // and insertion in map
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();
        clientMap.put("0",basictypes);
        // Server initialisation 
        LittleServer server = LittleServer.getInstance();
        // Server waiting for client then C++ client connection 
    	server.launchClientToServerCommunication(basicTypesExecutableFile);
		DataInputStream dataInputStream = server.getDataInputStream();
		try{
			// Reception of C++ client's hashtable
			Hashtable serverDecodedMap = (new basicTypes.server.CustomTypes()).decodeData(dataInputStream);
			// then comparison
			compareBasicTypes(basictypes, (BasicTypes) serverDecodedMap.get("0"));
		}catch (IOException e){
			e.printStackTrace();
		}finally{
			server.stop();
		}
	}

    public void testClientServerBasicTypesListsExchange () throws Exception
    {
        // BasicTypesLists creation in local ...
        BasicTypesLists basictypeslists = makeBasicTypesLists();
        // and insertion in map
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();
        clientMap.put("0",basictypeslists);
        // Server initialisation 
        LittleServer server = LittleServer.getInstance();
        // Server waiting for client then C++ client connection 
    	server.launchClientToServerCommunication(basicTypesListsExecutableFile);
		DataInputStream dataInputStream = server.getDataInputStream();
		try{
			// Reception of C++ client's hashtable
			Hashtable serverDecodedMap = (new basicTypes.server.CustomTypes()).decodeData(dataInputStream);
			// then comparison
			compareBasicTypesLists(basictypeslists, (BasicTypesLists) serverDecodedMap.get("0"));
		}catch (IOException e){
			e.printStackTrace();
		}finally{
			server.stop();
		}
    }

    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    static private BasicTypes makeBasicTypes()
    {
        BasicTypes basictypes = new BasicTypes();
	    
	    basictypes.setAByte((byte)1);
	    basictypes.setAShort((short)1);
	    basictypes.setAInt((int)1);
	    basictypes.setALong(1l);
	    basictypes.setADouble(1d);
	    basictypes.setAFloat(1f);
	    basictypes.setABoolean(true);
	    basictypes.setAString(new String("Hello world !"));

        return basictypes;
    }
    
    static private BasicTypesLists makeBasicTypesLists()
    {
        BasicTypesLists basictypeslists = new BasicTypesLists();

    	basictypeslists.setAByteList(new byte[]{Byte.MIN_VALUE,Byte.MAX_VALUE});
	    basictypeslists.setAShortList(new short[]{Short.MIN_VALUE,Short.MAX_VALUE});
	    basictypeslists.setAIntList(new int[]{Integer.MIN_VALUE,Integer.MAX_VALUE});
	    basictypeslists.setAFloatList(new float[]{1f,2f,3f});
	    basictypeslists.setALongList(new long[]{Long.MIN_VALUE,Long.MAX_VALUE});
	    basictypeslists.setADoubleList(new double[]{1d,2d,3d});
	    basictypeslists.setABooleanList(new boolean[]{true, false});
	    basictypeslists.setAStringList(new String[]{"Hello","world !"});

        return basictypeslists;
    }
    
    static private void compareBasicTypes(BasicTypes basictypes1, BasicTypes basictypes2)
    {
        assertEquals(basictypes1.getAByte(), basictypes2.getAByte());
	    assertEquals(basictypes1.getAShort(), basictypes2.getAShort());
	    assertEquals(basictypes1.getAInt(), basictypes2.getAInt());
	    assertEquals(basictypes1.getALong(), basictypes2.getALong());
	    assertEquals(basictypes1.getADouble(), basictypes2.getADouble());
	    assertEquals(basictypes1.getAFloat(), basictypes2.getAFloat());
	    assertEquals(basictypes1.getABoolean(), basictypes2.getABoolean());
	    assertEquals(basictypes1.getAString(), basictypes2.getAString());
    }    


    static private void compareBasicTypesLists(BasicTypesLists basicTypesLists1, BasicTypesLists basicTypesLists2)
    {	
        assertTrue(Arrays.equals(basicTypesLists1.getAByteList(), basicTypesLists2.getAByteList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAShortList(), basicTypesLists2.getAShortList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAIntList(), basicTypesLists2.getAIntList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAFloatList(), basicTypesLists2.getAFloatList()));
        assertTrue(Arrays.equals(basicTypesLists1.getALongList(), basicTypesLists2.getALongList()));
        assertTrue(Arrays.equals(basicTypesLists1.getADoubleList(), basicTypesLists2.getADoubleList()));
        assertTrue(Arrays.equals(basicTypesLists1.getABooleanList(), basicTypesLists2.getABooleanList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAStringList(), basicTypesLists2.getAStringList()));
    }
}

   
