package resourcegenerator;

import java.io.*;
import java.net.*;
import org.apache.commons.cli.*;

public class CppRunner
{
	public static void main(String[] args)
	{
		CommandLine line = null;
		String outputFileName = "";
		String executableFile = "";

		CommandLineParser parser = new GnuParser();
		Options options = new Options();
		
		Option outputfile   	= OptionBuilder.withArgName( "output" )
                           			.hasArg()
                           			.withDescription(  "output file" )
                           			.create( "output" );

		Option executablefile 	= OptionBuilder.withArgName( "cppExecutable" )
                           			.hasArg()
                           			.withDescription(  "cpp executable path" )
                           			.create( "cppExecutable" );

		options.addOption(outputfile);
		options.addOption(executablefile);

		try {
		    // parse the command line arguments
		    line = parser.parse( options, args );
		}
		catch( ParseException exp ) {
		    System.err.println( "Parsing failed.  Reason: " + exp.getMessage() );
		}

		outputFileName = line.getOptionValue("output");
		executableFile = line.getOptionValue("cppExecutable");

		try
		{
			Runtime runtime = Runtime.getRuntime();
			System.out.println("start");

			Process process = null;
			LittleServer littleserver = null;
			File fileout = new File(outputFileName);
			littleserver = new LittleServer(fileout);
			Object key = new Object();
		
			synchronized(key)
			{
				Thread t = new Thread(littleserver);
				t.start();	
				while(!(littleserver.isRunning))		
				{
					key.wait(10);
				}
			}

			process = runtime.exec(executableFile);
			InputStream stream = process.getInputStream();
			InputStreamReader reader = new InputStreamReader(stream);
			BufferedReader buffered = new BufferedReader(reader);
			
			process.waitFor();

			// From Alain, just first line displayed
			// System.out.println(buffered.readLine());
			// From gab, trying to loop for displaying all outputs
			try{
				String newLine = null;
				while(true){
					newLine = buffered.readLine();
					if(newLine == null)
						break;
					System.out.println(newLine);
				}
			}catch (IOException e) {
				e.printStackTrace();
				buffered.close();
			}
			System.out.println("send stop message");
			littleserver.stop=true;

			synchronized(key)
			{
				while(littleserver.isRunning)		
				{
					key.wait(10);
				}
			}

			System.out.println("little server stopped");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}	

	static private class LittleServer implements Runnable
	{
		public volatile boolean stop=false;
		public volatile boolean isRunning=false;
		private File fileout;
		private FileWriter filewriter;

		public LittleServer(File file_out)
		{
			fileout = file_out;
			try
			{
				filewriter = new FileWriter(fileout);
			}
			catch(IOException ioe)
			{
				ioe.printStackTrace();
			}
		}

		synchronized public void  run()
		{			
			try
			{
				Socket sock = null;
				// Server socket on localhost
				ServerSocket server = new ServerSocket(9090,0,InetAddress.getByName("127.0.0.1"));
				System.out.println("server address : "+server.getInetAddress());

				synchronized(this)
				{
					isRunning=true;
					sock = server.accept();
				

					System.out.println("connection !");

					InputStream stream = sock.getInputStream();
					InputStreamReader reader = new InputStreamReader(stream);

					int code = reader.read();
					while(code != -1)
					{
						filewriter.write(code);
						code = reader.read();
					}

					filewriter.flush();
					filewriter.close();

					while (!stop)
					{
						System.out.println("running");
						wait(10);
					}	
						
					server.close();	
					isRunning = false;
				}	
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}

}
