#include "SocketStream.h"

#ifdef WIN32 
size_t	SocketStream::mNbSockets = 0;
#endif

// constructor
SocketStream::SocketStream()
{
#ifndef WIN32 
	m_sock = -1;
	// flush the buffer
	memset ( &m_addr,
			0,
			sizeof ( m_addr ) );	
#else
	if (!mNbSockets)
		WSAStartup(MAKEWORD(2, 0), &mWSAData);
	mNbSockets++;
	create();
#endif
}

// destructor
SocketStream::~SocketStream()
{
#ifndef WIN32
	if ( is_valid() ){
		::close ( m_sock );
		connected = false;
	}
#else
	close();

	mNbSockets--;
	if (mNbSockets)
		WSACleanup();
#endif
}

// Socket creation, used by constructor
bool SocketStream::create()
{
	connected = false;
#ifndef WIN32
	m_sock = socket ( AF_INET,
					 SOCK_STREAM,
					 0 );
	
	if ( ! is_valid() )
		return false;
	
	
	// TIME_WAIT - argh
	int on = 1;
	/*
	if ( setsockopt ( m_sock, SOL_SOCKET, SO_NOSIGPIPE, ( const char* ) &on, sizeof ( on ) ) == -1 )
		return false;

		SO_NOSIGPIPE n'est pas portable, on le remplace par MSG_NOSIGNAL
	*/
	
/*
	if ( setsockopt ( m_sock, SOL_SOCKET, MSG_NOSIGNAL, ( const char* ) &on, sizeof ( on ) ) == -1 )
		return false;
	
	struct timeval tv;
	tv.tv_sec = 5; 
	tv.tv_usec = 0;
	
	if ( setsockopt( m_sock, SOL_SOCKET, SO_RCVTIMEO,(struct timeval *)&tv,sizeof(struct timeval)) == -1){
		return false;
	}
	
	if ( setsockopt( m_sock, SOL_SOCKET, SO_SNDTIMEO,(struct timeval *)&tv,sizeof(struct timeval)) == -1){
		return false;
	}
*/
	return true;
#else
	return true;
#endif
}

// Bind the socket to host with mentionned port number
bool SocketStream::connect ( const std::string host, const int port )
{
	connected = false;
	// creates and connect socket to the server
	if ( ! SocketStream::create() )
    {
		throw SocketException ( "Could not create client socket." );
    }
#ifndef WIN32
	if ( ! is_valid() ) return false;
	
	m_addr.sin_family = AF_INET;
	m_addr.sin_port = htons ( port );
	
	if (inet_pton ( AF_INET, host.c_str(), &m_addr.sin_addr ) != 1)
	{
		throw SocketException ( "Could not decode address." );
		return false;
	}
	if (::connect ( m_sock, ( sockaddr * ) &m_addr, sizeof ( m_addr ) ) != 0)
	{
		throw SocketException ( "Could not connect to this address." );
		return false;
	}
#else
	mSocketAddr.sin_addr.s_addr	= inet_addr(host.c_str());
	mSocketAddr.sin_family		= AF_INET;
	mSocketAddr.sin_port		= htons(port);
	m_sock = socket(AF_INET, SOCK_STREAM, 0);
	if (::connect(m_sock, (SOCKADDR*)&mSocketAddr, sizeof(mSocketAddr)) != 0)
	{
		int toto = WSAGetLastError();
		throw SocketException ( "Could not connect." );
		return false;
	}
#endif
	connected = true;
	return true;
}

bool SocketStream::close()
{
#ifndef WIN32
	if ( is_valid() ){
		int status = ::close ( m_sock );
		if(status == 0){
			m_sock = -1;
			connected = false;
			printf("SocketStream closed \n");
			return true;
		}
	}
	return false;
#else
	connected = false;
	return closesocket(m_sock) == 0;
#endif
}

/*********************************** BASIC DATA TRANSMISSION METHODS  **************************************/

								/**********  OUTPUT METHODS ***********/

// Write a char* on the stream
bool SocketStream::write ( const char* tab, int size ) const
{
	//MSG_HAVEMORE remplace MSG_NOSIGNAL
	//int status = ::send ( m_sock, tab, size, MSG_HAVEMORE);
#ifdef WIN32
	int status = ::send ( m_sock, tab, size, 0);
#else
	int status = ::send ( m_sock, tab, size, 0);
	//int status = ::send ( m_sock, tab, size, MSG_HAVEMORE);
#endif
	if ( status == -1 )
    {
		throw SocketException ( "Connexion reseau perdue : deplacez vous!\n." );
		return false;
    }
	else
    {
		return true;
    }
}

// Write one char on the stream
bool SocketStream::put ( char c ) const
{
	const char *pointC = &c;
#ifdef WIN32
	int status = ::send ( m_sock, pointC, 1, 0);
#else
	int status = ::send ( m_sock, pointC, 1, 0);
#endif
	if ( status == -1 )
    {
		throw SocketException ( "Connexion reseau perdue : deplacez vous!\n." );
		return false;
    }
	else
    {
		return true;
    }
}

								/**********  INPUT METHODS ***********/

// Reads from the stream a number of bytes specified by the "size" param and puts it in tab
int SocketStream::read ( char* tab, int size ) const
{
#ifdef WIN32
	// renvoie 0 si connexion arretee "proprement"
	int readbytes = ::recv ( m_sock, tab, size, 0 );
#else
	int readbytes = ::recv ( m_sock, tab, size, 0);
#endif

	if ( readbytes == -1 || readbytes == 0) 
    {
		// Error, as expected.
		perror( "Perror recv" );
		printf( "...: %s\n", strerror( errno ) );
		throw SocketException ( "Connexion reseau perdue : deplacez vous!\n" );
		return 0;
    }
	else
    {
		return readbytes;
    }
}

char SocketStream::peek() const
{
	char value;
	while (read(&value,1) != 1)
			;
	
	return value;
}



/******************************** ADVANCED DATA TRANSMISSION METHODS ****************************/

								/**********  OUTPUT METHODS ***********/

void SocketStream::writeBool(bool b)
{
	put(b? 1 : 0);
}

void SocketStream::writeByte(char a)
{
	put(a);
}

void SocketStream::writeShort(short int s)
{
	short int *pointS = &s;
	char *res = new char[2];
	// si LITTLE ENDIAN, on inverse l ecriture pour etre compatible a JAVA
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 2; i++){
			res[i] =  ((char *)pointS)[1-i];
		}
		write(res,2);
	} else{
		write((char *)pointS,2);
	}
	delete[] res;
}

void SocketStream::writeInt(int i)
{	
	int *pointI = &i;
	char *res = new char[4];
	// si LITTLE ENDIAN, on inverse l ecriture pour etre compatible a JAVA
	if(Endian::endian() == L_ENDIAN){
		for(int k = 0; k < 4; k++){
			res[k] =  ((char *)pointI)[3-k];
		}
		write(res,4);
	} else{
		write((char *)pointI,4);
	}
	delete[] res;
}

void SocketStream::writeLong(long long l){
	long long *pointL = &l;
	char *res = new char[8];
	// si LITTLE ENDIAN, on inverse l ecriture pour etre compatible a JAVA
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 8; i++){
			res[i] =  ((char *)pointL)[7-i];
		}
		write(res,8);
	} else{
		write((char *)pointL,8);
	}
	delete[] res;
}


void SocketStream::writeFloat(float f)
{
	float *pointF = &f;
	char *res = new char[4];
	// si LITTLE ENDIAN, on inverse l ecriture pour etre compatible a JAVA
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 4; i++){
			res[i] =  ((char *)pointF)[3-i];
		}
		write(res,4);
	} else{
		write((char *)pointF,4);
	}
	delete[] res;
}

// recupere le binaire d un double et l ecrit dans le flux
void SocketStream::writeDouble(double d)
{
	double *pointD = &d;
	char *res = new char[8];
	// si LITTLE ENDIAN, on inverse l ecriture pour etre compatible a JAVA
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 8; i++){
			res[i] =  ((char *)pointD)[7-i];
		}
		write(res,8);
	} else{
		write((char *)pointD,8);
	}
	delete[] res;
}

void SocketStream::writeUTF(std::string s)
{
	// conversion de la string en tableau de char
	const char* chaine = s.c_str(); 
	int c, utflen = 0;
	// calcul du nombre de char necessaire
	for (size_t i = 0; i < s.size(); i++) {
		c = chaine[i];
		if ((c >= 0x0001) && (c <= 0x007F)) {
			utflen++;
		} else if (c > 0x07FF) {
			utflen += 3;
		} else {
			utflen += 2;
		}
	}
	if (utflen > 65535){
		std::cout << "Encodage trop long";
	}
	// creation du tableau qui va etre passe a la methode write
	char* res = new char[utflen +2];
	// deux premieres cases pour la taille 
	res[0] = (utflen >> 8) & 0xFF;
	res[1] = (utflen >> 0) & 0xFF;
	
	for (size_t i = 0; i < s.size(); i++) {
		c = chaine[i];
		if ((c >= 0x0001) && (c <= 0x007F)) {
			res[i+2] = c;
		} else if (c > 0x07FF) {
			res[i+2] =  0xE0 | ((c >> 12) & 0x0F);
			res[i+3] = 0x80 | ((c >>  6) & 0x3F);
			res[i+4] = 0x80 | ((c >>  0) & 0x3F);
			i = i+2;
		} else {
			res[i] = 0xC0 | ((c >>  6) & 0x1F);
			res[i+1] = 0x80 | ((c >>  0) & 0x3F);
			i++;
		}
	}
	write(res, utflen+2);
	delete[] res;
}

								/**********  INPUT METHODS ***********/

bool SocketStream::readBool() const
{
	int a = peek();
	return (a == 1);
}

// cast to int for display
char SocketStream::readByte() const
{
	int a = peek();
	return a;
}

short int SocketStream::readShort() const
{
	char *tab = new char[2];
	short int res;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 2; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[1-i] = peek();
		}
	} else{
		for(int i = 0; i < 2; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	res = *((short int*) tab);
	delete[] tab;
	return res;
}

unsigned short SocketStream::readUnsignedShort() const
{
	char *tab = new char[2];
	unsigned short res;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 2; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[1-i] = peek();
		}
	} else{
		for(int i = 0; i < 2; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	res = *((short*) tab);
	delete[] tab;
	return res;
}

int SocketStream::readInt() const
{
	char *tab = new char[4];
	int res;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 4; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[3-i] = peek();
		}
	} else{
		for(int i = 0; i < 4; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	res = *((int*) tab);
	delete[] tab;
	return res;
}


 // long data type in java: from -922337000 to 922337000
long long SocketStream::readLong() const
{
	char *tab = new char[8];
	long long l;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 8; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[7-i] = peek();
		}
	} else{
		for(int i = 0; i < 8; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	l = *((long long*) tab);
	delete[] tab;
	return l;
}

float SocketStream::readFloat() const
{	
	char *tab = new char[4];
	float f;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 4; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[3-i] = peek();
		}
	} else{
		for(int i = 0; i < 4; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	f = *((float*) tab);
	delete[] tab;
	return f;
	
}

// pour cette fonction, on a juste a caster en double * pour que le compilo lise les 8 octets ecrits
double SocketStream::readDouble() const
{
	char *tab = new char[8];
	double d;
	if(Endian::endian() == L_ENDIAN){
		for(int i = 0; i < 8; i++){
			// on inverse l'ordre pour le LITTLE ENDAN
			tab[7-i] = peek();
		}
	} else{
		for(int i = 0; i < 8; i++){
			// on conserve l'ordre pour le BIG ENDAN
			tab[i] = peek();	
		}
	}
	d = *((double*) tab);
	delete[] tab;
	return d;
	
}

std::string SocketStream::readUTF() const
{ 
	// recupere la taille de la string encodee
	int utflen = readUnsignedShort();
	// tableau pour recuperer les donnees de la string
	char *byteArray = new char[utflen];
	// tableau pour restituer la string
	char *charArray = new char[utflen];
	// recuperation des donnees
	read(byteArray,utflen);
	
	int c, char2, char3;
	int count = 0;
	int chararr_count=0;
	// optimisation dans les cas majortaires des char simples
	while (count < utflen) {
		c = (int) byteArray[count] & 0xff;      
		if (c > 127) break;
		count++;
		charArray[chararr_count++] = (char)c;
	}
	// pour le cas ou l'on trouve des char speciaux
	while (count < utflen) {
		c = (int) byteArray[count] & 0xff;
		switch (c >> 4) {
			case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7:
				// 0xxxxxxx
				count++;
				charArray[chararr_count++]=(char)c;
				break;
			case 12: case 13:
				// 110x xxxx   10xx xxxx
				count += 2;
				//if (count > utflen) std::cout << "malformed input: partial character at end"<< std::endl;
				char2 = (int) byteArray[count-1];
				//if ((char2 & 0xC0) != 0x80) std::cout << "malformed input around byte "<< count << std::endl;
				charArray[chararr_count++]=(char)(((c & 0x1F) << 6) | (char2 & 0x3F));  
				break;
			case 14:
				// 1110 xxxx  10xx xxxx  10xx xxxx 
				count += 3;
				if (count > utflen) std::cout << "malformed input: partial character at end"<< std::endl;
				char2 = (int) byteArray[count-2];
				char3 = (int) byteArray[count-1];
				if (((char2 & 0xC0) != 0x80) || ((char3 & 0xC0) != 0x80)) std::cout << "malformed input around byte " << (count - 1)<< std::endl;
				charArray[chararr_count++]=(char)(((c     & 0x0F) << 12) |((char2 & 0x3F) << 6)  |((char3 & 0x3F) << 0));
				break;
			default:
				// 10xx xxxx,  1111 xxxx 
				//throw SocketException("malformed input, not UTF format\n" );
				throw SocketException("");
		}
	}
	std::string str(charArray,chararr_count);
	delete[] byteArray;
	delete[] charArray;
	return str;
}
