package tests;

import junit.framework.*;
import basicTypes1.client.*;
import basicTypes1.server.*;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "hashtable" client and "hashtable" server
*/

public class TestBasicTypes1 extends TestCase
{
    public void testClientServerBasicTypesExchange () throws Exception
    {
        Map<String, Object> map = makeBasicTypeMap();
        Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
        hashtable.put("0", map);

        Hashtable serverDecodedMap = transmitHashtable(hashtable);
                
        assertTrue(serverDecodedMap.equals(hashtable));
    }

    public void testClientServerBasicTypesListsExchange() throws Exception
    {
        Map<String, Object> map = makeBasicTypesListsMap();
        Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
        hashtable.put("0", map);

        Hashtable serverDecodedMap = transmitHashtable(hashtable);
        
        compareBasicTypesListsMap(hashtable, serverDecodedMap);
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            basicTypes1.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new basicTypes1.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }



    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    static private Map<String, Object> makeBasicTypeMap ()
    {
        Map<String, Object> map = new Hashtable<String,Object>();

	    map.put("type",new Byte(basicTypes1.client.CustomTypes.TYPE_BASICTYPES));

	    map.put("aByte",new Byte((byte)1));
	    map.put("aShort",new Short((short)2));
	    map.put("aInt",new Integer(3));
	    map.put("aLong",new Long(4));
	    map.put("aDouble",new Double(5));
	    map.put("aFloat",new Float(6));
	    map.put("aBoolean",new Boolean(true));
	    map.put("aChar",new Character('a'));
	    map.put("aString",new String("hello, world!"));

        return map;
    }

    static private Map<String, Object> makeBasicTypesListsMap()
    {
        Map<String, Object> map = new Hashtable<String,Object>();

	    map.put("type",new Byte(basicTypes1.client.CustomTypes.TYPE_BASICTYPESLISTS));

    	map.put("aByteList",new byte[]{(byte)1,(byte)2,(byte)3});
	    map.put("aShortList",new short[]{(short)1,(short)2,(short)3});
	    map.put("aIntList",new int[]{1,2,3});
	    map.put("aFloatList",new float[]{1f,2f,3f});
	    map.put("aLongList",new long[]{1l,2l,3l});
	    map.put("aDoubleList",new double[]{1d,2d,3d});
	    map.put("aBooleanList",new boolean[]{true, false, true});
	    map.put("aCharList",new char[]{'a','B','?'});
	    map.put("aStringList",new String[]{"hello",", world!"});

        return map;
    }
    
    static private void compareBasicTypesListsMap (Map<String, Object> map1, Map<String, Object> map2)
    {
        assertTrue(Arrays.equals((Object[])map1.get("aByteList"), (Object[])map2.get("aByteList")));
        assertTrue(Arrays.equals((Object[])map1.get("aShortList"), (Object[])map2.get("aShortList")));
        assertTrue(Arrays.equals((Object[])map1.get("aIntList"),(Object[]) map2.get("aIntList")));
        assertTrue(Arrays.equals((Object[])map1.get("aLongList"), (Object[])map2.get("aLongList")));
        assertTrue(Arrays.equals((Object[])map1.get("aDoubleList"), (Object[])map2.get("aDoubleList")));
        assertTrue(Arrays.equals((Object[])map1.get("aFloatList"), (Object[])map2.get("aFloatList")));
        assertTrue(Arrays.equals((Object[])map1.get("aBooleanList"), (Object[])map2.get("aBooleanList")));
        assertTrue(Arrays.equals((Object[])map1.get("aCharList"), (Object[])map2.get("aCharList")));
        assertTrue(Arrays.equals((Object[])map1.get("aStringList"), (Object[])map2.get("aStringList")));
    }



}
