package tests;

import junit.framework.*;
import basicTypes2.client.*;
import basicTypes2.server.*;
import basicTypes2.BasicTypes;
import basicTypes2.BasicTypesLists;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "objects" client and "objects" server
*/

public class TestBasicTypes2 extends TestCase
{
    public void testClientServerBasicTypesExchange () throws Exception
    {
        BasicTypes basictypes = makeBasicTypes();
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();

        clientMap.put("0",basictypes);
        Hashtable serverDecodedMap = transmitHashtable(clientMap);
                
        compareBasicTypes(basictypes, (BasicTypes) serverDecodedMap.get("0"));
    }

    public void testClientServerBasicTypesListsExchange () throws Exception
    {
        BasicTypesLists basictypeslists = makeBasicTypesLists();
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();
        
        clientMap.put("0", basictypeslists);
        Hashtable serverDecodedMap = transmitHashtable(clientMap);
        
        compareBasicTypesLists(basictypeslists, (BasicTypesLists) serverDecodedMap.get("0"));
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            basicTypes2.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new basicTypes2.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }



    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    static private BasicTypes makeBasicTypes()
    {
        BasicTypes basictypes = new BasicTypes();
	    
	    basictypes.setAByte((byte)1);
	    basictypes.setAShort((short)2);
	    basictypes.setAInt(3);
	    basictypes.setALong(4l);
	    basictypes.setADouble(5d);
	    basictypes.setAFloat(6f);
	    basictypes.setABoolean(true);
	    basictypes.setAChar('a');
	    basictypes.setAString(new String("hello, world!"));

        return basictypes;
    }

    static private BasicTypesLists makeBasicTypesLists()
    {
        BasicTypesLists basictypeslists = new BasicTypesLists();

    	basictypeslists.setAByteList(new byte[]{(byte)1,(byte)2,(byte)3});
	    basictypeslists.setAShortList(new short[]{(short)1,(short)2,(short)3});
	    basictypeslists.setAIntList(new int[]{1,2,3});
	    basictypeslists.setAFloatList(new float[]{1f,2f,3f});
	    basictypeslists.setALongList(new long[]{1l,2l,3l});
	    basictypeslists.setADoubleList(new double[]{1d,2d,3d});
	    basictypeslists.setABooleanList(new boolean[]{true, false, true});
	    basictypeslists.setACharList(new char[]{'a','B','?'});
	    basictypeslists.setAStringList(new String[]{"hello",", world!"});

        return basictypeslists;
    }

    static private void compareBasicTypes(BasicTypes basictypes1, BasicTypes basictypes2)
    {
        assertEquals(basictypes1.getAByte(), basictypes2.getAByte());
	    assertEquals(basictypes1.getAShort(), basictypes2.getAShort());
	    assertEquals(basictypes1.getAInt(), basictypes2.getAInt());
	    assertEquals(basictypes1.getALong(), basictypes2.getALong());
	    assertEquals(basictypes1.getADouble(), basictypes2.getADouble());
	    assertEquals(basictypes1.getABoolean(), basictypes2.getABoolean());
	    assertEquals(basictypes1.getAChar(), basictypes2.getAChar());
	    assertEquals(basictypes1.getAString(), basictypes2.getAString());
    }    

    static private void compareBasicTypesLists(BasicTypesLists basicTypesLists1, BasicTypesLists basicTypesLists2)
    {
        assertTrue(Arrays.equals(basicTypesLists1.getAByteList(), basicTypesLists2.getAByteList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAShortList(), basicTypesLists2.getAShortList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAIntList(), basicTypesLists2.getAIntList()));
        assertTrue(Arrays.equals(basicTypesLists1.getALongList(), basicTypesLists2.getALongList()));
        assertTrue(Arrays.equals(basicTypesLists1.getADoubleList(), basicTypesLists2.getADoubleList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAFloatList(), basicTypesLists2.getAFloatList()));
        assertTrue(Arrays.equals(basicTypesLists1.getABooleanList(), basicTypesLists2.getABooleanList()));
        assertTrue(Arrays.equals(basicTypesLists1.getACharList(), basicTypesLists2.getACharList()));
        assertTrue(Arrays.equals(basicTypesLists1.getAStringList(), basicTypesLists2.getAStringList()));
        
    }


}
