package tests;

import junit.framework.*;
import basicTypes3.client.*;
import basicTypes3.server.*;
import basicTypes3.*;
import java.util.*;
import java.io.*;


/*
tests the exchange between a "hashtable" client and "object" server
*/

public class TestBasicTypes3 extends TestCase{

	public void testClientServerBasicTypesExchange() throws Exception
	{
		Map<String, Object> basicTypes = makeBasicTypesMap();
		Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
		hashtable.put("0", basicTypes);

		Hashtable serverDecodedMap = transmitHashtable(hashtable);

		compareBasicTypes((BasicTypes) serverDecodedMap.get("0"), basicTypes);
	} 
	
	public void testClientServerBasicTypesListsExchange() throws Exception
	{
		Map<String, Object> basicTypesLists = makeBasicTypesListsMap();
		Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
		hashtable.put("0", basicTypesLists);

		Hashtable serverDecodedMap = transmitHashtable(hashtable);

		compareBasicTypesLists((BasicTypesLists)serverDecodedMap.get("0"), basicTypesLists);
	}

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            basicTypes3.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new basicTypes3.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }
	
	 //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    static private Map<String, Object> makeBasicTypesMap ()
    {
        Map<String, Object> map = new Hashtable<String,Object>();

	    map.put("type",new Byte(basicTypes3.client.CustomTypes.TYPE_BASICTYPES));

	    map.put("aByte",new Byte((byte)1));
	    map.put("aShort",new Short((short)2));
	    map.put("aInt",new Integer(3));
	    map.put("aLong",new Long(4));
	    map.put("aDouble",new Double(5));
	    map.put("aFloat",new Float(6));
	    map.put("aBoolean",new Boolean(true));
	    map.put("aChar",new Character('a'));
	    map.put("aString",new String("hello, world!"));

        return map;
    }
    
    static private Map<String, Object> makeBasicTypesListsMap()
    {
        Map<String, Object> map = new Hashtable<String,Object>();

	    map.put("type",new Byte(basicTypes3.client.CustomTypes.TYPE_BASICTYPESLISTS));

    	map.put("aByteList",new byte[]{(byte)1,(byte)2,(byte)3});
	    map.put("aShortList",new short[]{(short)1,(short)2,(short)3});
	    map.put("aIntList",new int[]{1,2,3});
	    map.put("aFloatList",new float[]{1f,2f,3f});
	    map.put("aLongList",new long[]{1l,2l,3l});
	    map.put("aDoubleList",new double[]{1d,2d,3d});
	    map.put("aBooleanList",new boolean[]{true, false, true});
	    map.put("aCharList",new char[]{'a','B','?'});
	    map.put("aStringList",new String[]{"hello",", world!"});

        return map;
    }
    
    static private void compareBasicTypes(BasicTypes basictypes1, Map map)
    {
    	assertEquals(basictypes1.getAByte(), map.get("aByte"));
	    assertEquals(basictypes1.getAShort(),map.get("aShort"));
	    assertEquals(basictypes1.getAInt(), map.get("aInt"));
	    assertEquals(basictypes1.getALong(), map.get("aLong"));
	    assertEquals(basictypes1.getADouble(), map.get("aDouble"));
	    assertEquals(basictypes1.getABoolean(), map.get("aBoolean"));
	    assertEquals(basictypes1.getAChar(), map.get("aChar"));
	    assertEquals(basictypes1.getAString(), map.get("aString"));
    }    

    static private void compareBasicTypesLists (BasicTypesLists basictypeslists, Map<String, Object> map)
    {
        assertTrue(Arrays.equals(basictypeslists.getAByteList(), (byte[])map.get("aByteList")));
        assertTrue(Arrays.equals(basictypeslists.getAShortList(), (short[])map.get("aShortList")));
        assertTrue(Arrays.equals(basictypeslists.getAIntList(),(int[]) map.get("aIntList")));
        assertTrue(Arrays.equals(basictypeslists.getALongList(), (long[])map.get("aLongList")));
        assertTrue(Arrays.equals(basictypeslists.getADoubleList(), (double[])map.get("aDoubleList")));
        assertTrue(Arrays.equals(basictypeslists.getAFloatList(), (float[])map.get("aFloatList")));
        assertTrue(Arrays.equals(basictypeslists.getABooleanList(), (boolean[])map.get("aBooleanList")));
        assertTrue(Arrays.equals(basictypeslists.getACharList(), (char[])map.get("aCharList")));
        assertTrue(Arrays.equals(basictypeslists.getAStringList(), (String[])map.get("aStringList")));
        
    }

}
