package tests;

import junit.framework.*;
import basicTypes4.client.*;
import basicTypes4.server.*;
import basicTypes4.*;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "objects" client and "hashtable" server
*/

public class TestBasicTypes4 extends TestCase
{
    public void testClientServerBasicTypesExchange () throws Exception
    {
        BasicTypes basictypes = makeBasicTypes();
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();

        clientMap.put("0",basictypes);
        Hashtable serverDecodedMap = transmitHashtable(clientMap);
                
        compareBasicTypes(basictypes, (Hashtable) serverDecodedMap.get("0"));
    }

    public void testClientServerBasicTypesListsExchange () throws Exception
    {
        BasicTypesLists basictypeslists = makeBasicTypesLists();
        Hashtable<String, Object> clientMap = new Hashtable<String, Object>();
        
        clientMap.put("0", basictypeslists);
        Hashtable serverDecodedMap = transmitHashtable(clientMap);
        
        compareBasicTypesLists(basictypeslists, (Hashtable) serverDecodedMap.get("0"));
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            basicTypes4.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new basicTypes4.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }

    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    static private BasicTypes makeBasicTypes()
    {
        BasicTypes basictypes = new BasicTypes();
	    
	    basictypes.setAByte((byte)1);
	    basictypes.setAShort((short)2);
	    basictypes.setAInt(3);
	    basictypes.setALong(4l);
	    basictypes.setADouble(5d);
	    basictypes.setAFloat(6f);
	    basictypes.setABoolean(true);
	    basictypes.setAChar('a');
	    basictypes.setAString(new String("hello, world!"));

        return basictypes;
    }

    static private BasicTypesLists makeBasicTypesLists()
    {
        BasicTypesLists basictypeslists = new BasicTypesLists();

    	basictypeslists.setAByteList(new byte[]{(byte)1,(byte)2,(byte)3});
	    basictypeslists.setAShortList(new short[]{(short)1,(short)2,(short)3});
	    basictypeslists.setAIntList(new int[]{1,2,3});
	    basictypeslists.setAFloatList(new float[]{1f,2f,3f});
	    basictypeslists.setALongList(new long[]{1l,2l,3l});
	    basictypeslists.setADoubleList(new double[]{1d,2d,3d});
	    basictypeslists.setABooleanList(new boolean[]{true, false, true});
	    basictypeslists.setACharList(new char[]{'a','B','?'});
	    basictypeslists.setAStringList(new String[]{"hello",", world!"});

        return basictypeslists;
    }

    static private void compareBasicTypes(BasicTypes basictypes1, Map map)
    {
        assertEquals(basictypes1.getAByte(), map.get("aByte"));
	    assertEquals(basictypes1.getAShort(), map.get("aShort"));
	    assertEquals(basictypes1.getAInt(), map.get("aInt"));
	    assertEquals(basictypes1.getALong(), map.get("aLong"));
	    assertEquals(basictypes1.getADouble(), map.get("aDouble"));
	    assertEquals(basictypes1.getABoolean(), map.get("aBoolean"));
	    assertEquals(basictypes1.getAChar(), map.get("aChar"));
	    assertEquals(basictypes1.getAString(), map.get("aString"));
    }    

    static private void compareBasicTypesLists (BasicTypesLists basictypeslists, Map<String, Object> map)
    {
        assertTrue(Arrays.equals(basictypeslists.getAByteList(), (byte[])map.get("aByteList")));
        assertTrue(Arrays.equals(basictypeslists.getAShortList(), (short[])map.get("aShortList")));
        assertTrue(Arrays.equals(basictypeslists.getAIntList(),(int[]) map.get("aIntList")));
        assertTrue(Arrays.equals(basictypeslists.getALongList(), (long[])map.get("aLongList")));
        assertTrue(Arrays.equals(basictypeslists.getADoubleList(), (double[])map.get("aDoubleList")));
        assertTrue(Arrays.equals(basictypeslists.getABooleanList(), (boolean[])map.get("aBooleanList")));
        assertTrue(Arrays.equals(basictypeslists.getAStringList(), (String[])map.get("aStringList")));
        assertTrue(Arrays.equals(basictypeslists.getACharList(), (char[])map.get("aCharList")));
        assertTrue(Arrays.equals(basictypeslists.getAFloatList(), (float[])map.get("aFloatList")));
    }


}
