package tests;

import junit.framework.*;
import objectTypes1.client.*;
import objectTypes1.server.*;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "hashtable" client and "hashtable" server
*/

public class TestObjectTypes1 extends TestCase
{
    public void testClientServerObjectExchange () throws Exception
    {
        Hashtable objectA = makeObjectAHashtable();
        Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
        hashtable.put("0", objectA);

        Hashtable serverDecodedMap = transmitHashtable(hashtable);
                
        assertTrue(serverDecodedMap.equals(hashtable));
    }

    public void testClientServerObjectTypesListsExchange() throws Exception
    {
        Hashtable objectB = makeObjectBHashtable();
        Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
        hashtable.put("0", objectB);

        Hashtable serverDecodedMap = transmitHashtable(hashtable);
        
        compareObjectsB(objectB, (Hashtable) serverDecodedMap.get("0"));
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            objectTypes1.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new objectTypes1.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }


    private Hashtable makeObjectAHashtable()
    {
        Hashtable h = new Hashtable();
        h.put("type", objectTypes1.client.CustomTypes.TYPE_OBJECTA);
        h.put("objectC", makeObjectCHashtable("toto"));
        
        return h;
    }


    private Hashtable makeObjectBHashtable()
    {
        Hashtable h = new Hashtable();
        h.put("type", objectTypes1.client.CustomTypes.TYPE_OBJECTB); 
        
        Hashtable[] objectCTab = new Hashtable[2];
        objectCTab[0] = makeObjectCHashtable("titi");
        objectCTab[1] = makeObjectCHashtable("toto");
       
        h.put("objectC", objectCTab);

        return h;
    }
    
    private Hashtable makeObjectCHashtable(String chaine)
    {
        Hashtable h = new Hashtable();
        h.put("type", "objectc");
        h.put("aString", chaine);
        return h;
    }

    private void compareObjectsB(Hashtable objectB1, Hashtable objectB2)
    {
        Hashtable[] objectB1_objectC = (Hashtable[]) objectB1.get("objectC");
        Hashtable[] objectB2_objectC = (Hashtable[]) objectB2.get("objectC");

        assertEquals(objectB1_objectC[0], objectB2_objectC[0]);
        assertEquals(objectB1_objectC[1], objectB2_objectC[1]);
    }

}
