package tests;

import junit.framework.*;
import objectTypes3.client.*;
import objectTypes3.server.*;
import objectTypes3.*;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "hashtable" client and "objects" server
*/

public class TestObjectTypes3 extends TestCase
{
    public void testClientServerObjectExchange() throws Exception
    {
            Hashtable objectA = makeObjectAHashtable();
            Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
            hashtable.put("0", objectA);

            Hashtable serverDecodedMap = transmitHashtable(hashtable);
                    
            compareObjectsA((ObjectA) serverDecodedMap.get("0"), objectA);
    }

    public void testClientServerObjectTypesListsExchange() throws Exception
    {
            Hashtable objectB = makeObjectBHashtable();
            Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
            hashtable.put("0", objectB);

            Hashtable serverDecodedMap = transmitHashtable(hashtable);
            
            compareObjectsB((ObjectB) serverDecodedMap.get("0"), objectB);
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            objectTypes3.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new objectTypes3.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }


    private Hashtable makeObjectAHashtable()
    {
        Hashtable h = new Hashtable();
        h.put("type", objectTypes3.client.CustomTypes.TYPE_OBJECTA);
        h.put("objectC", makeObjectCHashtable("toto"));
        
        return h;
    }


    private Hashtable makeObjectBHashtable()
    {
        Hashtable h = new Hashtable();
        h.put("type", objectTypes3.client.CustomTypes.TYPE_OBJECTB); 
        
        Hashtable[] objectCTab = new Hashtable[2];
        objectCTab[0] = makeObjectCHashtable("titi");
        objectCTab[1] = makeObjectCHashtable("toto");
       
        h.put("objectC", objectCTab);

        return h;
    }
    
    private Hashtable makeObjectCHashtable(String chaine)
    {
        Hashtable h = new Hashtable();
        h.put("type", "objectc");
        h.put("aString", chaine);
        return h;
    }


    private void compareObjectsA(ObjectA objectA1, Hashtable objectA2)
    {
    	assertTrue(objectA1.getObjectC()!=null);
    	assertTrue(objectA2.get("objectC")!=null);
    	assertEquals(objectA1.getObjectC().getAString(), ((Hashtable)objectA2.get("objectC")).get("aString"));
    }
    
    private void compareObjectsB(ObjectB objectB1, Hashtable objectB2)
    {
    	
        ObjectC[] objectB1_objectC = objectB1.getObjectC();
        Hashtable[] objectB2_objectC = (Hashtable[]) objectB2.get("objectC");

        assertTrue(objectB1_objectC != null);
        assertTrue(objectB2_objectC != null);
        assertEquals(objectB1_objectC[0].getAString(), objectB2_objectC[0].get("aString"));
        assertEquals(objectB1_objectC[1].getAString(), objectB2_objectC[1].get("aString"));
    }
}
