package tests;

import junit.framework.*;
import objectTypes4.client.*;
import objectTypes4.server.*;
import objectTypes4.*;
import java.util.*;
import java.io.*;

/*
    tests the exchange between a "objects" client and "hashtable" server
*/

public class TestObjectTypes4 extends TestCase
{
    public void testClientServerObjectExchange () throws Exception
    {
            ObjectA objectA = makeObjectA();
            Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
            hashtable.put("0", objectA);

            Hashtable serverDecodedMap = transmitHashtable(hashtable);
                    
            compareObjectsA(objectA, (Hashtable) serverDecodedMap.get("0"));
    }

    public void testClientServerObjectTypesListsExchange () throws Exception
    {
        	ObjectB objectB = makeObjectB();
            Hashtable<String, Object> hashtable = new Hashtable<String, Object>();
            hashtable.put("0", objectB);

            Hashtable serverDecodedMap = transmitHashtable(hashtable);
            
            compareObjectsB(objectB, (Hashtable) serverDecodedMap.get("0"));
    }

    private Hashtable transmitHashtable(Hashtable totheclient) throws Exception
    {
            File tempFile = TempFileManager.makeTempFile();

            // writes the hashtable to the temp file
            FileOutputStream fileOutputStream = new FileOutputStream(tempFile);
            DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
            objectTypes4.client.CustomTypes.encodeData(totheclient, dataOutputStream);
            dataOutputStream.flush();
            dataOutputStream.close();            
            
            // reads the hashtable from the temp file
            DataInputStream dataInputStream = new DataInputStream(new FileInputStream(tempFile));
            Hashtable serverDecodedMap = (new objectTypes4.server.CustomTypes()).decodeData(dataInputStream);
            dataInputStream.close();

            TempFileManager.removeTempFile(tempFile);

            return serverDecodedMap;
    }


    private ObjectA makeObjectA()
    {
        ObjectA objectA = new ObjectA();
        ObjectC objectC = makeObjectC("toto");
        objectA.setObjectC(objectC);
        
        return objectA;
    }


    private ObjectB makeObjectB()
    {
    	ObjectB objectB = new ObjectB();
    	
        ObjectC[] objectCTab = new ObjectC[2];
        objectCTab[0] = makeObjectC("titi");
        objectCTab[1] = makeObjectC("toto");
       
        objectB.setObjectC(objectCTab);

        return objectB;
    }
    
    private ObjectC makeObjectC(String chaine)
    {
    	ObjectC objectC = new ObjectC();
    	objectC.setAString(chaine);
        return objectC;
    }


    private void compareObjectsA(ObjectA objectA1, Hashtable objectA2)
    {
    	assertTrue(objectA1.getObjectC()!=null);
    	assertTrue(objectA2.get("objectC")!=null);
    	assertEquals(objectA1.getObjectC().getAString(), ((Hashtable)objectA2.get("objectC")).get("aString"));
    }
    
    private void compareObjectsB(ObjectB objectB1, Hashtable objectB2)
    {
    	
        ObjectC[] objectB1_objectC = objectB1.getObjectC();
        Hashtable[] objectB2_objectC = (Hashtable[]) objectB2.get("objectC");

        assertTrue(objectB1_objectC != null);
        assertTrue(objectB2_objectC != null);
        assertEquals(objectB1_objectC[0].getAString(), objectB2_objectC[0].get("aString"));
        assertEquals(objectB1_objectC[1].getAString(), objectB2_objectC[1].get("aString"));
    }
}
